# Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:17 PM CET 2014
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

# Note:  This file uses UNIX coding, i.e. lines are separated by LF, not CR-LF.

package require commands

proc Init {} {
    global artifactNumber 
    set artifactNumber -1 

    # contextual generation commands
    std::AddCommand -variable cmdADSimGenSM -name "Create simulation model" -statusmessage "Generate a simulation model from an activity." -tooltip "Generate State Machine for simulation from activity." -accelerator "" -imagefile "" -onactivatecommand OnADSimGenSM -onenablecommand OnADSimGenSMEnable
    std::AddCommand -variable cmdASLaunchSimulation -name "Simulate" -statusmessage "Launch activity simulation." -tooltip "Launch activity simulation" -accelerator "" -imagefile "" -onactivatecommand OnADSimLaunchSimulation -onenablecommand OnADSimLaunchSimulationEnable

    # menu definitions
    std::AddContextMenu -variable cxmADSimGen -commands { separator cmdADSimGenSM cmdASLaunchSimulation } -onenablemenu OnADSimGenCxMEnable

    std::AddMenu -variable mbmADSimGen -commands { cmdADSimGenSM cmdASLaunchSimulation } -path { &ADSim } -position last
}

# =============================================================================
# Auxilary validation routines

proc ValidateActivity {element} {
    # element must be a UML element ("U2") 
    if { ("" == $element) || ("U2" != [GetKind $element]) } {
        # invalid selection
        return 0
    }

    # disable if selectiong is not an activity    
    if { ! ( [u2::IsKindOf $element "Operation"] ) } {
        # invalid selection
        return 0
    }

    set eInlineMethod [u2::GetEntity $element InlineMethod]
    if {$eInlineMethod == "0"} {
        return 0
    }

    if { ! [u2::IsKindOf $eInlineMethod "ActivityImplementation"]} {
        return 0
    }

    return 1
}

proc ValidateSelection {} {
    set selected [std::GetSelection]

    # only single element must be selected
    if { (1 != [llength $selected]) } {
        # invalid selection
        return 0
    }

    set element [lindex $selected 0]

    set res [ValidateActivity $element]

    return $res
}

# End of Auxilary validation routines
# =============================================================================

# =============================================================================
# Routines for "u2" model tab

# The cxmADSimGen only appears in the "u2" model tab.
proc OnADSimGenCxMEnable { cmd } {
    # check that selection is in model view tab ("u2") and also valid
    if { ($cmd != "u2") || ! [ValidateSelection] } {
        #disable activity simulation
        return 0
    }

    # enable menu commands to be shown
    return 1
}

# End of Routines for "u2" model tab
# =============================================================================

# =============================================================================
# Simulation routines.

proc OnADSimLaunchSimulationEnable { cmd } {
    set element [lindex [std::GetSelection] 0]
    set res [ValidateActivity $element]

    return $res
}

proc OnADSimGenSMEnable { cmd } {
    set element [lindex [std::GetSelection] 0]
    set res [ValidateActivity $element]

    return $res
}

proc CallASAgent {activity} {

    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]
    set agent [u2::FindByGuid $model "@ADSim@Generate"]
    if { 0 == $agent } {
        std::Output "Failed to find ADSim agent\n"
        return 0
    }
	
	set location [std::GetProjectPath $curProject]
	if { [file isdirectory $location] == 0 } {
		set pathFormUri [GetDefaultOutputLocation $location]
		if { [file isdirectory $pathFormUri] == 0 } {
			std::Output "Activity Simulation Model can't be created in following location: $pathFormUri\n"
			return 0
		}
		set projectPathDir $pathFormUri
	} else {
		set projectPathDir [join [list [string trimright [file dirname $location]] [file separator]] ""]
	}	
	
    lappend params $projectPathDir

    lappend params 0 
    # return parameter for BuildArtifact

    if { [catch { u2::InvokeAgent $model $agent $activity params } e ] } {
        std::Output "Error when invoking ADSim agent!\n"
        std::Output $e
    }

    return [lindex $params end]
}

proc CallABAgent {BuildArtifact option} {
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]
    set agent [u2::FindByGuid $model "@TTDApplicationBuilder@PerformBuildArtifactOperation"]
    if { 0 == $agent } {
        std::Output "Failed to find PerformBuildArtifactOperation agent.\n"
        # failed to find agent
        return 0
    }

    set projectPathDir [join [list [string trimright [file dirname [std::GetProjectPath $curProject]]] [file separator]] ""]
    lappend params $option
    # build option

    if { [catch { u2::InvokeAgent $model $agent $BuildArtifact params } e ] } {
        std::Output "Error when invoking PerfromBuildArtifactOperation agent!\n"
        std::Output $e
        return 0
    }
    return 1
}

proc OnADSimLaunchSimulation { cmd } {

    set activity [lindex [std::GetSelection] 0]
    set BA [CallASAgent $activity]

    if {$BA == 0} {
        std::Output "Some errors were encountered during the activity simulation."
        return 0
    }

    set res [CallABAgent $BA "bopBuildLaunchArtifact"]

    return $res
}

proc OnADSimGenSM { cmd } {

    set activity [lindex [std::GetSelection] 0]
    set BA [CallASAgent $activity]

    if {$BA == 0} {return 0}

    return 1
}

# End of Simulation routines.
# =============================================================================

set addin "ADSim"

# run
Init
