# Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:17 PM CET 2014
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# Creates an operation in the class typing the destination lifeline.
# If the lifeline is untyped, or the type is unbound, an error is generated.
# The parameters and return value of the operation is calculated by
# the values of the method call line. The reply line is used to calculate 
# the return type.
# 
# If the operation is already bound and the owner is an interface, the 
# operation is copied from the interface to the class.
proc UpdateModelMethodCallLine { messageLine } {
	if ![IsKindOf $messageLine MessageLine] {
		error "Element $messageLine is a '[Class $messageLine]', but should be a 'MethodCallLine'"
	}

	# Setup context and other variables
	set message [u2::GetEntity $messageLine ModelElement]
	set diagram [u2::GetEntity $messageLine Owner]
	set interaction [u2::GetEntity $diagram Owner]
	set operation [u2::GetEntity $interaction Specification]
	if { $operation == 0 } {
		set operation [u2::GetEntity $interaction ImplementedSignature]
	}

	# Get destination and type of destination
	set dst [u2::GetEntity $messageLine Dst]
	if {$dst != 0} {
		set dstType [u2::GetEntity [u2::GetEntity [u2::GetEntity $messageLine Dst] Lifeline] Type]
	}
	if {$dstType == 0} {
		AMReport $messageLine "Error" "Destination has no type, unable to create operation"
		return
	}

        set session [GetSession $messageLine]
        
	set ie [u2::GetEntity $message Data]
	set signal 0
	if {0 != $ie} {
	    set signal [u2::GetEntity $ie InstanceOf]	
	}	

	if { $signal != 0 } {
		set owner [u2::GetEntity $signal Owner]
		if {($dstType != 0) && ($owner != $dstType)} {
			set newOperation [u2::Parse $session [u2::Unparse $signal] -parseAs Definition]
                        u2::SetEntity $dstType OwnedMember $newOperation
			u2::SetValue $newOperation Visibility VkPublic
			AMReport $newOperation "Create" "Created operation '[u2::GetValue $newOperation Name]' in class '[u2::GetValue $dstType Name]'"
		}
	} else {
		global sigName foundSignal
                set sigName "" 
    
                if {0 !=$ie} {
                    set sigName [u2::Unparse $ie]
                    regsub {( *)[(]\..*\.[)]} $sigName "" sigName
                }

# seersa Note that using u2::Create yields strange results in undo, operation name references becomes
# empty ''. Using u2::New works better, but introduces a crash on undo/redo followed by revert file
#		set signal [u2::New $session Operation]
		set signal [u2::Create $dstType Operation]
		if { $sigName != "" } {
			u2::SetValue $signal Name $sigName
		} else {                        
			set ie [u2::Parse $session "[u2::GetValue $signal Name] (..)" -parseAs Expression]
			u2::SetEntity $message Data $ie
		}
#		u2::SetEntity $dstType OwnedMember $signal
		AMReport $signal "Create" "Created operation '[u2::GetValue $signal Name]' in [Class $dstType] '[u2::GetValue $dstType Name]'"
		set paramCount 1	
		foreach expr [u2::GetEntities $ie Expression] {
			if { 0 == $expr } {
			    continue
			}
			set newParam [u2::Create $signal Parameter]
			set paramType [u2::GetEntity $expr Type]
			if { $paramType != 0 } {
				u2::SetValue $newParam Type ref:[u2::GetValue $paramType Name]
			} else {
				u2::SetValue $newParam Type ref:Any
				set variable [u2::Create $interaction Attribute]
				u2::SetValue $variable Type ref:Any
				u2::SetValue $variable Name [u2::GetValue $expr Name]
				AMReport $variable "Create" "Created variable '[u2::GetValue $variable Name]' of type 'Any' in diagram '[u2::GetValue $diagram Name]'"
			}
			u2::SetValue $newParam Name p${paramCount}
			incr paramCount
		}
	}

	# Update the reply message line
	set dstAction [u2::GetEntity $messageLine DstActionOccurrence]
	set activation [u2::GetReferringEntities $dstAction ActionOccurrence]
	if {[llength $activation] > 0} {
		set replyAction [u2::GetEntity [lindex $activation 0] EndActionOccurrence]
		set replyMessageLine [u2::GetReferringEntities $replyAction SrcActionOccurrence]
		if {[llength $replyMessageLine] != 0} {
		    set replyMessage [u2::GetEntity [lindex $replyMessageLine 0] ModelElement]
		    set returnValue [u2::GetEntity $replyMessage ReturnValue]
		    if {$returnValue != 0} {
		        set returnType [u2::GetEntity $returnValue Type]
		        set returnParam [u2::GetEntity $signal Return]
		        if {$returnParam != 0} {
			        set returnParamType [u2::GetEntity $returnParam Type]
			        if {$returnParamType != $returnType} {
				        AMReport $replyMessageLine "Error" "Return type '[u2::GetValue $returnType Name]' different from declaration of '[u2::GetValue $signal Name]': '[u2::GetValue $returnParamType Name]'"
			        }
		        } else {
                                set returnParam [u2::Create $signal Parameter]
			        u2::SetValue $returnParam Direction DkReturn

			        if { 0 != $returnType } {
                                    u2::SetValue $returnParam Type ref:[u2::GetValue $returnType Name]
			        } else {
			            u2::SetValue $returnParam Type ref:Any
			            set variable [u2::Create $interaction Attribute]
			            u2::SetValue $variable Type ref:Any
			            u2::SetValue $variable Name [u2::GetValue $returnValue Name]
			            AMReport $variable "Create" "Created variable '[u2::GetValue $variable Name]' of type 'Any' in diagram '[u2::GetValue $diagram Name]'"
			        }
        			        			
			        AMReport $returnParam "Create" "Created return parameter in operation '[u2::GetValue $signal Name]'"
		        }
		    }
		}
	}

	u2::Bind $message
}


