/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
///////////////////////////////////////////////////////////////////////////////
// torContainers.h
// ---------------
// C++ implementations of UML2 containers such as String
///////////////////////////////////////////////////////////////////////////////

#ifndef __TOR_CONTAINERS__H__
#define __TOR_CONTAINERS__H__

#include "torPlatform.h"

#include "torTypes.h"
#include "torExceptions.h"

#ifndef TOR_STRING
#   include <vector>
#endif

// Type definitions
namespace tor {

#ifndef TOR_STRING
    template <class T >
    class String : public std::vector<T> {
      typedef std::vector<T> inherited;
    public:
      typedef typename inherited::size_type size_type;
      typedef typename inherited::reference reference;
      typedef typename inherited::const_reference const_reference;
      typedef typename inherited::iterator iterator;
      typedef typename inherited::const_iterator const_iterator;
    public:
        // Construction of empty string
        String() 
        { }

	// Construction of string initialized with n initial instances.
	// The element type T must have a default constructor.
        String(Natural n) 
        {
            inherited::reserve(n);
	    for (Natural i = 0; i < n; i++)
		append(T());	    
	}

	static String<T> mkstring(T t) {
	    String<T> s;
	    s.append(t);
	    return s;
	}

	Natural length() { 
	    return static_cast<Natural>(inherited::size());
	}

	static Natural length(String<T>& s) {
	    return s.length();
	}

	T first() throw (EOutOfRange) { 
	    try {
                return inherited::at(0); // operator[] is faster, but without range checks!
	    }
	    catch (std::out_of_range){
		throw EOutOfRange();
	    }
	}

	T last() throw (EOutOfRange) {
	    try {
		return inherited::at(length() - 1); // operator[] is faster, but without range checks!
	    }
	    catch (std::out_of_range){
		throw EOutOfRange(); 
	    }
	}

	void append(T t) { 
	    inherited::push_back(t); 
	}  

	static void append(String<T>& s, T c) { 
	    s.append(c); 
	}

	void insert(T t, Natural i) {
            iterator it = (inherited::begin() + (i - 1) /* indexOffset */);
	    inherited::insert(it, t);
	}

	static void insert(String<T>& s, T t, Natural i) {
	    s.insert(t, i);
	}


	static String<T> substring(String<T>& s, Natural from, Natural to) {
	    String<T> substring;
	    for (Natural i = from - 1 /* indexOffset */; i <= to - 1 /* indexOffset */; i++)
		substring.push_back(s.at(i));

	    return substring;
	}

	void remove(Natural index) {
	    inherited::erase(inherited::begin() + index - 1 /* indexOffset */, inherited::begin() + index - 1 /* indexOffset */ + 1);
	}
   
	static void remove(String<T>& s, Natural index) {
	    s.remove(index);
	}

	void erase() {
	    inherited::clear();
	}

	static void erase(String<T>& s) {
	    s.erase();
	}

	String<T>& operator+(const String<T>& s) { 
	    for (const_iterator it = s.begin(); it != s.end(); it++)
		inherited::push_back(*it);
	    return *this;
	}

	const_reference operator[](Natural n) const {
	    return inherited::operator [](n - 1 /* indexOffset */);
	}

	reference operator[](Natural n) {
	    return inherited::operator [](n - 1 /* indexOffset */);
	}

        // Delete all elements in the string. Do not call this function on strings
        // with value elements!
        void deleteAll() {
            for (const_iterator it = inherited::begin(); it != inherited::end(); it++)
	        delete *it;
        }
    };

    // Initialize the String s with n instances of type T.
    template <class T> void initString(String<T*>& s, Natural n) 
    {
        s.reserve(n);
        for (Natural i = 0; i < n; i++)
            s.append(new T);
    }

    // Deleter class used for implementing part semantics for an attribute
    // with multiple multiplicity.
    template <class T> struct MultiDeleter {
        MultiDeleter(String<T*>& s) : m_s(s) {}

        ~MultiDeleter() {
            m_s.deleteAll();
        }

        String<T*>& m_s;
    };


#endif

}

#endif // __TOR_CONTAINERS__H__

