/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torDispatchableClass_h__
#define __torDispatchableClass_h__ (1)

#include "torPlatform.h"

#include "torDispatchable.h"
#include <list>

#include "torMetaData.h"

namespace tor {
  
    class Event;
    class StateMachine;
    class Port;  

    class TOR_EXT_DEF DispatchableClass : public Dispatchable 
    {
    public:
        virtual ~DispatchableClass();
    
        // Attempts to dispatch an event on this dispatchable class.
        // Returns true if the event was consumed, false otherwise.
        // N.B. If the event is consumed, it will also be deleted. 
        // If it is not consumed it is the caller's responsibility 
        // to delete it (or do whatever it likes to do with it).
        virtual bool receive(Event* event);

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
        // Instrumented version of the above
        virtual bool receive(Event* event, const Entity& receiver, meta::Type* eventMetaType);
#endif

        // Initializes this instance to make it ready for execution.
        // This typically involves creating the statemachine of the class 
        // and initialize it, and also create and initialze and architecture 
        // of the class.
        virtual void init();
    
        virtual void finish();
    
        // Starts this instance and its statemachine, making it receptive to
        // events. The instance must be initialized before it is started.
        virtual void start();
    
        // Executes the event in the statemachine of this class.
        // Returns the status to indicate how event was handled. 
        Dispatchable::EventAction execute(Event* e);
    
        // Returns the (statemachine) behavior of this class.
        virtual StateMachine *getClassifierBehavior()
        {
            return 0;
        }

        // Returns this dispatchable class.
        DispatchableClass* getDispatchableClass() 
        {
            return this;
        }

        // Adds the given port as an out port of this class.
        virtual void addOutPort(Port *port)
        {
          m_listOutPorts.push_back(port);
        }
    
        // Disconnects the architecture.
        virtual void disconnect();

        // Sends an event via a port.
        // Returns true if the event was consumed by the receiver, false otherwise.
        // If the event is not consumed it will be deleted.
        virtual bool send(Event *e);

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
        // Instrumented version of the above
        virtual bool send(Event *e, const Entity& sender, meta::Type* eventMetaType);
#endif

    protected:
        std::list<Port *> m_listOutPorts;
    };
  
    // Inits the 'n' first dispatchable classes of the string. If 'n' is negative
    // all dispatchable classes in the string will be inited.
    template <class DC>
    inline void init(String<DC> &ds, int n = -1)
    {
        int size = static_cast<int>(ds.size());
        unsigned int nMax = (n < 0 || n > size) ? size : n;
        for (unsigned int i = 1 /* indexOffset */; i < nMax + 1 /* indexOffset */; ++i)
        {
            if (ds[i])
            ds[i]->init();
        }
    }

    // Starts the 'n' first dispatchable classes of the string. If 'n' is negative
    // all dispatchable classes in the string will be started.
    template <class DC>
    inline void start(String<DC> &ds, int n = -1)
    {
        int size = static_cast<int>(ds.size());
        unsigned int nMax = (n < 0 || n > size) ? size : n;
        for (unsigned int i = 1 /* indexOffset */; i < nMax + 1 /* indexOffset */; ++i)
        {
            if (ds[i])
            ds[i]->start();
        }
    }
  
} // tor namespace

#endif  // __torDispatchableClass_h__

