// Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:21 PM CET 2014
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torDispatcherThread.h"

namespace
{
    class QuitEvent : public tor::Event {};
}

namespace tor
{
    //
    // DispacherData
    //

    DispatcherData::DispatcherData()
      : m_dispatcher(),
        m_queue(),
        m_mutex(),
        m_threadShallEnd(false),
        m_threadDone(),
        m_useSemaphore(true),
        m_behaviourId(0)
    {
        m_mutex.init();
        m_threadDone.init(0);
        m_dispatcher.setEventQueue(&m_queue);
    }

    void DispatcherData::stepDispatcher()
    {
        m_dispatcher.step();
    }

    bool DispatcherData::endThread()
    {
        postEndThread();
        if (m_behaviourId == Thread::id())
        {
            os::auto_lock lock(m_mutex);
            m_useSemaphore = false;
            return true;
        }

        return m_threadDone.get() == os::StatusOK;
    }

    bool DispatcherData::endThread(os::Time timeout)
    {
        postEndThread();
        if (m_behaviourId == os::Thread::id())
        {
            os::auto_lock lock(m_mutex);
            m_useSemaphore = false;
            return true;
        }

        return m_threadDone.get(timeout) == os::StatusOK;
    }

    void DispatcherData::postEndThread()
    {
        {
            os::auto_lock lock(m_mutex);
            m_threadShallEnd = true;
        }

        // Insert an event into the underlying event queue to
        // wake the dispatcher thread if it is blocking.
        m_queue.insert(new QuitEvent);
    }

    bool DispatcherData::threadShallEnd()
    {
        os::auto_lock lock(m_mutex);
        return m_threadShallEnd;
    }

    void DispatcherData::threadIsDone()
    {
        {
            os::auto_lock lock(m_mutex);
            if (!m_useSemaphore)
                return;
        }
        m_threadDone.put();
    }

    void DispatcherData::setBehaviourId()
    {
        os::auto_lock lock(m_mutex);
        m_behaviourId = os::Thread::id();
    }

    void DispatcherData::addDispatchable(Dispatchable *d)
    {
        m_dispatcher.add(d);
    }

    void DispatcherData::removeDispatchable(Dispatchable *d)
    {
        m_dispatcher.remove(d);
    }

    //
    // DispatcherBehaviour
    //

    DispatcherBehaviour::DispatcherBehaviour(os::Thread::Data<DispatcherData> &data) : m_data(data)
    {
    }

    int DispatcherBehaviour::execute()
    {
        m_data->setBehaviourId();
        while (!m_data->threadShallEnd())
            m_data->stepDispatcher();
        m_data->threadIsDone();
        return 0;
    }

    //
    //
    //

    ThreadedDispatcher::ThreadedDispatcher()
      : m_data()
    {}

    ThreadedDispatcher::~ThreadedDispatcher()
    {
        endThread();
    }

    void ThreadedDispatcher::launch()
    {
        os::Thread::launch(new DispatcherBehaviour(m_data));
    }

    bool ThreadedDispatcher::endThread()
    {
        return m_data->endThread();
    }

    bool ThreadedDispatcher::endThread(os::Time timeout)
    {
        return m_data->endThread(timeout);
    }

    void ThreadedDispatcher::add(Dispatchable *d)
    {
        m_data->addDispatchable(d);
    }

    void ThreadedDispatcher::remove(Dispatchable *d)
    {
        m_data->removeDispatchable(d);
    }

}

