// Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:21 PM CET 2014
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#ifdef TOR_USE_INSTRUMENTATION

#include "torPlatform.h"

#include "torOs.h"
#include "torEventMedia.h"
#include "torEventManager.h"
#include "torMetaData.h"
#include "torCommunication.h"
#include <assert.h>
#include <fstream>

using namespace std;

#if _MSC_VER
#include <WinSock2.h>
#else
#include <netdb.h>
#include <arpa/inet.h>
#include <sys/ioctl.h>

#define closesocket     close
#define ioctlsocket     ioctl
#define IOCPARM_MASK    0x7f /* parameters must be < 128 bytes */
#define IOC_IN          0x80000000 /* copy in parameters */
#define _IOW(x,y,t)     (IOC_IN|(((long)sizeof(t)&IOCPARM_MASK)<<16)|((x)<<8)|(y))
#define FIONBIO         _IOW('f', 126, u_long) /* set/clear non-blocking i/o */
#define INADDR_NONE     -1
#define SOCKET_ERROR    -1
#endif

namespace tor 
{
    using namespace os;

    namespace meta 
    {
        //
        // EventMedia
        //

        EventMedia::EventMedia(bool registerMedia) 
        {
#ifndef TOR_NO_EVENTMANAGER
            if (registerMedia)
                tor::meta::EventManager::registerMedia(this);
#endif
        }

        //
        //
        //

        EventMediaImpl::EventMediaImpl(EventMedia *impl, bool registerMedia) 
          : EventMedia(registerMedia),
            m_impl(impl)
        {}

        //
        // LogFile
        //

        class LogFileImpl : public EventMedia
        {
        public:
            LogFileImpl(const Charstring &strFileName) 
              : EventMedia(false), 
                m_stream()
            {
                m_stream.open(strFileName.getNarrowString().c_str());
            }

            void emit(const Charstring &str) 
            {
                m_stream << str.getNarrowString(); // N.B. This operator<< is in std namespace!
            }

            void flush()
            {
                m_stream.flush();
            }

        private:
            ofstream m_stream;
        };

        LogFile::LogFile(const Charstring &strFileName, bool registerMedia)
          : EventMediaImpl(new LogFileImpl(strFileName), registerMedia)
        {}

        void LogFile::flush()
        {
            assert(m_impl != 0);
            static_cast<LogFileImpl *>(m_impl)->flush();
        }

        namespace TraceServer
        {
            struct Message
            {
                unsigned char check1;
                unsigned char kind;
                unsigned char status;
                unsigned char length;
                unsigned char check2;
                unsigned char data[255];
            };
            
            struct Reply
            {
                unsigned char check1;
                unsigned char reply;
                unsigned char check2;
            };

            const unsigned char CheckValue1 = char(0xa5);
            const unsigned char CheckValue2 = char(0xff);

            const unsigned char QuitMessage       = 1;
            const unsigned char LogonMessage      = 5;
            const unsigned char TraceEventMessage = 2;
            const unsigned char ConnectMessage    = 3;
            const unsigned char DisconnectMessage = 4;

            const unsigned char LastPacket = 110;
            const unsigned char OnlyPacket = 110;
            const unsigned char MoreToCome = 111;

            const unsigned char Ok    = 11;
            const unsigned char NotOk = 12;

            bool logon(SOCKET s, const char *user)
            {
                Message msg;
                msg.check1 = CheckValue1;
                msg.check2 = CheckValue2;
                msg.kind = LogonMessage;
                msg.status = OnlyPacket;
                msg.length = 0;
                memset(msg.data, 0, sizeof(msg.data));

                unsigned int length = min(strlen(user), sizeof(msg.data));
#if defined(WIN32) && !defined(MAINWIN)
                strncpy_s(reinterpret_cast<char *>(msg.data), sizeof(msg.data), user, length);
#else
                strncpy(reinterpret_cast<char *>(msg.data), user, length);
#endif


                if (!tcpip::send_bytes(s, (const char *) &msg, sizeof(msg)))
                    return false;

                Reply reply;
                if (!tcpip::receive_bytes(s, (char *) &reply, sizeof(reply)))
                    return false;
                
                return 
                    reply.check1 == CheckValue1 && 
                    reply.check2 == CheckValue2 && 
                    reply.reply == Ok;
            }

            SOCKET setup_connection()
            {
                // Figure out who is currently running
#if _MSC_VER
                TCHAR username_str[100];
                DWORD username_size = sizeof(username_str);
                if (GetUserName(username_str, &username_size) == FALSE)
                    return invalid_socket;
#else
                char username_str[L_cuserid];
		        cuserid(username_str);
                if (!username_str)
                    return invalid_socket;
#endif
                
                Charstring wstr(username_str);
                string str = wstr.getNarrowString();

                const char *username = str.c_str();

                // Build an address to trace server.

                hostent *localhost = gethostbyname("localhost");
                if (localhost == 0)
                    return invalid_socket;

                char *local_ip = inet_ntoa(*(in_addr *) *localhost->h_addr_list);
                if (local_ip == 0)
                    return invalid_socket;

                unsigned long local_ip_number = inet_addr(local_ip);
                if (local_ip_number == INADDR_NONE)
                    return invalid_socket;

                sockaddr_in server;
                server.sin_family = AF_INET;
                server.sin_addr.s_addr = local_ip_number;

                // Connect to a trace server

                SOCKET sockets[50];
                const unsigned int socket_count = sizeof sockets / sizeof invalid_socket;
                
                unsigned int i;
                for (i = 0; i < socket_count; ++i)
                {
                    sockets[i] = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
                    if (sockets[i] == invalid_socket)
                        continue;

                    unsigned long n = 1;
                    if (ioctlsocket(sockets[i], FIONBIO, &n) == SOCKET_ERROR)
                    {
                        closesocket(sockets[i]);
                        sockets[i] = invalid_socket;
                        continue;
                    }

                    server.sin_port = htons(56000 + i);

                    if (::connect(sockets[i], reinterpret_cast<sockaddr *>(&server), sizeof(server)) == SOCKET_ERROR)
                    {
#if _MSC_VER
                        int result = WSAGetLastError();
                        if (result != WSAEWOULDBLOCK)
#else
                        if (errno != EWOULDBLOCK)
#endif
                        {
                            closesocket(sockets[i]);
                            sockets[i] = invalid_socket;
                            continue;
                        }
                    }

                }

                bool handled[socket_count];
                for (i = 0; i < socket_count; ++i)
                    handled[i] = false;

                fd_set read_fds, write_fds, except_fds;

                while (true)
                {
                    FD_ZERO(&read_fds);
                    FD_ZERO(&write_fds);
                    FD_ZERO(&except_fds);

                    unsigned int items = 0;

                    for (i = 0; i < socket_count; ++i)
                        if (!handled[i] && sockets[i] != invalid_socket)
                        {
                            ++items;
                            FD_SET(sockets[i], &read_fds);
                            FD_SET(sockets[i], &write_fds);
                            FD_SET(sockets[i], &except_fds);
                        }

                    if (items == 0)
                        break;
                    
                    int result = select(sizeof(sockets), &read_fds, &write_fds, &except_fds, 0);
                    if (result == SOCKET_ERROR || result == 0)
                    {
                        // TODO: Some kind of error handling here!
                        break;
                    }

                    for (i = 0; i < socket_count; ++i)
                    {
                        if(sockets[i]==invalid_socket) continue;

                        if (FD_ISSET(sockets[i], &read_fds))
                        {
                            assert(!handled[i] && sockets[i] != invalid_socket);
                            handled[i] = true;
                        }
                        if (FD_ISSET(sockets[i], &write_fds))
                        {
                            assert(!handled[i] && sockets[i] != invalid_socket);
                            handled[i] = true;
                        }
                        if (FD_ISSET(sockets[i], &except_fds))
                        {
                            assert(!handled[i] && sockets[i] != invalid_socket);
                            closesocket(sockets[i]);
                            sockets[i] = invalid_socket;
                            handled[i] = true;
                        }
                    }
                }

                SOCKET result = invalid_socket;

                for (i = 0; i < socket_count; ++i)
                {
                    if (sockets[i] != invalid_socket)
                    {
                        if (result == invalid_socket)
                        {
                            result = sockets[i];
                            unsigned long n = 0;
                            if (ioctlsocket(result, FIONBIO, &n) != SOCKET_ERROR)
                                if (TraceServer::logon(result, username))
                                    continue;
                            result = invalid_socket;
                        }

                        closesocket(sockets[i]);
                        sockets[i] = invalid_socket;
                    }
                }

                return result;
            }

            bool connect(SOCKET s)
            {
                Message msg;
                msg.check1 = CheckValue1;
                msg.check2 = CheckValue2;
                msg.kind = ConnectMessage;
                msg.status = OnlyPacket;
                msg.length = 0;
                memset(msg.data, 0, sizeof(msg.data));

                if (!tcpip::send_bytes(s, (const char *) &msg, sizeof(msg)))
                    return false;

                Reply reply;
                if (!tcpip::receive_bytes(s, (char *) &reply, sizeof(reply)))
                    return false;
                
                return 
                    reply.check1 == CheckValue1 &&
                    reply.check2 == CheckValue2 &&
                    reply.reply == Ok;
            }

            bool send_message(SOCKET s, const char *buffer, unsigned int bytes_to_send)
            {
                Message msg;
                msg.check1 = CheckValue1;
                msg.check2 = CheckValue2;
                msg.kind = TraceEventMessage;
                msg.status = OnlyPacket;
                msg.length = 0;

                Reply reply;

                unsigned int bytes_sent = 0;
                while (bytes_sent < bytes_to_send)
                {
                    // Send one package per loop iteration. For each package we 
                    // get a confirmation that it has been received.

                    msg.status = bytes_sent + 255 >= bytes_to_send ? LastPacket : MoreToCome;
                    msg.length = min((int)(bytes_to_send - bytes_sent), 255);

                    memcpy(msg.data, buffer + bytes_sent, msg.length);
                    if (msg.length < 255)
                        memset(msg.data + msg.length, 0, 255 - msg.length);
            
                    if (!tcpip::send_bytes(s, (const char *) &msg, sizeof(msg)))
                        return false;
            
                    if (!tcpip::receive_bytes(s, (char *) &reply, sizeof(reply)))
                        return false;

                    if (reply.check1 != CheckValue1 || reply.check2 != CheckValue2 || reply.reply == NotOk)
                        return false;

                    bytes_sent += msg.length;
                }

                return true;
            }

            bool disconnect(SOCKET s)
            {
                Message msg;
                msg.check1 = CheckValue1;
                msg.check2 = CheckValue2;
                msg.kind = DisconnectMessage;
                msg.status = OnlyPacket;
                msg.length = 0;
                memset(msg.data, 0, sizeof(msg.data));

                if (!tcpip::send_bytes(s, (const char *) &msg, sizeof(msg)))
                    return false;

                Reply reply;
                if (!tcpip::receive_bytes(s, (char *) &reply, sizeof(reply)))
                    return false;

                return
                    reply.check1 == CheckValue1 &&
                    reply.check2 == CheckValue2 &&
                    reply.reply == Ok;
            }
        }



        //
        // LogOnline
        //

        class TauHostTracerImpl : public EventMedia
        {
        public:
            TauHostTracerImpl() 
              : EventMedia(false),
                m_connected(false),
                m_socket(invalid_socket)
            {
#ifdef WIN32
                WSADATA wsaData;
                if (WSAStartup(MAKEWORD(2,2), &wsaData) != NO_ERROR)
                {
                    exit(1); // WSAStartup: error
                }
#endif
                if (meta::theApplication->isInstrumented())
                    m_socket = TraceServer::setup_connection();

            }

            ~TauHostTracerImpl() 
            {
                if (m_connected)
                    TraceServer::disconnect(m_socket);

                if (m_socket != invalid_socket)
                {
                    closesocket(m_socket);
                    m_socket = invalid_socket;
                }
#ifdef WIN32
                if (WSACleanup() == FALSE) {
                    ; // WSACleanup: error
                }
#endif
            }

            void emit(const Charstring &str)
            {
                if (m_socket == invalid_socket)
                    return;
                
                if (!m_connected)
                {
                    if (!TraceServer::connect(m_socket))
                        return;
                    m_connected = true;
                }

                TraceServer::send_message(m_socket, str.getNarrowString().c_str(), str.getNarrowString().size());
            }

        private:
            bool m_connected;
            SOCKET m_socket;
        };

        TauHostTracer::TauHostTracer(bool registerMedia)
          : EventMediaImpl(new TauHostTracerImpl, registerMedia)
        {}
    }
}

#endif
