/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torOs_linux_h
#define __torOs_linux_h

#include "torTypes.h"
extern "C"
{
#include <pthread.h>
}
#include <stdlib.h>
#include <errno.h>
#include <ostream>
#include <vector>

namespace tor 
{
    namespace os 
    {
        typedef int SOCKET;
        extern const SOCKET invalid_socket;

        typedef pid_t PID;
        extern const PID invalid_pid;

        typedef int FD;
        extern const FD invalid_fd;

        //
        // tcp/ip
        //

        namespace tcpip
        {
            void init();
            bool close(SOCKET s);
            SOCKET create_client_socket(const char *host, unsigned int port);
            SOCKET create_server_socket(unsigned int port, bool reuse);

            SOCKET accept(SOCKET s);

            bool wait_for_data(const std::vector<SOCKET> &wait_for, std::vector<SOCKET> &activity);

            bool send_bytes(SOCKET s, const char *bytes, unsigned int left_to_send);
            bool receive_bytes(SOCKET s, char *buffer, unsigned int left_to_recv);

            bool send_text(SOCKET s, const std::string &text);
            bool receive_text(SOCKET s, std::string &text);
        }

        //
        // TimeRep
        //

        class TimeRep
        {
        public:
            TimeRep() : m_ts() {set(0,0);}
            TimeRep(long s, long ns) : m_ts(){ set(s, ns); }
            TimeRep(const TimeRep &a, const TimeRep &b) : m_ts(){ set(a.m_ts.tv_sec - b.m_ts.tv_sec, a.m_ts.tv_nsec - b.m_ts.tv_nsec); }
            TimeRep(const TimeRep &a) : m_ts() { set(a.m_ts.tv_sec, a.m_ts.tv_nsec); }
            ~TimeRep() {}

            void norm();
            void zero();
            void setNow();

            void set(const TimeRep &x);
            void set(long s, long ns);
            void set(double d);

            void add(const TimeRep &x);
            void add(long s, long ns);

            void sub(const TimeRep &x);
            void sub(long s, long ns);
            void sub(const TimeRep &a, const TimeRep &b);

            int cmp(const TimeRep &x) const;

            double to_double() const;
            long to_ms() const;

            std::ostream &print(std::ostream &os) const;

            static double setUnit(double u);

        private:
            timespec m_ts;
            static double unit;
        };

        //
        // MutexRep
        //

        class MutexRep 
        {
        public:
            MutexRep();
            ~MutexRep();

            Status init(bool initialOwner);
            Status destroy();

            Status lock();
            Status unlock();

        private:
            pthread_mutex_t m_mutex;
            bool m_bInit;   // true if initialized
        };

        //
        // SemaphoreRep
        //

        class Time;
        class SemaphoreRep
        {
        public:
            SemaphoreRep();
            ~SemaphoreRep();

            Status init(unsigned int initialCount, unsigned int maxCount);
            Status destroy();

            Status put(unsigned int count);

            Status get();
            Status get(Time timeout);

        private:
            pthread_mutex_t m_mutex; 
            pthread_cond_t m_cond; 
            unsigned m_count;
            unsigned m_maxCount;
            bool m_bInit;   // true if initialized
        };

        //
        // ProcessRep
        //

        class ProcessRep : public ProcessRepCommon
        {
        public:
            typedef void (*OnExit)();

            ProcessRep();
            ~ProcessRep();

            bool launch(const tstring &cmd, const std::vector<tstring> &argv, const tstring &dir);
            bool terminate();

            bool isTerminated(int& terminationValue);

            PID getPID() const { return m_pid; }

        private:
            PID m_pid;
        };
    }
}

#endif

