// Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:21 PM CET 2014
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torProcess.h"
#include "torThread.h"
#include "torOs.h"

#include <iterator>
#include <utility>
#include <algorithm>
#include <functional>
#include <numeric>
#include <sstream>
#if TGTOS == OS_LINUX || TGTOS == OS_SOLARIS
#include <unistd.h>
#endif

namespace
{
    using namespace tor::os;

    // helper
    tstring quote(const tstring &str)
    {
        tstring result;

        if (str.empty())
            return result;

        bool quote = *str.rbegin() != _T('\"') && str.find_first_of(_T(" \t")) != tstring::npos;

        if (quote) 
            result.append(_T("\""));

        result.append(str);
        if (quote) 
            result.append(_T("\""));

        return result;
    }
}

namespace tor
{
    namespace os
    {

#if TOR_USE_PUTENV
        // Utility class to set up environment for launched process and clean them back later
        class EnvSetter{
            std::vector<std::string> m_toDelete;    // env vars that did not exist
            Process::ENV m_oldValues;   // stored previous env var values

            char* getEnvVar(const std::string& var_name);
            void setEnvVar(const Process::ENV_var& env);
            void delEnvVar(const std::string& var_name);

        public:
            EnvSetter(const Process::ENV &env);
            ~EnvSetter();
        };

        char* EnvSetter::getEnvVar(const std::string& var_name)
        {
            char *t=NULL;

#if TGTOS == OS_LINUX || TGTOS == OS_SOLARIS
            t=getenv(var_name.c_str());
#else
            size_t sz;
            getenv_s(&sz,NULL,0,var_name.c_str()); // get size

            if(sz>0)
            {
                t=new char[sz];
                getenv_s(&sz,t,sz,var_name.c_str());
            }
#endif
            return t;
        }

        void EnvSetter::setEnvVar(const Process::ENV_var& env)
        {
            std::string e = env.first;
            e.append("=");
            e.append(env.second);
#if TGTOS == OS_LINUX || TGTOS == OS_SOLARIS
            size_t len=e.length()+1;
            char* pVar=new char[len];
            strncpy(pVar, e.c_str(), len );

            putenv(pVar);
#else
            _putenv(e.c_str());
#endif
        }

        void EnvSetter::delEnvVar(const std::string& var_name)
        {
#if TGTOS == OS_LINUX || TGTOS == OS_SOLARIS
            unsetenv(var_name.c_str());
#else   // win
            std::string tmp=var_name;
            tmp+="=";  // make "var=" string to delete value
            _putenv(tmp.c_str());
#endif
        }


        EnvSetter::EnvSetter(const Process::ENV& env)
        {
            Process::ENV::const_iterator pVar=env.begin();

            while(pVar!=env.end())
            {
                if(pVar->first.empty()) 
                {
                   ++pVar;
                   continue;
                }

                char* pVal=getEnvVar(pVar->first);

                if(!pVal) // no such variable
                    m_toDelete.push_back(pVar->first);  // delete afterwards
                else
                if(!strcmp(pVal, pVar->second.c_str()) ) // the same value, keep untouched
                {
                    ++pVar;
                    continue;
                }
                else
                {
                    // remember the value to restore afterwards
                    m_oldValues.push_back(Process::ENV_var(pVar->first, std::string(pVal)));
                    delete pVal;
                }

                setEnvVar(*pVar++);
            }
        }

        EnvSetter::~EnvSetter()
        {
            // Delete values that did not exist before
            std::vector<std::string>::iterator pDel=m_toDelete.begin();

            while(pDel!=m_toDelete.end())
                delEnvVar(*pDel++);

            // Restore previous values
            Process::ENV::iterator pSet=m_oldValues.begin();

            while(pSet!=m_oldValues.end())
                setEnvVar(*pSet++);
        }
#endif

        Process::Process() : m_impl(), m_bTerminateOnDelete(true)
        {
        }

        Process::~Process()
        {
            if (m_bTerminateOnDelete) 
                terminate();
        }

        bool Process::launch(const tstring &cmd, const std::vector<tstring> &argv, const ENV &env, const tstring &dir)
        {
            // Handle environment. Reuse the current process env block with
            // some additions.
#if TOR_USE_PUTENV
            EnvSetter setter(env);
#endif
            return m_impl.launch(cmd, argv, dir);
        }

        void Process::setOptions(int options)
        {
            m_impl.setOptions(options);
        }

        bool Process::isTerminated(int& terminationValue)
        {
            return m_impl.isTerminated(terminationValue);
        }
    }
}

// End of File

