/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torRegion_h__
#define __torRegion_h__ (1)

#include "torPlatform.h"

#include "torDispatchable.h"

namespace tor {

    class EntryPoint;
    class ExitPoint;
    class Event;
    class State;

    class TOR_EXT_DEF Region {
    public:
        virtual ~Region();
        Region( State* owner ) : m_owner(owner), m_current(0), m_previous(0), m_finished(false) {};

        // Executes an event by passing it to the current state. If there's no current
        // state, the event is passed to the state owning this region.
        // Returns Dispatchable::Consumed is the event was consumed (and thereby deleted),
        // or Dispatchable::NoMatch if no match was found. In this case it is the responsibility
        // of the caller to delete the event.
        Dispatchable::EventAction execute( Event* );
    
        // Enters the region by entering its current state if any. The initial 
        // transition is used the first time or if there's no current state.
        void enter();

        // Enter the region using history, i.e. entering the previous state.
        // the default history transition is executed if there's no previous state.
        // The deepHistory flag specifies if shallow or deep history shall be used.
        // If the flag is true any composite states or submachines in the region 
        // will be recursively entered using history. Otherwise history only 
        // applies to this region.
        void enterHistory(bool deepHistory = false);

        // Leaves the region by leaving the current state.
        void leave();

        // Finishes this region by marking it as finished, and checks if all 
        // regions of the owning state are finished. In that case it calls
        // finish on the state as well.
        virtual void finish(ExitPoint* ep = 0);
    
        // Performs a call to 'initialTransition'. The call is synchronous.
        // Overload this function to make the call in an asynchronous way.
        virtual void callInitialTransition() {initialTransition();}

        // The initial transition of the region. Executed the first time the region 
        // is entered. Implemented by concrete subclasses.
        virtual void initialTransition() {};

        // The default history transition of the region. Executed if the region
        // is entered using history, but there is no previous state.
        // Implemented by concrete subclasses.
        virtual void defaultHistoryTransition() {};

        // Used to execute transitions from exit points of owned submachines. When a submachine
        // is left this operation is called passing the exit point as a parameter and any matching
        // transition is taken.
        virtual void exitPointTransitions(ExitPoint* ep) {};

        // Used to execute transitions from entry points of the region.
        virtual void entryPointTransitions(EntryPoint* ep) {};

        // The owning state. Events not handled by the region are passed on to the
        // owning state.
        State*	    m_owner;

        // The currently active state of the region. Events received by the region 
        // are passed on to this state. Is set when entering a state in the region 
        // and cleared when leaving a state.
        State*	    m_current;

        // The previously active state of the region. Used when entering the region 
        // using history. Is set when changing the current state, e.g. when a state 
        // is entered or left.
        State*	    m_previous;

        // Flag stating if the regions is finished or not.
        bool        m_finished;
    };

} // tor namespace

#endif  // __torRegion_h__

