/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torState_h__
#define __torState_h__ (1)

#include "torPlatform.h"

#include "torDispatchable.h"
#include <list>

namespace tor {

    class EntryPoint;
    class ExitPoint;
    class Region;
    class StateMachine;

    class TOR_EXT_DEF State {
    public:
        State( Region* owner ) : m_owner( owner ) {}

        // Deletes the owned submachine and all regions owned by the state.
        virtual ~State();
    
        // Execute an event in this state. Implements transitions in response to events
        // and guards.
        // Returns Dispatchable::Consumed if the event was consumed (and thereby deleted),
        // or Dispatchable::NoMatch if no match was found. In this case it is the responsibility
        // of the caller to delete the event.
        // Must be implemented by concrete subclasses.
        virtual Dispatchable::EventAction execute( Event* ) = 0;

        // Initialize the state. Default implementation does nothing.
        // A derivate representing a composite state should override
        // this function and initialize the statemachine of the composite state.
        virtual void init() {}

        // Returns the statemachine of a composite state. If the state is not
        // a composite state, NULL is returned.
        virtual StateMachine* getStateMachine() { return 0; }

        // Enter the state, optionally using the supplied entry point.
        // If the state has a submachine, the submachine is entered, otherwise
        // all top level regions of the state are entered.
        virtual void enter(EntryPoint* ep = 0);

        // Same as enter, but in addition uses the source state of the transition
        // to calculate which regions/states that needs to be left/entered to
        // enter this state. Every region/state from the source state to the
        // least common ancestor of the source state and this state should be left,
        // and every state from the ancestor to this state should be entered.
        virtual void enterFrom(State*, EntryPoint* ep = 0);

        // Enter the state using history, i.e. enter all top level regions using history.
        virtual void enterHistory(bool deepHistory = false);
    
        // Same as enterHistory, but use source state to calculate which regions/states
        // to enter/leave (same as for enterFrom).
        virtual void enterHistoryFrom(State*);  

        // Leave the state by leaving the submachine or all top level regions.
        virtual void leave();

        // Leave the state for a specific target state, i.e. leave all states up to
        // the least common ancestor of this state and the target state.
        virtual void leaveFor(State*);

        // Same as leaveFor, but use an exit point to calculate the least common
        // ancestor.
        virtual void leaveFor(ExitPoint*);
        
        // Finishes this state by generating a CompletedEvent and executing it.
        void finish();

        // Returns true if this state is finished, i.e. all owned regions are 
        // finished, false otherwise.
        bool isFinished();

        // Adds a region to this state.
        void addRegion(Region* r);

        // Returns the region that's least common ancestor of the two states.
        // Returns 0 if no common ancestor can be found.
        static Region*  GetLCR(const State*, const State*);

        // Returns the parent/ancestor of the supplied state in the supplied region.
        // Returns 0 if no parent is found in the region.
        static State*   GetParentInRegion(State*, Region*);

        // The region owning this state.
        Region*	m_owner;

        // List of regions owned by this state.
        std::list<Region*> m_regions;
    };

} // tor namespace

#endif  // __torState_h__

