/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
#ifndef __torSync_h
#define __torSync_h

#include "torPlatform.h"

#include "torTime.h"
#include "torOsRep.h"

namespace tor
{
    namespace os
    {
        //
        // Mutex
        //
        
        class TOR_EXT_DEF Mutex 
        {
        public:
            Mutex() : m_rep() {}
            ~Mutex() {}
            
            Status init(bool initialOwner = false) { return m_rep.init(initialOwner); }

            // Note that the mutex is automatically destroyed when destructing the MutexRep object, so usually
            // this function should not be explicitly called.
            Status destroy() { return m_rep.destroy(); }

            Status lock() { return m_rep.lock(); }
            Status unlock() { return m_rep.unlock(); }
        
        private:
            Mutex(const Mutex &);            // Do not use!
            Mutex &operator=(const Mutex &); // Do not use!

            MutexRep m_rep;
        };

        //
        // Semaphore 
        //
        
        class TOR_EXT_DEF Semaphore 
        {
        public:
            Semaphore() : m_rep() {}
            ~Semaphore() {}

            Status init(unsigned int initialCount = 0, unsigned int maxCount = INT_MAX) { return m_rep.init(initialCount, maxCount); }

            // Note that the semaphore is automatically destroyed when destructing the SemaphoreRep object, so usually
            // this function should not be explicitly called.
            Status destroy() { return m_rep.destroy(); }

            Status put(unsigned int count = 1) { return m_rep.put(count); }
            Status get() { return m_rep.get(); }
            Status get(Time timeout) { return m_rep.get(timeout); }  // Get with time-out

        private:
            Semaphore(const Semaphore &);            // Do not use!
            Semaphore &operator=(const Semaphore &); // Do not use!

            SemaphoreRep m_rep;
        };

        // 
        //
        //

        class TOR_EXT_DEF RWLock 
        {
        public:
            RWLock();
            ~RWLock();

            Status read_lock();
            Status read_unlock();

            Status write_lock();
            Status write_unlock();

        private:
            RWLock(const RWLock &);            // not supported
            RWLock &operator=(const RWLock &); // not supported

        private:
            Mutex m_mutex;
            Semaphore m_read;
            Semaphore m_write;
            unsigned m_read_count;          // No of read tokens checked out 
            unsigned m_pending_read;        // No of PENDING read requests
            unsigned m_pending_write;       // No of PENDING write requests 
        };

        //
        // A Gate for thread synchronization. 
        // Only one thread may pass the gate at any time.
        //

        class TOR_EXT_DEF Gate 
        {
        public:
            Gate();
            ~Gate();

            // Set or determine the lock status
            bool isLocked();
            void setLocked(bool l);

            // Unlock the gate.
            void unlock();

            // Lock the gate.
            void lock();

            // Try to enter the gate.
            void enter();

        private:
            Gate(const Gate &);            // not supported
            Gate &operator=(const Gate &); // not supported

        private:
            Mutex m_mutex;
            Semaphore m_sem;
            bool m_locked;
        };

        // Utility class that couples a Mutex lock to the lifetime of an instance.

        class TOR_EXT_DEF auto_lock
        {
        public:
            auto_lock(Mutex &r) : m_r(r) {m_r.lock();}
            ~auto_lock() {m_r.unlock();}

        private:
            auto_lock(const auto_lock &);

        private:
            Mutex &m_r;
        };
    }
}

#endif

