// Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:21 PM CET 2014
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torThreadSafeEventQueue.h"
#include <algorithm>
#include <functional>
#include <assert.h>

tor::ThreadSafeEventQueue::ThreadSafeEventQueue()
  : EventQueue(),
    m_mutex(),
    m_read()
{
    m_mutex.init();
    m_read.init(0);
}

tor::ThreadSafeEventQueue::~ThreadSafeEventQueue() {}

// "blocking"
void tor::ThreadSafeEventQueue::insert(Event *e)
{
    os::auto_lock lock(m_mutex);
    EventQueue::insert(e);
    m_read.put();
}

// non-blocking
bool tor::ThreadSafeEventQueue::remove(Event *event)
{
    os::Time now;
    now.setNow();

    os::Status status = m_read.get(now);
    if (status == os::StatusTIMEOUT)
        return false;
    if (status != os::StatusOK)
        return false; // ERROR

    os::auto_lock lock(m_mutex);
    assert(!m_list.empty());

    if (!EventQueue::remove(event))
    {
        m_read.put();
        return false;
    }
    return true;
}

// blocking
tor::Event *tor::ThreadSafeEventQueue::get()
{
    if (m_read.get() != os::StatusOK)
        return 0; // ERROR

    os::auto_lock lock(m_mutex);
    Event *event = EventQueue::get();
    assert(event != 0);
    return event;
}

// blocking
tor::Event *tor::ThreadSafeEventQueue::get(os::Time timeout)
{
    os::Status status = m_read.get(timeout);
    if (status == os::StatusTIMEOUT)
        return 0;
    if (status != os::StatusOK)
        return 0; // ERROR

    os::auto_lock lock(m_mutex);
    Event *event = EventQueue::get();
    assert(event != 0);
    return event;
}


// non-blocking
bool tor::ThreadSafeEventQueue::empty() const
{
    os::auto_lock lock(m_mutex);
    return EventQueue::empty();
}

