// Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:21 PM CET 2014
// Licensed Materials - Property of IBM 
// (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
// 
// U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#include "torPlatform.h"

#include "torUtilities.h"
#include "torEvent.h"
#include "torDispatchableClass.h"
#include "torPort.h"

#include "torMetaEvents.h"
#include "torEventManager.h"

using namespace tor;

namespace {
    bool isInitialized = true; // TOR is automatically initialized when first loaded.
}

std::ostream &operator<< (std::ostream &os,  const tor::Charstring &str)
{
    return os<<str.getNarrowString();
}

std::wostream &operator<<(std::wostream &os, const tor::Charstring &str)
{
    return os<<str.getWideString();
}

void tor::initializeLibrary()
{
    if (isInitialized)
        return; // Already initialized.

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
    meta::EventManager::initialize();
#endif

    isInitialized = true;
}

void tor::finalizeLibrary()
{
    if (!isInitialized)
        return; // Already finalized.

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
    meta::EventManager::finalize();
#endif

    isInitialized = false;
}

bool tor::sendTo(Event *event, InstanceManager::ObjectId receiverId)
{
    if (!InstanceManager::instance().isValidReceiver(receiverId))
        return false;

    EventReceiver *receiver = InstanceManager::instance().getReceiverFromId(receiverId);
    if (receiver == 0)
        return false;

    return sendTo(event, receiver);
}

bool tor::sendTo(Event* e, EventReceiver* receiver) {
  if (!receiver)
      return false; // No receiver specified!

  // Send the event to the receiver.
  bool bReceived = receiver->receive( e ); 

  // Delete the event if not received.
  if (!bReceived && e)
    delete e;

  return bReceived;
}

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
bool tor::sendTo(Event* e, EventReceiver* receiver, const Entity& sender, meta::Type* eventMetaType) {
  if (!receiver)
      return false; // No receiver specified!

  if (!InternalEvent::isTypeOf(e)) // Do not trace internal events
      meta::EventManager::processEvent(new meta::EventSent(e, sender, new meta::StructuredValue(eventMetaType)));

  return sendTo(e, receiver);
}
#endif

bool tor::sendTo(Event* e, Port* pp) {
  if (!pp)
      return false; // No port specified!

  // Send the event to the port. The responsibiliy of the event
  // is passed on to the port.
  bool bReceived = pp->send( e, false /* deleteEvent */ );

  // Delete the event if not received.
  if (!bReceived && e)
    delete e;

  return false;
}

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
bool tor::sendTo(Event* e, Port* pp, const Entity& sender, meta::Type* eventMetaType) {
  if (!pp)
      return false; // No port specified!

  if (!InternalEvent::isTypeOf(e)) // Do not trace internal events
      meta::EventManager::processEvent(new meta::EventSent(e, sender, new meta::StructuredValue(eventMetaType)));

  return sendTo(e, pp);
}
#endif

void tor::setTimeUnit(double seconds) {
    tor::os::Time::setUnit(seconds);
}

#if defined(TOR_USE_INSTRUMENTATION) && !defined(TOR_NO_EVENTMANAGER)
void tor::debugBreak(const tor::Charstring& strGuid) {
    meta::EventManager::processEvent(new meta::BreakpointHit_Charstring(strGuid));
}
#endif

