# Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:17 PM CET 2014
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
if {[info exists env(TAU_ECLIPSE_DEBUG)]} {
    proc Verbose {s} {
        Output "ECLIPSE_DEBUG: $s"
    }
    Verbose "[info script]\n"
} else {
    proc Verbose {s} {
    }
}

source 	[file join [std::GetInstallationDirectory] addins/JavaApplication/Script/nameUtils.tcl]
source 	[file join [std::GetInstallationDirectory] addins/JavaApplication/Script/JavaUtils.tcl]
source 	[file join [std::GetInstallationDirectory] addins/EclipseIntegration/Script/OptionFile.tcl]

proc putSock {sock str} {
    Verbose "putSock $sock $str\n"
    puts $sock "$str"
}
proc getSock {sock} {
    Verbose "getSock reading $sock ... "
    set str [gets $sock]
    Verbose " got :$str:\n"
    return "$str"
}

set protoDone 0
set version "version 1"
set tclPort [GetTauTclPort]
set eclipsePort [GetTauIntegrtnPort]
set gotVersion 0
set gotPort [expr {$eclipsePort > 0}]
set initPort 0

proc Echo_Accept {sock addr port} {
    global echo

    Verbose "Accept $sock from $addr port $port\n"
    set echo(addr,$sock) [list $addr $port]

    fconfigure $sock -buffering line

    fileevent $sock readable [list Echo $sock]
}

proc Echo {sock} {
    global echo protoDone gotVersion version tclPort gotPort eclipsePort

    if {[eof $sock] || [catch {gets $sock line}]} {
	close $sock
	Verbose "Close $echo(addr,$sock)\n"
	unset echo(addr,$sock)
        set protoDone 1
    } else {
        Verbose "got:$line:\n"
        if { ! $gotVersion } {
            if { $line == $version } {
                set gotVersion 1
            } else {
                Output "bad version: $line\n"
                close $sock
                Verbose "Close $echo(addr,$sock)\n"
                unset echo(addr,$sock)
                set protoDone 1
            }
        } elseif { ! $gotPort } {
            if { [regexp {port ([0-9]+)} $line match port] } {
                set gotPort 1
                set eclipsePort $port
                puts $sock $version
                puts $sock "port $tclPort"
                Verbose "put: $version\n"
                Verbose "put: port $tclPort\n"
            } else {
                Output "protocol error: $line\n"
                close $sock
                Verbose "Close $echo(addr,$sock)\n"
                unset echo(addr,$sock)
                set protoDone 1
            }
        }
    }
}

# Returns the enclosing package of the element. Based on the Owner metafeature.
proc GetPackage { element } {
	if [u2::IsKindOf $element Package] {
		return $element
	} elseif [u2::IsKindOf $element Session] {
		return 0
	} else {
		return [GetPackage [u2::GetEntity $element Owner]]
	}
}

proc GetProjectId { projectName } {
	if [catch {set projects  [std::GetProject]}] {
		set project 0
	}
	foreach project $projects {
		if {[file root [file tail [Get $project pathname]]] == $projectName} {
			return $project
		}
	}
	return 0
}

# TODO: Reimplement this. Since the options file isn't needed anymore we
# need a new mechanism to know if a project is "Eclipse-enabled" or not
proc IsEclipseProject { project } {
#	set projectPath [Get $project pathname]
#  	set projectLocation [file dirname $projectPath]
#  	set projectName [file tail [file root $projectPath]]
#  	set eclipseOptionsPath [file join ${projectLocation} ${projectName}.ecl]
#	return [file exists $eclipseOptionsPath]
	return 1
}

# returns element that can be located in Eclipse
# (e.g. eclipse doesn't support locate for local variables)
proc GetLocatableElement { element } {
	if {(0 == $element) || ("" == $element)} {
		return 0
	}
	
	if { [u2::IsKindOf $element Session] } {
		return 0
	}

	if {[u2::IsKindOf $element PresentationElement] && ![u2::IsKindOf $element Diagram]} {
		set target [u2::GetEntity $element ModelElement]
		if {(0 == $target) || ("" == $target)} {
			return 0
		}
	} else {
		set target $element
	}
	
	# can't locate unnamed package in eclipse
	if { [IsUnnamedPackage $target] } {
		return 0
	}

	if {[IsJavaElement $target] || [u2::IsKindOf $target Package] || [u2::IsKindOf $target Operation] } {
			return $target
	}
	
	set owner [u2::GetEntity $target Owner]

	# this is attribute in class/interface
	if { [u2::IsKindOf $target Attribute] && [u2::IsKindOf $owner Classifier] } {
		return $target
	}

	# in case of non locatable elements traverse the owner hierarchy until the owner is a classifier or a package
	return [GetLocatableElement $owner]
}

proc IsLocatable { element } {
	set locatableElement [GetLocatableElement $element]
	if { $locatableElement != 0 } {
		return 1
	} else { 
		return 0
	}
}

# Update all files of the project and refresh Eclipse
proc SynchronizeProject { project } {
	# Update java files of project
	set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@GenerateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        set param {1}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }

	# Refresh project in Eclipse
	if {![CanConnectToEclipse] && ![EstablishConnection]} {
		return
	}
  	set projectName [file tail [file root [Get $project pathname]]]
	RefreshResource $projectName
}

proc LocateElementInEclipse { element } {
	set javaName [GetJavaQualifiedName $element]
	if {"" != $javaName} {
		# Launch Eclipse if needed
		if {![CanConnectToEclipse] && ![EstablishConnection]} {
			return 0
		}
		set project [std::GetActiveProject]
		set projectName [file tail [file root [Get $project pathname]]]
		if [IsJavaElement $element] {
			set kind "type"
		} elseif [u2::IsKindOf $element Package] {
			set kind "package"
		} elseif [u2::IsKindOf $element Attribute] {
			set kind "field"
 		} elseif [u2::IsKindOf $element Operation] {			
			if {[u2::GetValue $element OperationKind] == "OkConstructor"} {
				set kind "constructor"
			} else {
				set kind "method"
			}
		} else {
			return 0
		}
		Output "\nLocating '$javaName' in Eclipse ... "
		return [NavigateToJavaElement $projectName $javaName $kind]
	}
	return 0	
}

proc ExportProjectToEclipse { project } {
	if {![CanConnectToEclipse] && ![EstablishConnection]} {
		return 0
	}
	SetTauLocation
	set projectPath [Get $project pathname]
	set projectLocation [GetDefaultOutputLocation $projectPath]
	set projectName [file tail [file root $projectPath]]
	set projFileName [file tail $projectPath]
	if [CreateEclipseProject $projectName ""] {
		CreateLinkToTauFolder $projectName $projectLocation $projFileName $projectPath
		set session [std::GetModels -kind U2 -project $project]
		set sourceDirs [GetProjectSourceLocationsFromEclipse $projectName]
		foreach sourceDir $sourceDirs {
			if {$sourceDir != ""} {
				foreach rootPackage [u2::GetEntities $session Root] {
                                    if {0 == $rootPackage ||
                                        ![u2::IsKindOf $rootPackage Package] ||
                                        ![u2::HasAppliedStereotype $rootPackage "javaPackage"] ||
                                        [u2::HasAppliedStereotype $rootPackage "@TTDCppAppGen@GenerationResult" -guid 1] ||
                                        [u2::HasAppliedStereotype $rootPackage "@Predefined@hidden" -guid 1]} {
                                        continue
                                    }
                                    
                                    Verbose "Source dir: $sourceDir \n"
                                    Verbose "Root package: $rootPackage \n"
                                    
                                    #ExportPackageToDirectory $sourceDir $rootPackage
                                    set curProject [std::GetActiveProject]
                                    set model [std::GetModels -kind "U2" -project $curProject]
                                    set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@ExportToDirectory]
        
                                    if { 0 == $agent } {
                                        std::Output "Failed to find Import Java agent \n"
                                        # failed to find agent
                                        return 0
                                    }
                                    
                                    set param [list $sourceDir]

                                    # Agent should run from project directory
                                    set currentDir [pwd]
                                    cd $projectLocation
                                    if { [catch {
                                        u2::InvokeAgent $model $agent $rootPackage param
                                    } e ] } {
                                        std::Output "Error when invoking Export Java agent : $e\n"
                                    }
                                    cd $currentDir
				}
				set nativeDir [file nativename $sourceDir]
				#AddProjectSourcelocation $project $nativeDir
			}
		}
		RefreshResource $projectName
		return 1
	} else {
		return 0
	}
}

#############################################################
# Procedures used to communicate with Eclipse
#############################################################
proc GetEclipseSocket { {silent 0} } {
        global eclipsePort gotPort env
        set result 0
        if { ! $gotPort } {            
			# try to get port from temporary file
			if {[info exist env(TEMP)]} {
				set filename [file join $env(TEMP) .EclipseIntegration_port]
				if { ! [file exists $filename] } {
					return $result
				}
			} elseif {[info exist env(TAUECLIPSEINT)]} {
				set filename [file join $env(TAUECLIPSEINT) .EclipseIntegration_port]
				if { ! [file exists $filename] } {
					return $result
				}
			} else {
				output "\nNo integration enviroment variables are declared. Please use TEMP in case of Windows and TAUECLIPSEINT otherwise.\n"
			}
			set fileId [open $filename r]
			set eclipsePort [gets $fileId]
			set gotPort [expr {$eclipsePort > 0}]
			close $fileId	

			if { !$gotPort } {
				return $result
			}
        }
	if {[catch {set result [socket localhost $eclipsePort]}] && !$silent} {
		output "\nError: Unable to open socket $eclipsePort on localhost. Please try again.\nIf problem persists, contact IBM Rational Software Support."
	} 
	if {0 != $result} {	   
		fconfigure $result -buffering line
	}	
	return $result
}

proc CanConnectToEclipse {} {
	set eclipseSocket [GetEclipseSocket 1]
	if  {0 == $eclipseSocket} {
		return 0
	} else {
		close $eclipseSocket
		return 1
	}
}

proc EstablishConnection {} {
        global protoDone initPort tclPort eclipsePort gotVersion gotPort initPort
	if ![CanConnectToEclipse] {
		set optionsFile [file join [GetUserOptionsDir] eclipseOptions.ini]
		set eclipseExe [GetOptionValue $optionsFile ExecutablePath]
		set platformOptions [GetOptionValue $optionsFile PlatformOptions]
		if ![file exists $eclipseExe] {
			set eclipseExe [Dialog -variable "LocateEclipseExecutable" -name "Locate Eclipse Executable" -w 365 -h 180 -onbuilddialog OnBuildLocateEclipseDialog -oninitdialog OnInitLocateEclipseDialog -onclosedialog OnCloseLocateEclipseDialog -closecmddialog CloseDialog]
			if {($eclipseExe != 0) && [file exists $eclipseExe]} {
				SetOptionValue $optionsFile ExecutablePath $eclipseExe
  			} else {
				Output "\nError: Invalid Eclipse executable specified."
				return 0
			}
		}
            
            set protoDone 0
            set tclPort [GetTauTclPort]
            set eclipsePort [GetTauIntegrtnPort]
            set gotVersion 0
            set gotPort 0 
            set initPort 0

		Output "\nLaunching Eclipse..."
                set s [socket -server Echo_Accept $initPort]
                set initPort [lindex [fconfigure $s -sockname] 2]
                Verbose "listen on port $initPort\n"
                Verbose "Ask for workspace"
                set wsdir [DirectoryDialogEx -title "Please, select Eclipse workspace folder."]
				Verbose "\nWorkDir: $wsdir\n"
				if {0 != $wsdir} {
					if {"" != $platformOptions} {
						if [catch {eval [subst {exec << {[pid]} {$eclipseExe} -data $wsdir $platformOptions -Integration $initPort &}]} result] {
                            output "$result\n"
						} else {
                            Verbose "eclipse pid $result\n"
                            Verbose "before vwait protoDone\n"
                            vwait protoDone
                        }
					} else {
						if [catch {exec << [pid] $eclipseExe -data $wsdir -Integration $initPort &} result] {
                            output "$result\n"
						} else {
                            Verbose "eclipse pid $result\n"
                            Verbose "before vwait protoDone\n"
                            vwait protoDone
                        }
					}
				}
            Verbose "protoDone: tclPort=$tclPort eclipsePort=$eclipsePort\n"
            close $s
            set initPort 0
	}
	set counter 0
	while {(![CanConnectToEclipse] && (4 > $counter))} {
		incr counter
		after 1000
		if {$counter >= 15} {
			if {[MessageDialog -name "IBM Rational Tau" -message "Unable to communicate with the Eclipse executable.\nWould you like to try again?" -style yesno -icon stop] == 6} {
				set counter 0
			} else {
				Output "\nFailed to communicate with Eclipse. Aborted."
				return 0
			}
		}
	}
	return 1
}

proc GetDefaultProjectLocation { } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return ""
	}
	putSock $eclipseSocket "GetDefaultProjectLocation"
	set result [getSock $eclipseSocket]
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result ""
	}	
	close $eclipseSocket
	return $result
}

proc GetProjectSourceLocationsFromEclipse { projName } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return ""
	}
	putSock $eclipseSocket "GetProjectSourceLocations"
	putSock $eclipseSocket $projName

	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result ""
	} else {
		if [catch {set count [expr [getSock $eclipseSocket]]}] {
			close $eclipseSocket
			return ""
		}
		for {set i 0} {$i < $count} {incr i} {
			lappend result [getSock $eclipseSocket]
		}
	}
	close $eclipseSocket
	return $result
}

proc RefreshResource { resource } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return 0
	}
	putSock $eclipseSocket "RefreshResource"
	putSock $eclipseSocket $resource
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result 0
	} else {
		set result 1
	}
	close $eclipseSocket
	return $result
}

proc CreateEclipseProject { name path } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return 0
	}

	# Find out the Java version
	# "JAVA_1_4" represents version 1.4
	# "JAVA_5" represents version 1.5
	set session [GetActiveSession]
	if { $session != 0 } {    
		set javaVersion [GetTaggedValue $session "javaSettings (. languageVersion .)" -asString]
	} else {
		# error ? - default 1.4
		set javaVersion "JAVA_1_4"
	}

	putSock $eclipseSocket "CreateProject"
	putSock $eclipseSocket $name
	# Protocol modified: the Java version is written to socket as
	# a third argument
	putSock $eclipseSocket $javaVersion
	if {"" != $path} {
		putSock $eclipseSocket "1"
		putSock $eclipseSocket $path
	} else {
		putSock $eclipseSocket "0"
	}
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result 0
	} else {
		set result 1
	}
	close $eclipseSocket
	return $result
}

proc NavigateToJavaElement {projectName elementName elementKind} {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return 0
	}
	putSock $eclipseSocket "NavigateToJavaElement"
	putSock $eclipseSocket $projectName
	putSock $eclipseSocket $elementName
	putSock $eclipseSocket $elementKind
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result 0
	} else {
		set result 1
	}
	close $eclipseSocket
	return $result
}

proc SetTauLocation { } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return 0
	}
	putSock $eclipseSocket "SetTauLocation"
	putSock $eclipseSocket [std::GetInstallationDirectory]
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result 0
	} else {
		set result 1
	}
	close $eclipseSocket
	return $result
}

proc CreateLinkToTauFolder { projectName projectLocation projFileName projectPath } {
	set eclipseSocket [GetEclipseSocket]
	if {0 == $eclipseSocket} {
		return 0
	}
	putSock $eclipseSocket "CreateLinkToTauFolder"
	putSock $eclipseSocket $projectName
	putSock $eclipseSocket $projectLocation
	putSock $eclipseSocket $projFileName
	putSock $eclipseSocket $projectPath
	set status [getSock $eclipseSocket]
	if {$status != "#SUCCESS#"} {
		Output "\n[getSock $eclipseSocket]"
		set result 0
	} else {
		set result 1
	}
	close $eclipseSocket
	return $result
}

#############################################################
# Procedures used to communicate with Tau from Eclipse
#############################################################
proc CreateAndSynchronizeProject { ackPort name projectLocation srcLocations javaVersion generateDiagrams} {
    Ack $ackPort
	CreateProject $name $projectLocation $javaVersion
	SetTauLocation
	set projectFile [file join $projectLocation $name].ttp
	if [file exists $projectFile] {
		OpenDocument $projectFile
		set project [GetProjectId $name]
		if {0 != $project} {
			set session [lindex [std::GetModels -kind U2 -project $project] 0]
			foreach root [u2::GetEntities $session Root] {
                if {0 == $root} {
                    continue
                }
				u2::Delete $root
			}
			foreach srcLocation $srcLocations {
			#AddProjectSourcelocation $project [file native $srcLocation]

			#UpdateModelProject $project ""
			# Make path relative
            set relativePath $srcLocation
			if {[file pathtype $srcLocation] == "absolute"} {
                set projectDir [GetProjectDir $project]
                if { ("" != $projectDir) && ($projectDir != $srcLocation) } {
                    set relativePath [GetRelativePath $srcLocation $projectDir]
                    # If path equals project directory, add full path
                    if { "" == $relativePath } {
                        set relativePath "."
                    }
                }
            }
			
			##################### Invokation of agent ###############
                        set curProject [std::GetActiveProject]
                        set model [std::GetModels -kind "U2" -project $curProject]
                        set agent [u2::FindByGuid $model "@TTDJavaModelCodeSync@ImportDirectory"]
        
                        if { 0 == $agent } {
                            std::Output "Failed to find Import Java agent \n"
                            # failed to find agent
                            return 0
                        }
                        if $generateDiagrams {
                            set generateDiagrams 1
                        } else {
                            set generateDiagrams 0
                        }
                     
                        set param [list $relativePath $generateDiagrams]

                        if { [catch {
                            u2::InvokeAgent $model $agent $model param
                        } e ] } {
                            std::Output "Error when invoking Import Java agent : $e\n"
                        }
                        }
                        
			set resources [u2::GetEntities $session Resource]
			foreach resource $resources {
                                if {0 == $resource} {
                                    continue
                                }
				u2::SaveResource $resource
			}
		}
	}
}

proc CreateProject { name projectLocation javaVersion } {    
	set defaultProjectPath [file join [std::GetInstallationDirectory] addins/EclipseIntegration/Etc/Default.ttp]
	set newProjectPath [file join $projectLocation $name].ttp
	set newU2FilePath [file join $projectLocation $name].u2

	if ![file exists $defaultProjectPath] {
		Output "\nDefault project file missing!"
		return 0
	}
	if [catch {set defaultFile [open $defaultProjectPath r]}] {
		return 0
	}
	if [catch {set newProjectFile [open $newProjectPath w]}] {
		close $defaultFile
		return 0
	}
	while {[gets $defaultFile line] >= 0} {
  		if [regexp "(.*)%U2FILE_NAME%(.*)" $line] {
			regsub "(.*)%U2FILE_NAME%(.*)" $line "\\1$name\\2" line
		} elseif [regexp "(.*)%JAVA_VERSION%(.*)" $line] {
			regsub "(.*)%JAVA_VERSION%(.*)" $line "\\1$javaVersion\\2" line
		}
		puts $newProjectFile $line
	}
	close $defaultFile
	close $newProjectFile
	if ![catch {set newU2File [open $newU2FilePath w]}] {
		close $newU2File
	}
	return 1
}

proc LocateJavaElementInProject { ackPort name projectLocation qualifiedName } {
    Ack $ackPort
	set projectFile [file join $projectLocation $name].ttp
	if ![file exists $projectFile] {	    
		return 0
	}
	OpenDocument $projectFile
	return [LocateJavaElement $qualifiedName]	
}

proc SynchronizeProjectFromEclipse { ackPort name projectLocation } {    
    Ack $ackPort
	set projectFile [file join $projectLocation $name].ttp
	if [file exists $projectFile] {
		OpenDocument $projectFile
	}
	set project [GetProjectId $name]
	if {0 == $project } {
		return 0
	}
	#UpdateModelProject $project "" 1	
	##################### Invokation of agent ###############
        set curProject [std::GetActiveProject]
        set model [std::GetModels -kind "U2" -project $curProject]
        set agent [u2::FindByGuid $model @TTDJavaModelCodeSync@Menu@UpdateAll]
        
        if { 0 == $agent } {
                std::Output "Failed to find Update Java agent \n"
                # failed to find agent
                return 0
        }
        set param {1}

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Update Java agent : $e\n"
        }
}

proc Ack { ackPort } {
	global env
	
	# save eclipsePort in temporary file
	# it is because of different tcl interpretators (for "Locate in Tau" and for "Locate in Eclipse")
	if {[info exist env(TEMP)]} {
		set filename [file join $env(TEMP) .EclipseIntegration_port]
	} elseif {[info exist env(TAUECLIPSEINT)]} {
		set filename [file join $env(TAUECLIPSEINT) .EclipseIntegration_port]
	} else {
		Output "No integration enviroment variables are declared. Please use TEMP in case of Windows and TAUECLIPSEINT otherwise.\n"
		return 0
	}
	
	set fileId [open $filename w+]
	puts -nonewline $fileId $ackPort
	close $fileId	

    if {0 == $ackPort } {
        return 0
    }     
    set eclipseSocket [socket localhost $ackPort]
	if {0 == $eclipseSocket} {
	    Output "socket is 0\n"
		return 0
	}
	putSock $eclipseSocket "AckFromTAU"	
	close $eclipseSocket	
}

#########################################################################
# Procedures related to the Locate Eclipse Executable dialog

proc OnBuildLocateEclipseDialog { dialog } {
  global eclipseTextField

  # Source file controls
  Label -name "Unable to locate the Eclipse executable. Please specify the location manually." -parent $dialog -x 10 -y 10 -w 355 -h 40
  Label -name "Eclipse executable:" -parent $dialog -x 10 -y 55 -w 180 -h 25
  Edit -variable editSource -name "" -parent $dialog -x 10 -y 80 -w 300 -h 23
  Button -name "..." -parent $dialog -x 320 -y 80 -w 23 -h 23 -command OnBrowseForEclipse

  Button -name "OK" -parent $dialog -x 203 -y 120 -w 60 -h 25 -command OnOK
  Button -name "Cancel" -parent $dialog -x 283 -y 120 -w 60 -h 25 -command OnCancel

  set eclipseTextField $editSource
}

proc OnBuildResetEclipseDialog { dialog } {
  global eclipseTextField

  # Source file controls
  Label -name "Please specify the location of the Eclipse executable." -parent $dialog -x 10 -y 10 -w 355 -h 40
  Label -name "Eclipse executable:" -parent $dialog -x 10 -y 55 -w 180 -h 25
  
  set optionsFile [file join [GetUserOptionsDir] eclipseOptions.ini]
  set eclipseExe [GetOptionValue $optionsFile ExecutablePath]
  		
  Edit -variable editSource -name $eclipseExe -parent $dialog -x 10 -y 80 -w 300 -h 23
  Button -name "..." -parent $dialog -x 320 -y 80 -w 23 -h 23 -command OnBrowseForEclipse

  Button -name "OK" -parent $dialog -x 203 -y 120 -w 60 -h 25 -command OnOK
  Button -name "Cancel" -parent $dialog -x 283 -y 120 -w 60 -h 25 -command OnCancel

  set eclipseTextField $editSource
}

proc OnOK {} {
	global eclipseTextField
	if [file exists [$eclipseTextField -name]] {
		CloseDialog
	} else {
		MessageDialog -name "IBM Rational Tau" -message "The specified file does not exist. Please specify an existing file." -style ok -icon stop
	}
}

proc OnCancel {} {
	CloseDialog
	return ""
}

proc OnBrowseForEclipse {} {
	global eclipseTextField
	variable filer
	if { $::tcl_platform(platform) == "windows" } {
	    set filter "Executable files (*.exe)|*.exe|All files (*.*)|*.*||"
	} else {
	    set filter "All files (*.*)|*.*||"
	} 
	set newFile [FileOpenDialog -filter $filter]
	if {$newFile != 0} {
	    $eclipseTextField -name $newFile
	}
}

proc OnInitLocateEclipseDialog { dialog } {
}

proc OnCloseLocateEclipseDialog {} {
	global eclipseTextField
	return [$eclipseTextField -name]
}

#########################################################################
