package com.telelogic.tau.tor;

import java.util.ArrayList;
import com.telelogic.tau.tor.os.*;

public class Dispatcher
{	
	public Dispatcher(EventQueue eq)
	{
		m_eventQueue = eq;
	}
	
    // Associates a dispatchable with this dispatcher to make the dispatchable
    // receive events in the event queue of this dispatcher.
    public void add(Dispatchable d)
    {
    	m_dispatchables.add(d);
    	d.m_eventQueue = m_eventQueue;
    	d.m_dispatcher = this; 
    }

    public EventQueue getEventQueue()
    {
    	return m_eventQueue;
    }
    
    // Removes the association between a dispatchable and this dispatcher.
    // The dispatchable will no longer receive events from this dispatcher,
    // and events in the queue addressed to the dispatchable are removed.
    public boolean remove(Dispatchable d) 
    {
    	if (d == null || d.m_dispatcher != this)
    		return false;
    	
    	m_dispatchables.remove(d);
    	d.m_eventQueue = null;
    	d.m_dispatcher = null;
    	
    	return true;
    }

    // As long as the event queue is not empty, retrieves events from the 
    // queue and executes them in their receivers. Returns when the queue is
    // empty.
    public void run()
    {
    	if (m_eventQueue == null)
    		return;
    	
    	while (!m_eventQueue.empty() || !m_timerQueue.empty())
    		step();
    }

    // Retrieves one event from the queue and executes it in the receiver
    // stored in the event.
    public void step()
    {
    	if (m_eventQueue == null)
    		return;
    	
    	Time now = new Time();
    	Time time = new Time();
    	while (true)
    	{
    		Event event = null;
    		if (!m_timerQueue.empty())
    		{
    			now.setNow();
    			time.set(m_timerQueue.getTime());
    			
    			// Generate events for all timers in the timer queue that have expired
    			while (time.compareTo(now) <= 0) // time <= now
    			{
    				TimerObject timerObject = m_timerQueue.get();
    				if (timerObject != null)
    					m_eventQueue.insert(timerObject.event());
    				
    				if (m_timerQueue.empty())
    					break;
    				
    				time.set(m_timerQueue.getTime());
    			}
    			
				if (m_eventQueue.empty())
				{
					// Wait until the time of the next timer to expire
					event = m_eventQueue.get(time); 
					if (event == null)
						continue; // Timeout!
				}
				else
				{
					event = m_eventQueue.get();					
				}
			}		
    		else
    		{
    			event = m_eventQueue.get();
    		}
    		    		
    		if (event == null)
    			break; // No more events to process
    		
    		if (InstanceManager.instance().isValidExecutor(event.m_receiver))
    		{
    			EventExecutor executor = InstanceManager.instance().getExecutorFromId(event.m_receiver);
    			Dispatchable.EventAction ea = executor.execute(event);
    		}
    		break;
    	}
    }

    ///////////////////////////////////////////////////////////////////////////////////////////////
    
    // The Timer queue of this dispatcher.
    public TimerQueue m_timerQueue = new TimerQueue();
    
    // The event queue of this dispatcher.
    public EventQueue m_eventQueue = new EventQueue();
    
    // The list of dispatchables managed by this dispatcher.
    public ArrayList<Dispatchable> m_dispatchables = new ArrayList<Dispatchable>();
}
