package com.telelogic.tau.tor;

import java.util.HashMap;

// Singleton, i.e there must only exist one instance of the InstanceManager in an entire application.
public class InstanceManager
{
	// ObjectId is represented by a long integer	
	
	// Executor

    public synchronized boolean isValidExecutor(long id)
    {
    	return m_executors.containsKey(id);
    }

    public synchronized long getIdFromExecutor(EventExecutor executor)
    {
    	return executor.getExecutorId();
    }
    
    public synchronized EventExecutor getExecutorFromId(Long id)
    {
    	EventExecutor ee = m_executors.get(id.longValue());
    	return ee;
    }

    public synchronized long addExecutor(EventExecutor executor)
    {
    	long id = newId();
    	m_executors.put(id, executor);
    	return id;
    }

    public synchronized void removeExecutor(EventExecutor executor)
    {
    	m_executors.remove(executor.getExecutorId());
    }
    
    public synchronized void removeExecutor(long id)
    {
    	m_executors.remove(id);
    }

    // Receiver

    public synchronized boolean isValidReceiver(long id)
    {
    	return m_receivers.containsKey(id);
    }

    public synchronized long getIdFromReceiver(EventReceiver receiver)
    {
    	return receiver.getReceiverId();
    }
    
    public synchronized EventReceiver getReceiverFromId(Long id)
    {
    	EventReceiver er = m_receivers.get(id.longValue());
    	return er;
    }

    public synchronized long addReceiver(EventReceiver receiver)
    {
    	long id = newId();
    	m_receivers.put(id, receiver);
    	return id;
    }

    public synchronized void removeReceiver(EventReceiver receiver)
    {
    	m_receivers.remove(receiver.getReceiverId());
    }
    
    public synchronized void removeReceiver(long id)
    {
    	m_receivers.remove(id);
    }

    // Returns the one and only instance manager
    public static InstanceManager instance()
    {
    	return m_instanceManager;
    }

    protected long newId()
    {
    	return m_objectIdSeed++;
    }

	private static InstanceManager m_instanceManager = new InstanceManager(); 
	private static long m_objectIdSeed = 1;

    private HashMap<Long, EventExecutor> m_executors = new HashMap<Long, EventExecutor>(); // ExecutorMap
    private HashMap<Long, EventReceiver> m_receivers = new HashMap<Long, EventReceiver>(); // ReceiverMap
}
