package com.telelogic.tau.tor;

import java.util.LinkedList;

public class Port
{
	// Constructor. Attaches the port to the specified owner dispatchable class,
	// as either an in- or outgoing port.
	public Port(DispatchableClass owner, PortKind portKind)
	{
	    if (owner != null)
	    {
	        if (portKind == PortKind.InPort)
	            targetBehavior = owner;
	        else
	            owner.addOutPort(this);
	    }
	}
	
	// Connects the supplied port to this port using the supplied connector.
	public void attach(Port port, Connector connector)
	{
	    portList.add(port); 
	}

	// Sends an event through this port. The event is sent to one of the 
	// connected ports or to the target behavior. 
	public boolean send(Event event)
	{
	    // Note: We do not support more than one connected connector per port
	    boolean result = false;

	    // If port list is not empty, send the event through the 
	    // first port in the port list.
	    if (!portList.isEmpty()) 
	    {    
	        Port p = portList.getFirst();
	        result = p.send(event);
	    }
	    else if (targetBehavior != null) 
	    {
	        // Otherwise send the event to the target behavior
	        result = targetBehavior.receive(event);
	    }

	    return result;
	}

	public enum PortKind { InPort, OutPort };
	
	// A list of ports connected with this port, i.e. potential targets of 
	// events sent through this port.
	protected LinkedList<Port> portList = new LinkedList<Port>();

	// A dispatchable class that can receive events sent through this port.
	public DispatchableClass targetBehavior;
}
