package com.telelogic.tau.tor;

import com.telelogic.tau.tor.os.*;
import com.telelogic.tau.tor.os.thread.*;

public class ThreadedDispatcher
{
    // Associates a dispatchable with this dispatcher to make the dispatchable
    // receive events in the event queue of this dispatcher.
    public void add(Dispatchable d)
    {
    	m_data.addDispatchable(d);
    }
    
    // Launch a new thread and let it start dispatching the Dispatchables that have been added.
    public void launch()
    {
    	new DispatcherBehavior(m_data).start();
    }

    public boolean endThread()
    {
    	return m_data.endThread();
    }

    public boolean endThread(Time timeout)
    {
    	return m_data.endThread(timeout);
    }

    // 
    protected DispatcherData m_data = new DispatcherData();
    
    protected static class DispatcherBehavior extends Behaviour
    {
    	public DispatcherBehavior(DispatcherData data)
    	{
    		this.m_data = data;
    	}
    	
    	@Override
		public int execute()
		{
			m_data.setBehaviourId();
	        while (!m_data.threadShallEnd())
	            m_data.stepDispatcher();
	        m_data.threadIsDone();
	        
	        return 0;
		}
    	
		protected DispatcherData m_data;
    }
    
    protected static class QuitEvent extends Event {}

    protected static class DispatcherData 
    {    
    	public DispatcherData()
    	{
    		m_threadDone.init();	
    	}
    	
    	public synchronized void addDispatchable(Dispatchable d)
    	{
    		m_dispatcher.add(d);
    	}
    	
		public synchronized void setBehaviourId()
		{
			m_behaviourId = com.telelogic.tau.tor.os.thread.Utilities.getCurrentThreadId();
		}

		public void threadIsDone()
		{			
			synchronized(this)
			{
				if (!m_useSemaphore)
					return;
			}
			m_threadDone.put(); // Notify caller that the thread is done!
		}

		public void stepDispatcher()
		{
			m_dispatcher.step();
		}	
		
	    public boolean endThread()
	    {
	        postEndThread();
	        if (m_behaviourId == com.telelogic.tau.tor.os.thread.Utilities.getCurrentThreadId())
	        {
	            synchronized(this)
	            {
	            	m_useSemaphore = false;
	            }
	            return true;
	        }

	        return m_threadDone.get() == Status.StatusOK;
	    }

	    public boolean endThread(Time timeout)
	    {
	        postEndThread();
	        if (m_behaviourId == com.telelogic.tau.tor.os.thread.Utilities.getCurrentThreadId())
	        {
	            synchronized(this)
	            {
	            	m_useSemaphore = false;
	            }
	            return true;
	        }

	        return m_threadDone.get(timeout) == Status.StatusOK;
	    }

	    void postEndThread()
	    {
	        synchronized(this) 
	        {	            
	            m_threadShallEnd = true;
	        }

	        // Insert an event into the underlying event queue to
	        // wake the dispatcher thread if it is blocking.
	        m_queue.insert(new QuitEvent());
	    }

		public synchronized boolean threadShallEnd()
		{
			return m_threadShallEnd;
		}
    	
		private boolean m_threadShallEnd = false;
		private boolean m_useSemaphore = true;
		private long m_behaviourId = 0;
		private Semaphore m_threadDone = new Semaphore();
		private ThreadSafeEventQueue m_queue = new ThreadSafeEventQueue();
		private Dispatcher m_dispatcher = new Dispatcher(m_queue);
    }
}
