package com.telelogic.tau.tor;

import com.telelogic.tau.tor.os.*;

public class TimerObject implements Comparable<TimerObject>
{
	public TimerObject(Dispatchable owner)
	{
		m_owner = owner;
	}

    public void set(Time time, TimerEvent timerEvent)
    {
        if (isActive())
            reset();

        Dispatcher dispatcher = m_owner.m_dispatcher;
        if (dispatcher == null)
        	return;

        m_timerQueue = dispatcher.m_timerQueue;
        if (m_timerQueue == null)
        	return; // Associated dispatcher has no timer queue!

        timerEvent.m_receiver = InstanceManager.instance().getIdFromExecutor(m_owner);
        timerEvent.m_timer = this;
        m_t.set(time);
        m_timerEvent = timerEvent;
        m_timerQueue.insert(this);
    }
    
    public void reset()
    {
    	if (m_timerEvent != null)
    	{
    		softReset();
    	}
    }
    
    public boolean isActive() 
    { 
    	return m_timerEvent != null; 
    }

    public TimerEvent event() 
    { 
    	return m_timerEvent; 
    }

    protected boolean softReset()
    {
    	boolean b = false;
    	if (m_timerEvent != null)					// timer is active 
    	{					
    		if (m_timerQueue != null)				// .. has not expired 
    		{				
    			m_timerQueue.remove( this );
    			b = true;
    		}
    		else 									// .. has expired
    		{						
    			b = m_owner.m_eventQueue.remove(m_timerEvent);
    		}
    		m_timerEvent = null;
    	}
    	return b;
    }

    protected Dispatchable m_owner;
    protected TimerEvent m_timerEvent;
    protected TimerQueue m_timerQueue;
    Time m_t = new Time();

    public int compareTo(TimerObject t) {
        return m_t.compareTo(t.m_t);  //To change body of implemented methods use File | Settings | File Templates.
    }
}
