package com.telelogic.tau.tor.os;

import java.util.Date;

public class Time implements Comparable<Time>
{
	// Construct a time for zero
	public Time()
	{
		set(0);
	}
	
	// Construct a time for 'd' time units.
	public Time(double d)
	{
		set(d);
	}

	// Construct a time from a Java date
	public Time(Date d)
	{
		set(d);
	}

	// Construct a time by copying another date.
	public Time(Time t)
	{
		set(t);
	}

	// Construct a relative time a - b
	public Time(Time a, Time b)
	{
		set(a.to_double() - b.to_double());
	}
	
	// Set the time to 'd' time units.
	public Time set(double d)
	{
		m_rep = new Date(new Double(d * unit * 1000).longValue()); // Date constructor expects milliseconds
		return this;
	}
	
	// Set the time to Date d.
	public Time set(Date d)
	{
		m_rep = d;
		return this;
	}
	
	// Set the time to the time t.
	public Time set(Time t)
	{
		return set(t.m_rep);
	}

	// Get the value of the time as the number of time units.
	public double to_double()
	{
		return (m_rep.getTime() / 1000) / unit;
	}
	
	// Get the value of the time as the number of milliseconds.
	public long to_ms()
	{
		return m_rep.getTime();
	}
	
	// Return the current time
	public static Time now()
	{
		return new Time(new Date());
	}
	
	// Set the time to the current time.
	public Time setNow()
	{
		return set(now());
	}
	
	// Reset the time to zero.
	public Time zero()
	{
		set(0);
		return this;
	}
	
	// Sets the unit of time.
    // The time unit is specified in seconds. 
	public static void setUnit(double seconds)
	{
		unit = seconds;
	}
	
	// Comparable implementation
	public int compareTo(Time t)
	{
		return m_rep.compareTo(t.m_rep);
	}
	
	//////////////////////////////////////////////////////////////////////
	
	// Representation of a Time value
	protected Date m_rep;
	
	// The number of seconds for one unit of time.
	protected static double unit = 1;
}
