# Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:17 PM CET 2014
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# Global variables
set showBindingErrorsOption 0
set autoCheckOption 0

proc StoreAutoCheckOptions { } {
    global showBindingErrorsOption
    global autoCheckOption
    set showBindingErrorsOption [GetOption ShowBindingErrors]
    set autoCheckOption [GetOption AutoCheck]
    SetOption ShowBindingErrors 0
    SetOption AutoCheck 0
}

proc RestoreAutoCheckOptions { } {
    global showBindingErrorsOption
    global autoCheckOption
    SetOption ShowBindingErrors $showBindingErrorsOption
    SetOption AutoCheck $autoCheckOption
}

proc IsProfileLoaded { session } {
    if {(0 == $session) || (0 == [FindByGuid $session @JavaView@javaPackage])} {
        return 0
    } else {
        return 1
    }
}

# Checks if a class (or interface) has is the target of a <<manifest>> dependency
proc HasManifestDependency {class} {
  foreach d [GetReverseReferences $class Supplier] {
    set m [GetTaggedValue $d "manifest (. .)"]
    if [IsElementRef $m] {
      return 1
    }
  }
  return 0
}

proc CopyDependencies { e1 e2 } {
    # Move all imports from class to artifact
    set deps [u2::GetEntities $e1 ClientDependency]
    foreach e1Dep $deps {
                if {0 == $e1Dep} {
                    continue
                }
        set import [u2::GetTaggedValue $e1Dep "import (. .)"]
        set access [u2::GetTaggedValue $e1Dep "access (. .)"] 
        if {(0 != $import) || (0 != $access)} {
            set supplier [u2::GetEntity $e1Dep Supplier]
            if { [IsElementRef $supplier] && ([IsJavaElement $supplier] || [u2::IsKindOf $supplier Package]) } {
                set e2Dep [u2::Create $e2 Dependency]
                if [IsJavaElement $supplier] {
                    set stereotype [FindByGuid [GetSession $e1] "@Predefined@access"]
                } else {
                    set stereotype [FindByGuid [GetSession $e1] "@Predefined@import"]
                } 
                u2::SetEntity $e2Dep StereotypeInstance [u2::CreateInstance $stereotype]
                # Note: u2::SetValue doesn't work for unbound things
                u2::SetValue $e2Dep Supplier [u2::GetValue $e1Dep Supplier]
            }
        }
    }
}

# Returns an artifact based on a <<manifest>> dependency
proc GetArtifact {class} {
    if { $class == 0 } {
        return 0
    }
    foreach d [GetReverseReferences $class Supplier] {
        set m [GetTaggedValue $d "manifest (. .)"]
        if { [IsElementRef $m] } {
            set a [u2::GetEntity $d Client]
            if { [IsElementRef $a] } {
                if { [u2::GetTaggedValue $a "javaFile (. .)"] != 0 } {
                    return $a
                }
            }
        }
    }
    return 0
}

# Creates and returns a new java file artifact in the given context with the given name.
# Manifest dependencies are created to all elements in the manifestedElements list.
proc CreateManifestingArtifact { context name manifestedElements } {
    # Create artifact
    if {0 != $context} {
        set artifact [u2::Create $context Artifact]
    } else {
        set artifact [u2::New [GetActiveSession] Artifact]
    }
    u2::SetValue $artifact Name $name

    # Apply javaFile stereotype on artifact
    set javaFileStereotype [FindByGuid [GetSession $context] "@TTDFileModel@javaFile"]
    set javaFileStereotypeInstance [ApplyStereotype $artifact $javaFileStereotype]
        SetJavaFile $artifact $name

    # Set up manifest dependencies
    set manifestStereotype [FindByGuid [GetSession $context] "@Predefined@manifest"]
    foreach manifested $manifestedElements {
        set dep [u2::Create $artifact Dependency]
        set si [ApplyStereotype $dep $manifestStereotype]
        # Note: u2::SetValue doesn't work here
        SetA $dep Supplier [u2::GetValue $manifested Name]
    }
    return $artifact
}


# Returns the java directory of a package, creating it if it doesn't exist.
proc CreatePackageDirectory { pkg } {

  # Get output folder
  set dir [GetJavaDirectory $pkg]
  # Create directory if it doesn't exist
  if { ("" != $dir) && ![file exists $dir]} {
    output "\nCreating folder: $dir ... "
    file mkdir $dir
    output "Done\n"
  }
  return $dir
}

# Returns the fully qualified Java name of an element, e.g. X.Y.Z
proc GetJavaQualifiedName { element } {
    if { (0 == $element) || [IsUnnamedPackage $element] } {
        return ""
    }
    if { ![u2::IsKindOf $element Definition] } {
		# only Definitions have Name 
		return ""
    }
    set result [u2::GetValue $element Name]
    
    set owner [u2::GetEntity $element Owner]
    while {( 0 != $owner) && ![u2::IsKindOf $owner Session] && ![IsUnnamedPackage $owner] } {
        if { [u2::IsKindOf $owner Definition] && ![IsConstructor $owner] } {
            set result [u2::GetValue $owner Name].$result
        }
        set owner [u2::GetEntity $owner Owner]      
    }
    return $result
}

# Returns all elements manifested by the supplied artifact
proc GetManifested {artifact} {
  set deps [u2::GetEntities $artifact clientDependency]
  set manifested ""
  foreach dep $deps {
    if {0 == $dep} {
      continue
    }
    set manifest [GetTaggedValue $dep "manifest (. .)"]
    if [IsElementRef $manifest] {
      set supplier [u2::GetEntity $dep Supplier]
      if [IsElementRef $supplier] {
        lappend manifested $supplier
      }
    }
  }
  return [lsort -unique $manifested]
}

# Used to get or set the java syntax for an element.
# If the optional parameter enabled is left out it returns true if java syntax 
# is enabled and false otherwise. If a boolean value is supplied, the status
# is set to the supplied value
proc JavaSyntax { element {enabled ""} } {
  if {![IsElementRef $element] || (![u2::IsKindOf $element Package] && ![u2::IsKindOf $element PresentationElement]) } {
    return 0
  }
  if {("" != $enabled) && (0 != $enabled) && (1 != $enabled)} {
    error "\nIllegal value. Argument 'enabled' must be 0 or 1\n"
  }
  if [u2::IsKindOf $element Package] {
    set stereotypeName "packageLanguage"
    set taggedValueName "modelID"
  } else {
    set stereotypeName "presentationLanguage"
    set taggedValueName "languageID"
  }

  if {0 == [GetTaggedValue $element "$stereotypeName (. .)"]} {
    set applied 0
  } else {
    set applied 1
  }
  
  if { "" == $enabled } {
    if $applied {
        set value [GetTaggedValue $element "$stereotypeName (. $taggedValueName .)"]
        if { [u2::GetMetaClassName $value] == "IntegerValue" } {
			set value [u2::GetValue $value Value]
        } else {
			set value ""
        }
         
        if { $value == 2 } {
            return 1
        } else {
            return 0
        }
    }
    return 0
  } elseif {0 == $enabled } {
    if $applied {
        #Delete stereotype instance
        DeleteStereotypeInstance $element "$stereotypeName (. .)"
    }
    return 0
  } elseif {1 == $enabled} {
    if !$applied {
        #Create stereotype instance
        set s [FindByGuid [GetSession $element] "@TTDLineStyles@$stereotypeName"]
        set si [u2::CreateInstance $s]
        u2::SetEntity $element hiddenStereotypeInstance $si
    }
        # u2::Bind $element 
    u2::SetTaggedValue $element "$stereotypeName (. $taggedValueName .)" 2
    return 1
  }
}

proc GetTopLevelType {element} {
    if [IsJavaElement $element] {
        set owner [u2::GetEntity $element Owner]
        if [u2::IsKindOf $owner Package] {
            return $element
        } else {
            return [GetTopLevelType $owner]
        }
    } else {
        return 0
    }
}

# Returns the closest enclosing package of an element. Returns 0 if no
# enclosing package is found
proc GetEnclosingPackage { element } {
    set owner [u2::GetEntity $element Owner]
    if { (0 != $owner) && ![u2::IsKindOf $owner Session] } {
        if [u2::IsKindOf $owner Package] {
            return $owner
        } else {
            return [GetEnclosingPackage $owner]
        }
    }
    return 0    
}

# Returns the top-level package of an element. Returns 0 if no
# top-level package is found
proc GetTopLevelPackage { element } {
    set owner [u2::GetEntity $element Owner]
    set current $element
    while { (0 != $owner) && ![u2::IsKindOf $owner Session]} {
        set current $owner
        set owner [u2::GetEntity $current Owner]
    }
    return $current 
}

proc GetRootDirectory { pkg } {
    set charString [u2::GetTaggedValue $pkg "javaPackage (. path .)"]
    if {(0 == $charString) || ![u2::IsKindOf $charString CharstringValue]} {
        return ""
    } 
    return [u2::GetValue $charString Value]
}

proc GetFileFromArtifact { artifact } {
    if ![u2::IsKindOf $artifact Artifact] {
        return ""
    }
    set pathString [u2::GetTaggedValue $artifact "file (. path .)"]
    if {(0 == $pathString) || ![u2::IsKindOf $pathString CharstringValue]} {
        return ""
    } 
    return [u2::GetValue $pathString Value]
}

proc IsJavaElement { element } {
  if {[u2::IsKindOf $element Class] || [u2::IsKindOf $element Interface] || [u2::IsKindOf $element Stereotype] || [u2::IsKindOf $element "DataType"]} {
	# added Stereotype (annotation) and DataType (enum)
     return 1
  }
  return 0
}

proc IsConstructor { element } {
 if {[u2::IsKindOf $element Operation] && ([u2::GetValue $element OperationKind] == "OkConstructor")} {
    return 1
 }
 return 0
}


# Checks if a model entity is mapped to a Java file
# Based on the RootPackage stereotype
proc HasJavaFileMapping {e} {

  set target $e
  if {(0 == $target) || ("" == $target)} {
    return 0
  }

  # Get the model element from presentation elements
  # Ignore diagrams since Package is the ModelElement for diagrams
  if {[u2::IsKindOf $e PresentationElement] && ![u2::IsKindOf $e Diagram]} {
    set target [u2::GetEntity $e ModelElement]
    if {(0 == $target) || ("" == $target)} {
      return 0
    }
  }

  if {[IsJavaElement $target] || [u2::IsKindOf $target Artifact]} {

    set target [GetEnclosingPackage $target]
    if {(0 == $target) || ("" == $target)} {
      return 0
    }
  }

  if {[u2::IsKindOf $target Package] && [IsJavaPackage $target]} {
    if { "" != [GetJavaDirectory $target]} {
        return 1
    }
  }
  return 0
}

# Checks if owner of entity (attribute or operation) is mapped to a Java file
# Based on the RootPackage stereotype
proc HasOwnerJavaFileMapping {e} {
  set target $e
  if {(0 == $target) || ("" == $target)} {
    return 0
  }
  
  # Get the model element from presentation elements
  # Ignore diagrams since Package is the ModelElement for diagrams
  if {[u2::IsKindOf $e PresentationElement] && ![u2::IsKindOf $e Diagram]} {
    set target [u2::GetEntity $e ModelElement]
    if {(0 == $target) || ("" == $target)} {
      return 0
    }
  }
  
  set owner [u2::GetEntity $target Owner]
  return [HasJavaFileMapping $owner]
}

proc OpenJavaFiles { files } {
    set files [lsort -unique $files]
    foreach f $files {
        if [file exists $f] {
            OpenDocument $f
        } else {
            Output "\nNavigation failed"
            MessageDialog -name "IBM Rational Tau" -message "Unable to locate file $f" -style ok -icon warning
        }
    }
}

# Returns a list of files with the specified extension from the specified directory.
# By default the full path/name is returned, if 0 is passed as the last parameter only the file
# names are returned.
proc GetFilesFromDirectory { dir extension {includePath 1} } {
  set files ""
  if ![file exists $dir] {
    Output "\nWarning: Directory $dir does not exist."
    return $files
  }
  set files [glob -directory $dir -nocomplain "*$extension"]
  if { 0 == $includePath } {
    set fileNames ""
    foreach filePath $files {
        lappend fileNames [file tail $filePath]
    }
    return $fileNames
  }
  return $files
}    

# Returns all java files representing the package
# Files in any unnamed package within the package are also included
proc GetJavaFilesFromPackage { pkg {includePath 0}} {
   set javafiles ""
   set members [u2::GetEntities $pkg OwnedMember]
   foreach element $members {
      if {0 == $element} {
        continue
      }
      if {[IsJavaElement $element] || [u2::IsKindOf $element Artifact]} {
         lappend javafiles [GetJavaFile $element $includePath]
      }
      if {[u2::IsKindOf $element Package] && [IsUnnamedPackage $element]} {
         output "\nERROR: Unnamed packages can only exist at the top level"
     set javafiles [concat $javafiles [GetJavaFilesFromPackage $element]]
      }
   }
   return [lsort -unique $javafiles]
}    

# Returns all .class files represented in a namespace, typically a package
proc old_GetClassFiles { namespace {includePath 0}} {
    set classFiles ""
    if [IsJavaElement $namespace] {
    # class or interface
        set path ""
        if { $includePath } {
             # Get the path of the .java file
             set path [file dirname [GetJavaFile $namespace $includePath]]
        }
        set className [GetClassSeparatedName $namespace]
        lappend classFiles [file join $path $className.class]
        # search anonymous classes : className$*.class
        set anonymousClasses [glob -directory $path -nocomplain "$className\$*.class"]
        foreach anonymousClass $anonymousClasses {
            lappend classFiles [file join $path [file tail $anonymousClass]]
        }
    } else {
        if [IsJavaPackage $namespace] {
            set members [u2::GetEntities $namespace OwnedMember]
            foreach member $members {
                    if {0 == $member} {
                        continue
                    }
                    set classFiles [concat $classFiles [GetClassFiles $member $includePath]]
            }
        }
    }
    return [lsort -unique $classFiles]
}

proc GetFiles {path {mask {"*.class"}}} {
    set fileList {}
    foreach m $mask {
        foreach f [glob -nocomplain -type f [file join $path $m]] {
            lappend fileList $f
        }
    }
    foreach d [glob -nocomplain -type d [file join $path *]] {
        foreach f [GetFiles $d $mask] {
            lappend fileList $f
        }
    }
    return $fileList
}

# Returns a $ separated name matching the .class file name
proc GetClassSeparatedName { element } {
    if ![IsJavaElement $element] {
        return ""
    }
    set elementName [u2::GetValue $element Name]
    set owner [u2::GetEntity $element Owner]
    while {( 0 != $owner) && ![u2::IsKindOf $owner Session] && [IsJavaElement $owner] } {
        set elementName [u2::GetValue $owner Name]\$${elementName}
        set owner [u2::GetEntity $owner Owner]      
    }
    return $elementName
}

# Returns the name of the java file representing the element
proc GetJavaFile { element { includePath 0} } {
    set filePath ""
    if { 0 == $element } {
        return ""
    }
    if {[IsJavaElement $element]} {
        set topLevelElement [GetTopLevelType $element]
        set artifact [GetArtifact $topLevelElement]
        if {0 != $artifact} {
            set filePath [GetFileFromArtifact $artifact]
        } else {     
            set filePath "[u2::GetValue $topLevelElement Name].java"
        }
    } elseif [u2::IsKindOf $element Artifact] {
        set filePath [GetFileFromArtifact $element]
    } else {
        while { $element != 0 && ![IsJavaElement $element] } {
            set element [u2::GetEntity $element Owner]
        }
        set topLevelElement [GetTopLevelType $element]
        set artifact [GetArtifact $topLevelElement]
        if {0 != $artifact} {
            set filePath [GetFileFromArtifact $artifact]
        } else {     
            set filePath "[u2::GetValue $topLevelElement Name].java"
        }
    }

    if { $filePath == "" } { return "" } 

    if { 0 == $includePath } {
        return [file tail $filePath]
    }

    set dir [GetTargetDir [GetBuildArtifact $element]]
    return [file join $dir $filePath]
}

# Returns the full directory path of a java element/package using the javaPackage stereotype.
# Returns "" if no package has the path tagged value set.
proc GetJavaDirectory { element } {
    if { 0 == $element } {
        return ""
    }   
    if [u2::IsKindOf $element Package] {
        set pkg $element
    } else {
        set pkg [GetEnclosingPackage $element]
    }
    if ![IsJavaPackage $pkg] {
        return ""
    }

    set project [std::GetProject $pkg]
    set projectDir [GetProjectDir $project]

    set rootDirectory [GetRootDirectory $pkg]
    if { "" != $rootDirectory } {
        if { [file pathtype $rootDirectory] != "absolute" } {
            if { "" != $projectDir } {
                return [file normalize [file join $projectDir $rootDirectory]]
            }
            return $rootDirectory
        } else {
            return $rootDirectory
        }
    }
    set directory [u2::GetValue $pkg Name]
    set owner [u2::GetEntity $pkg Owner]
    while { ![u2::IsKindOf $owner Session] || ("" != [GetRootDirectory $owner]) } {
        if [u2::IsKindOf $owner Package] { 
            if ![IsJavaPackage $owner] {
                if { "" != $projectDir } {
                    return [file normalize [file join $projectDir $directory]]
                }
                return $directory
            } else {
                set ownerRoot [GetRootDirectory $owner]
                if { "" != $ownerRoot} {
                    if { [file pathtype $ownerRoot] != "absolute" } {
                        if { "" != $projectDir } {
                            return [file normalize [file join $projectDir $ownerRoot $directory]]
                        }
                        return [file join $ownerRoot $directory]
                    } else {
                        return [file normalize [file join $ownerRoot $directory]]
                    }
                } else {
                    set name [u2::GetValue $owner Name]
                    set directory [file join $name $directory]
                }
            }               
        }
        set owner [u2::GetEntity $owner Owner]
    }
    if [u2::IsKindOf $owner Session] {
        return ""
    }
    return $directory
}

# Returns the session of the active project.
proc GetActiveSession {} {
   set activeProject [std::GetActiveProject]
   if { "" != $activeProject } {
   	return [lindex [std::GetModels -kind U2 -project $activeProject] 0]
   } else { 
   	return ""
   }
}

# Returns the session of the supplied element using the Session metafeature.
# Note that not all metaclasses has this feature.
proc GetSession { element } {
    if [catch {set session [u2::GetEntity $element Session]}] {
        set session 0
    }
    return $session 
}

proc IsElementRef {e} {
  if {($e == 0) || ($e == "")} {
    return 0
  }
  return 1
}

proc DeleteStereotypeInstance { e pattern} {
  set cont 1
  while {$cont} {
    set cont 0
    set si [GetTaggedValue $e $pattern]
    if [IsElementRef $si] {
      Delete $si
      set cont 1
    }
  }
}

# Returns 1 if the package is an unnamed package (has the unnamedPackage stereotype applied).
# Otherwise returns 0.
proc IsUnnamedPackage {pkg} {
  if {! [IsElementRef $pkg]} {
    return 0
  }
  
  if { ![u2::IsKindOf $pkg Package] } {
	return 0
  }
  
  set si [GetTaggedValue $pkg "'@JavaSDK@unnamedPackage' (. .)" -guid]
  if {$si != 0} {
    return 1
  }
  return 0
}

# Returns the unnamed package of a session, creating it if it doesn't exist.
# Note that there may only be one unnamed package in a session.
proc GetUnnamedPackage { session {createIfNeeded 1}} {
    if { 0 == $session } {
        return 0
    }
    set unnamedPackageStereotype [u2::FindByGuid $session "@JavaSDK@unnamedPackage"]
    if { 0 != $unnamedPackageStereotype } {
        # NOTE: New API causes crash sometimes
        #set refs [u2::GetReferingEntities $unnamedPackageStereotype InstanceOf]
        set refs [GetReverseReferences $unnamedPackageStereotype InstanceOf]
        if { [llength $refs] == 0 } {
            if !$createIfNeeded {
                return 0
            }
            set resource [GetResource $session]
            if { 0 == $resource } {
                output "\nError: Unable to create resource!"
                return 0
            }
            set defaultPackage [u2::Create $resource Package]
            u2::SetValue $defaultPackage Name "defaultPackage"
            set s [u2::FindByGuid $session "@JavaSDK@unnamedPackage"]
            u2::SetEntity $defaultPackage StereotypeInstance [u2::CreateInstance $s]
            set jpStereotype [u2::FindByGuid $session "@JavaView@javaPackage"]
            u2::SetEntity $defaultPackage StereotypeInstance [u2::CreateInstance $jpStereotype]
            # u2::Bind $defaultPackage
            JavaSyntax $defaultPackage 1
        } else {
            set defaultPackage  [u2::GetEntity [lindex $refs 0] InstanceExprOwner]
        }
        return $defaultPackage
    } else {
        return 0
    }
}

proc IsJarPackage { pkg } {
    if {[IsElementRef $pkg] && [u2::IsKindOf $pkg Package]} {
        set si [GetTaggedValue $pkg "'@JavaView@jarPackage' (. .)" -guid]
        if {$si != 0} {
            return 1
        }
    }
    return 0
}

# Returns true if the element is contained in a jar package false otherwise
proc InJar { element } {
    if { 0 == $element } {
        return 0
    }

    if [u2::IsKindOf $element Package] {
        set pkg $element
    } else {
        set pkg [GetEnclosingPackage $element]
    }

    if { 0 != $pkg } {
        return [IsJarPackage $pkg]
    }
    return 0
}

# Returns true if the element is contained in a jar package false otherwise
proc InUnnamedPackage { element } {
    if { 0 == $element } {
        return 0
    }

    if [u2::IsKindOf $element Package] {
        set pkg $element
    } else {
        set pkg [GetEnclosingPackage $element]
    }

    if { 0 != $pkg } {
        return [IsUnnamedPackage $pkg]
    }
    return 0
}

proc IsJavaPackage {e} {
  if {$e != 0 } {
    set si [u2::GetTaggedValue $e "'@JavaView@javaPackage' (. .)" -interpretIdentsAsGuids true]
    if {$si != 0} {
      return 1
    }
  }
  return 0
}

# Returns 1 if the element is a jar file artifact, 0 otherwise
proc IsJarArtifact {e} {
  if {($e != 0) && [u2::IsKindOf $e Artifact] } {
    set si [u2::GetTaggedValue $e "'@TTDFileModel@jarFile' (. .)" -interpretIdentsAsGuids true]
    if {$si != 0} {
      return 1
    }
  }
  return 0
}

# Returns 1 if the supplied string is a valid java identifier name, 0 otherwise.
proc IsValidJavaIdentifier { identifier } {
  return [string is alnum $identifier]
}

###################  Import Model From Directory #####################

proc UpdateModelFromDirectory { dir } {

  if ![file exists $dir] {
    return
  }

  set session [GetActiveSession]
  
  if { "" == $session } {
  	return
  }

  # Insert any new java files in the directory into the unnamed package
  set javaFiles [GetFilesFromDirectory $dir ".java"]
  if { [llength $javaFiles] > 0 } {

    set unnamedPackage [GetUnnamedPackage $session]

    SetJavaDirectory $unnamedPackage $dir

    # Get files already represented in the unnamed package
    set existingFiles ""
    foreach element [GetCompilationUnitEntities $unnamedPackage] {
        if [u2::IsKindOf $element Artifact] {
        set tmpFile [GetJavaFile $element 1] 
        if { "" != $tmpFile } {
            lappend existingFiles $tmpFile
        }
        UpdateModelArtifact $element
    }
    }
    
    # Create elements from the new files not already present
    foreach javaFile $javaFiles {
      if ![PathInList $javaFile $existingFiles] {
        CreateElementsFromFile $javaFile $dir $unnamedPackage
      }
    }
  }

  # Create packages from directories not yet in model
  set directories [glob -directory $dir -nocomplain -type {d} "*"]
  if { [llength $directories] > 0 } {

    set rootPackages [u2::GetEntities $session Root]
    set existingRoots ""
    foreach rootPackage $rootPackages {
      if {0 == $rootPackage} {
        continue
      }
      if {[u2::IsKindOf $rootPackage Package]} {
        if [IsJavaPackage $rootPackage] {
          lappend existingRoots [GetJavaDirectory $rootPackage]
        }
      }
    }

    foreach directory $directories {
      set nativeDir [file native $directory]
      if ![PathInList $nativeDir $existingRoots] {
        set pkgName [file tail $nativeDir]
        set resource [GetResource $session]
        set pkg [u2::Create $resource Package]
        u2::SetValue $pkg Name $pkgName
    
    set s [u2::FindByGuid $session "@JavaView@javaPackage"]
    u2::SetEntity $pkg StereotypeInstance [u2::CreateInstance $s]
        
        SetJavaDirectory $pkg $nativeDir

        JavaSyntax $pkg 1
    
        UpdateModelPackage $pkg

    ApplyPackageStereotypeRecursively $pkg "@Predefined@openNamespace"
      } else {
        set root [FindRootWithPath $session $nativeDir]
    if { 0 != $root } {
            UpdateModelPackage $root    
    }
      }
    }
  }
}

# Returns the java root package in the session with a path
# matching the supplied path. Returns 0 if no matching root is found
proc FindRootWithPath { session path } {
    if { 0 == $session } {
        return 0
    }
    foreach rootPackage [u2::GetEntities $session Root] {
      if {0 == $rootPackage} {
        continue
      }
      if {[u2::IsKindOf $rootPackage Package]} {
        if [IsJavaPackage $rootPackage] {
          if [string equal [file normalize $path] [file normalize [GetRootDirectory $rootPackage]]] {
        return $rootPackage
      }
        }
      }
    }
    return 0
}

###################  Export Package to Directory #####################
proc ExportPackageToDirectory {dir pkg {silent 0} } {
  if [IsJarPackage $pkg] {
    set jarFile [GetJavaFile $pkg 1]
    Output "Copying file $jarFile to $dir ... "
    if [catch {file copy -force $jarFile $dir}] {
        Output "\nFailed. File might be open in another application.\n"
        return 0
    } else {
        Output " Done (1)\n"
        return 1
    }
  }

  set pkgDir [file join $dir [u2::GetValue $pkg Name]]
  if ![IsJavaPackage $pkg] {
    set s [FindByGuid [GetSession $pkg] @JavaView@javaPackage]
    set si [ApplyStereotype $pkg $s]
    set pkgDir [file nativename $pkgDir]
    set owner [u2::GetEntity $pkg Owner]
    if { (0 == $owner) || [u2::IsKindOf $owner Session] } {
        SetJavaDirectory $pkg $pkgDir
        JavaSyntax $pkg 1
        ApplyPackageStereotypeRecursively $pkg "@Predefined@openNamespace"
    }
  } else {
    set isUnnamedPackage [IsUnnamedPackage $pkg]
    if { $isUnnamedPackage } {
            set pkgDir $dir
    }
    if { [file normalize $pkgDir] != [file normalize [GetJavaDirectory $pkg]] } {
        MoveJavaPackage $pkg $pkgDir 1
    }
  }

  
  return [UpdateJavaPackage $pkg $silent]
}

###################  Update Java from Model #####################

proc UpdateJavaClass {class} {
  set topLevelClass [GetTopLevelType $class]
  if {![IsValidJavaIdentifier [u2::GetValue $topLevelClass Name]] || ![IsValidJavaIdentifier [u2::GetValue $class Name]]} {
    Output "\nElement name is not a valid Java identifer. Rename it and try again.\n"
    return
  }

  set pkg [u2::GetEntity $topLevelClass Container]
  if ![u2::IsKindOf $pkg Package] {
    Output "\nClass must be owned by package\n"
    return
  }

  # If correspnding artifact can be found update it instead
  if [HasManifestDependency $topLevelClass] {
    return [UpdateJavaArtifact [GetArtifact $topLevelClass]]
  }

  # Create artifact to represent compilation unit
  set artifactName [u2::GetValue $class Name].java
  set artifact [CreateManifestingArtifact $pkg $artifactName $topLevelClass]

  # Make sure associations/generalizations added by user are bound by name instead of GUID
  UpdateBindingClass $topLevelClass
  UpdateBindingDependencies $pkg 

  # Move imports from class and package to artifact
  CopyDependencies $class $artifact
  CopyDependencies $pkg $artifact

  # Add import of artifact to enclosing package
  set dep [u2::Create $pkg Dependency]
  set stereotype [u2::FindByGuid [GetSession $pkg] "@Predefined@import"]
  u2::SetEntity $dep StereotypeInstance [u2::CreateInstance $stereotype]
  u2::SetEntity $dep Supplier $artifact
  
  # Get output folder create new folder if needed
  set targetDir [CreatePackageDirectory $pkg]

  # Update file from model
  set fileName [file join $targetDir $artifactName]
  
  # Set path to java file  
  SetJavaFile $artifact $fileName

  output "Updating .java file: $fileName ... "
  set result [eval UpdateExternalFileFromModel "Java" {$fileName} $artifact]
    
  output "Done ($result)\n"
  return $result
}

proc UpdateJavaArtifact { artifact } { 
  set pkg [u2::GetEntity $artifact Container]
  if ![u2::IsKindOf $pkg Package] {
    Output "\nClass must be owned by package\n"
    return
  }

  # Create output folder if necessary
  CreatePackageDirectory $pkg

  set javaFile [GetJavaFile $artifact 1]
  output "Updating .java file: $javaFile ... "
  set manifested [GetManifested $artifact]
  foreach elem $manifested {
      if {![IsValidJavaIdentifier [u2::GetValue $elem Name]]} {
        Output "\nElement name '[u2::GetValue $elem Name]' is not a valid Java identifer. Rename it and try again.\n"
        return 0
      }
  }

  if {[llength $manifested] > 0} {
    set result [eval UpdateExternalFileFromModel "Java" {$javaFile} $artifact]
    output "Done ($result)\n"
  } else {
    output "Skipping artifact with no manifest relations.\n"
    set result 0
  }
  return $result
}

proc UpdateJavaPackage {pkg {silent 0}} {

    set packageName [u2::GetValue $pkg Name]
    if [IsJarPackage $pkg] {
    Output "\nSkipping package '$packageName' representing .jar file\n"
    return    
    }

    if ![IsValidJavaIdentifier $packageName] {
    Output "\nPackage name '$packageName' is not a valid Java identifer. Rename it and try again.\n"
    return
    }

    set isUnnamed [IsUnnamedPackage $pkg]

    # If package is a java package and path value is set, check if it matches
    # package name. If not, offer to rename file system directory.
    # NOTE: This is only done if the  path value of the javaPackage 
    # stereotype is set on this very package.
    set currentDir [GetJavaDirectory $pkg]
    if { (!$isUnnamed) && ("" != $currentDir) } {
    set oldName [file tail $currentDir]
    if { ![string equal $packageName $oldName] && [file exists $currentDir] } {
        set newDir [file join [file dirname $currentDir] $packageName]
        if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "The package '$oldName' has been renamed to '$packageName' in the model.\nDo you want to move the corresponding files in the file system?" -style yesno -icon information] == 6)} {
            output "\nMoving files related to package '[u2::GetValue $pkg Name]'.\n"
            MoveJavaPackage $pkg $newDir 0
        } 
    }
    }
    set dir [CreatePackageDirectory $pkg]

    foreach element [GetCompilationUnitEntities $pkg] {
        if [IsJavaElement $element] {
        UpdateJavaClass $element
    } elseif [u2::IsKindOf $element Artifact] {
        UpdateJavaArtifact $element
    }
    }

    foreach subPackage [u2::GetEntities $pkg SubPackage] {
       if {0 == $subPackage} {
          continue
       }
       if [IsJavaPackage $subPackage] {
           UpdateJavaPackage $subPackage $silent
       } else {
            if ![IsJarPackage $subPackage] {
            ExportPackageToDirectory $dir $subPackage
        }
       }
    }

   if !$isUnnamed {
   # Search for files in the file system that has no representation in the model and delete them 
     set files [GetFilesFromDirectory $dir ".java"]
     set modelFiles [GetJavaFilesFromPackage $pkg]
     foreach file $files {
        set fileName [file tail $file]
        if {[lsearch $modelFiles $fileName] == -1} {
      # Delete file
      if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "The file '$file' has no representation in the model.\nDo you want to delete it?" -style yesno -icon information] == 6)} {
        file delete $file
      } 
        }  
     }

   # Search for folders in the file system that has no representation in the model and delete them 
     set modelPackages ""
     set subPackages [u2::GetEntities $pkg SubPackage]
     foreach subpkg $subPackages {
        if {0 == $subpkg} {
          continue
        }
    if ![IsUnnamedPackage $subpkg] {
        lappend modelPackages [u2::GetValue $subpkg Name]
    }
     }
     set directories [glob -directory $dir -nocomplain -type {d} "*"]
     foreach newDir $directories {
        set newDirName [file tail $newDir]
        if {[lsearch $modelPackages $newDirName] == -1} {
        # Delete directory from file system
        if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "The directory '$newDir' has no representation in the model.\nDo you want to delete it?" -style yesno -icon information] == 6)} {
            file delete -force $newDir
        }
        }
     }
  }
}

# Update java files for entire project. Search for new directories in the
# supplied directory and add them as root packages.
proc UpdateJavaProject { project dir {silent 0} } {
    set sessions [std::GetModels -kind U2 -project $project]
    set existingRoots ""
    foreach session $sessions {
        set rootPackages [u2::GetEntities $session Root]
        foreach rootPackage $rootPackages {
                        if {0 == $rootPackage} {
                            continue
                        }
                if {[u2::IsKindOf $rootPackage Package]} {
                    if [IsJavaPackage $rootPackage] {
                    set javaDir [GetJavaDirectory $rootPackage]
                    # If a root package is a JavaPackage with no path value (unnamed packages excepted)
                    # Set the path value to the same as the name of the package
                    if {"" == $javaDir && ![IsUnnamedPackage $rootPackage]} {                       
                        set packageName [u2::GetValue $rootPackage Name]
                        u2::SetTaggedValue $rootPackage "javaPackage (. path .)" "\"$packageName\""
                    }
                    UpdateJavaPackage $rootPackage $silent
                    lappend existingRoots [GetJavaDirectory $rootPackage]
                    }
                }
        }
        set sourceLocations [GetSourceLocations $session]
        foreach location $sourceLocations {
            if [file exists $location] {
                set directories [glob -directory $location -nocomplain -type {d} "*"]
                foreach newDir $directories {
                    if ![PathInList [file native $newDir] $existingRoots] {
                        if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "The directory '$newDir' has no representation in the model.\nDo you want to delete it?" -style yesno -icon information] == 6)} {
                            file delete -force $newDir
                        }
                    }
                }
            } else {
                Output "Warning: '$location' is an invalid path."
            }
        }
    }
}

proc UpdateBindingClass { class } {
  foreach member [u2::GetEntities $class OwnedMember] {
        if {0 == $member} {
            continue
        }
    if [u2::IsKindOf $member Attribute] {
        BindByName $member "Type" 1
    } elseif {[u2::IsKindOf $member Class] || [u2::IsKindOf $member Interface]} {
        UpdateBindingClass $member
    }
  }
  foreach generalization [u2::GetEntities $class Generalization] {
        if {0 == $generalization} {
            continue
        }
    BindByName $generalization "Parent" 1
  }
  UpdateBindingDependencies $class
}

proc UpdateBindingDependencies { element } {
    set deps [u2::GetEntities $element ClientDependency]
    foreach dep $deps {
                if {0 == $dep} {
                    continue
                }
        set supplier [u2::GetEntity $dep Supplier]
        if { (0 != $supplier) && ([IsJavaElement $supplier] || [u2::IsKindOf $supplier Package]) } {
            # Make sure dependencies added by user are name bound instead of GUID bound
            BindByName $dep "Supplier" 1
        }
    }
}

###################  Update Model from Java #####################
proc SynchronizeClass { class {silent 0}} {

  set topClass [GetTopLevelType $class]

  set pkg [GetEnclosingPackage $topClass]
  if { 0 == $pkg } {
    Output "\nClass not owned by package\n"
    return
  }

  set dir [GetJavaDirectory $pkg]

  set artifact [GetArtifact $topClass]
  set fileName [GetJavaFile $topClass]

  set files [GetFilesFromDirectory $dir ".java" 0]
  if {[lsearch $files $fileName] == -1} {
    if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "'[u2::GetValue $class Name]' has no corresponding file in the file system.\nDo you want to delete the element from the model?" -style yesno -icon information] == 6)} {
        if {0 != $artifact} {
            set manifested [GetManifested $artifact]
            if {([llength $manifested] == 1) && ($manifested == $topClass)} {
                u2::Delete $artifact
            }
        }
        u2::Delete $topClass
    }
  } else {
     return [UpdateModelClass $dir $pkg $topClass $silent]
  }
}

# Deletes the artifact if it is not needed anymore.
# Imports are transferred to the manifestnig class.
proc SynchronizeArtifact { artifact } {
    set manifested [GetManifested $artifact]
    set fileNameNoExtension [file rootname [file tail [u2::GetValue $artifact Name]]]
    if {[llength $manifested] == 1} {
        set element [lindex $manifested 0]
        if {[u2::GetValue $element Name] == $fileNameNoExtension} {
            # Move all imports from artifact to class
            set deps [u2::GetEntities $artifact ClientDependency]
            foreach dep $deps {
                                if {0 == $dep} {
                                    continue
                                }
                if {0 != [u2::GetTaggedValue $dep "import (. .)"]} {
                    output "\nmoving $dep [u2::GetValue $dep Supplier]"
                    u2::SetEntity $dep Client $manifested
                }
            }
            # Delete imports from package to artifact
            foreach d [GetReverseReferences $artifact Supplier] {
                u2::Delete $d
            }
            # Delete artifact
            u2::Delete $artifact
        }
    }
}

proc UpdateModelClass {dir pkg class {silent 0}} {
    # If correspnding artifact can be found update it instead
    if [HasManifestDependency $class] {
        return [UpdateModelArtifact [GetArtifact $class] $silent]
    }

    # Create corresponding artifact
    set artifactName [u2::GetValue $class Name].java
    set artifact [CreateManifestingArtifact $pkg $artifactName $class]

    # Move imports from class and package to artifact
    CopyDependencies $class $artifact
    CopyDependencies $pkg $artifact

    # Update model from file
    # TODO: File will be incorrect here
    set file [GetJavaFile $artifact 1]
    set result [UpdateModelFromExternalFileWrapper $file $pkg $artifact]
        
    Output "Done ($result)\n"
    return $result
}


proc UpdateModelArtifact { artifact {silent 0}} {
  set container [u2::GetEntity $artifact Container]
  if ![u2::IsKindOf $container Package] {
    Output "\nArtifact not owned by package\n"
    return
  }

  set javaFile [GetJavaFile $artifact 1]

  if {("" != $javaFile) && ![file exists $javaFile] } {
    if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "Artifact '[u2::GetValue $artifact Name]' has no corresponding file in the file system.\nDo you want to delete the artifact and its manifested elements from the model?" -style yesno -icon information] == 6)} {
        foreach manifested [GetManifested $artifact] {
            u2::Delete $manifested
        }
        # Delete imports from package to artifact
        foreach d [GetReverseReferences $artifact Supplier] {
            u2::Delete $d
        }
        u2::Delete $artifact
        return
    }
  }

  set result [UpdateModelFromExternalFileWrapper $javaFile $container $artifact]

  # Bind artifact to make sure its dependencies are bound.
  Bind $artifact

  #The next line is a workaround for a bug in the TCL API, no space between
  #the elements in the list...
  set result [string map {i " i"} $result]

  output "Done ($result)\n"
  return $result
}

proc CreateElementsFromFile { file dir pkg } {
    
    # Create artifact to represent file
    set artifact [CreateManifestingArtifact $pkg [file tail $file] ""]
    
    # Set path to java file  
    SetJavaFile $artifact $file

    set result [UpdateModelFromExternalFileWrapper $file $pkg $artifact]
    if {"" == $result} {
        Output "Failed. Probably syntax error in file, see Check tab for details.\n"
    } else {
        Output "Done ($result)\n"
    } 

    # Bind artifact to make sure its dependencies are bound.
    Bind $artifact

    return $result
}

# Updates a package in the model from files.
# Checks if the package has a representation in the file system, and if not prompts 
# if the users want to delete the package. It then updates all existing model elements
# and searches the file system or any new files or directories and creates elements for them
proc UpdateModelPackage { pkg {silent 0} } {
   if ![u2::IsKindOf $pkg Package] {
    error "\nElement $pkg passed to UpdateModelPackage is not a package"
   }

   if [IsJarPackage $pkg] {
    Output "\nSkipping package [u2::GetValue $pkg Name] representing .jar file\n"
    return
   }
      
   set isUnnamed [IsUnnamedPackage $pkg]
   set targetDir [GetJavaDirectory $pkg]
   if {!$isUnnamed && ![file exists $targetDir]} {
    if {$silent || ([MessageDialog -name "IBM Rational Tau" -message "Package '[u2::GetValue $pkg Name]' has no corresponding directory in the file system.\nDo you want to delete the package and all its contents from the model?" -style yesno -icon information] == 6)} {
        u2::Delete $pkg
        return
    }
   }


   # Synchronize all elements in this package the model
   foreach element [GetCompilationUnitEntities $pkg] {
      if ![IsElementRef $element] {
        continue
      }
      if [IsJavaElement $element] {
        SynchronizeClass $element $silent
      } elseif [u2::IsKindOf $element Artifact] {
        UpdateModelArtifact $element $silent
      }
   }

   # Iterate over all subpackages and update the model from them
   set existingPackages ""
   set subPackages [u2::GetEntities $pkg SubPackage]
   foreach subpkg $subPackages {
        if {0 == $subpkg} {
            continue
        }
    UpdateModelPackage $subpkg $silent
    if {!$isUnnamed} {
        lappend existingPackages [u2::GetValue $subpkg Name]
    }
   }

   # Search for new files in the file system and create the corresponding model elements
   # New elements in the unnamed package owned by this package will be discovered here
   set files [GetFilesFromDirectory $targetDir ".java"]
   set modelFiles [GetJavaFilesFromPackage $pkg]
   foreach file $files {
      set fileName [file tail $file]
      if {[lsearch $modelFiles $fileName] == -1} {
    if {$isUnnamed} {
        set packageOwner [u2::GetEntity $pkg Owner]
        if ![u2::IsKindOf $packageOwner Session] {
            Output "\nUnnamed packages can only exist at the top-level"
            continue
        }
        CreateElementsFromFile $file $targetDir [GetUnnamedPackage [GetSession $pkg]]
    } else {
        CreateElementsFromFile $file $targetDir $pkg
    }
      }  
   }

   # Search for new folders in the file system and update the model according
   # to their contents. Should not be done for unnamed packages.
   if {!$isUnnamed} {
     set directories [glob -directory $targetDir -nocomplain -type {d} "*"]
     foreach newDir $directories {
        set newDirName [file tail $newDir]
        if {[lsearch $existingPackages $newDirName] == -1} {
      set newPackage [u2::Create $pkg Package]
      u2::SetValue $newPackage Name $newDirName
          set jpStereotype [u2::FindByGuid [GetSession $pkg] "@JavaView@javaPackage"]
          u2::SetEntity $newPackage StereotypeInstance [u2::CreateInstance $jpStereotype]

      UpdateModelPackage $newPackage $silent
        }
     }
  }
}

# Search for folders in the supplied directory and add and update corresponding
# root packages if needed.
proc UpdateModelProject { project dir {silent 0} } {   
    if { 0 == $project } {
        return
    }
    set session [std::GetModels -kind U2 -project $project]

    set rootPackages [u2::GetEntities $session Root]
    set sourceLocations [GetSourceLocations $session]
    foreach rootPackage $rootPackages {              
                if {0 == $rootPackage} {
                    continue
                }
            if {[u2::IsKindOf $rootPackage Package]} {                  
                if [IsJavaPackage $rootPackage] {                
                set update 1
                set javaDir [file normalize [GetJavaDirectory $rootPackage]]
                foreach srcLocation $sourceLocations {                   
                    if [IsSubDirOf [file normalize $srcLocation] $javaDir] {                        
                        set update 0
                        break
                    }
                }
                if $update {
                    UpdateModelPackage $rootPackage $silent             
                }
                }
            }
    }

    foreach location $sourceLocations {
        UpdateModelFromDirectory $location
    }
}


###################  Compile Java File #####################
proc IsJavaFileArtifact { element } {
	if { $element == 0 } {
		return false
	}
	
	if {![u2::IsKindOf $element Artifact] } {
		return false
	} else {
		set si [u2::GetTaggedValue $element "'@TTDFileModel@javaFile' (. .)" -interpretIdentsAsGuids true]
		if {$si != 0} {
			return true
		}
	}
	return false
}

# Returns a list of all entities representing java compilation units.
# This means file artifacts and non-manifested classes/interfaces
proc GetCompilationUnitEntities { pkg } {
    set result ""
    foreach member [u2::GetEntities $pkg OwnedMember] {
                if {0 == $member} {
                    continue
                }
        if {[IsJavaElement $member] && ![HasManifestDependency $member]} {
            lappend result $member
        } elseif [IsJavaFileArtifact $member] {
            lappend result $member
        }
    }
    return [lsort -unique $result]
}

# Returns a list of all javaFile artifacts in the namespace
proc GetJavaArtifacts { namespace } {
    set result ""
    foreach member [u2::GetEntities $namespace OwnedMember] {
                if {0 == $member} {
                    continue
                }
        if {[u2::IsKindOf $member Artifact] && ( 0 != [u2::GetTaggedValue $member "javaFile (. .)"])} {
            lappend result $member
        }
    }
    return $result
}

# Cleans from previously compiled classes
# For compile package - remove all class files from directory and subdirectories
# For class/artifact (A.java) - remove A.class and all A$*.class files
proc CleanPreviousCompile { element } {
  if {!([IsJavaElement $element] || [u2::IsKindOf $element Package] || [u2::IsKindOf $element Artifact])} {
    return
  }

  if [u2::IsKindOf $element Package] {
	set dir [GetJavaDirectory $element]
	if { ("" != $dir) && ![file exists $dir]} {
		return
	}
	 
    set classfiles [GetFilesFromDirectory $dir ".class"]
    foreach classfile $classfiles {
		file delete $classfile
    }
    
    foreach subPackage [u2::GetEntities $element SubPackage] {
      if {0 == $subPackage} {
  		continue
      }
      CleanPreviousCompile $subPackage
    }
    return
  } elseif {[IsJavaElement $element] && [HasManifestDependency $element]} {
    CleanPreviousCompile [GetArtifact $element]
    return
  } elseif { [u2::IsKindOf $element Artifact] } {
    # clean artifact
	set manifestedClasses [GetManifested $element]
	foreach manifested $manifestedClasses {
	  if { [IsJavaElement $manifested] } {
	    # get owner - package
		set owner [u2::GetEntity $manifested Owner]
		if {!([IsJavaPackage $owner])} {
          continue
        }
        # get directory of owner
        set ownerDir [GetJavaDirectory $owner]
        if { ($ownerDir == "") } {
          continue
        }
	  
		set name [u2::GetValue $manifested Name]
		if { ($name != "") } {
		  # delete class files
		  file delete [file join $ownerDir "$name.class"]
		  set nestedClasses [glob -directory $ownerDir -nocomplain "$name\$*.class"]
		  foreach nestedClass $nestedClasses {
			file delete nestedClass
		  }
		}
	  }
	}
  }
}

proc CompileJava { element } {
  if {!([IsJavaElement $element] || [u2::IsKindOf $element Package] || [u2::IsKindOf $element Artifact])} {
    return
  }
  if [u2::IsKindOf $element Package] {
    set compUnits [GetCompilationUnitEntities $element]
    foreach elem $compUnits {
        CompileJava $elem
    }
    foreach subPackage [u2::GetEntities $element SubPackage] {
                if {0 == $subPackage} {
                    continue
                }
        CompileJava $subPackage
    }
    return
  } elseif {[IsJavaElement $element] && [HasManifestDependency $element]} {
    CompileJava [GetArtifact $element]
    return
  }

  set inUnnamed [InUnnamedPackage $element]

  set dir [GetJavaDirectory $element]
  set completePath [GetJavaFile $element 1]
  
  if $inUnnamed {
    set classpath \"\"
  } else {
    set dir [file dirname $dir]
    set project [std::GetProject $element]
    if { 0 == $project } {
        output "Error: No project found!\n"
        return
    }
    set classpath [GetClassPath $project]
  }
  
  set javaVersion ""
  set session [GetActiveSession]
  if { $session != 0 } {
    # error ? - default 1.4
    set javaVersion [GetTaggedValue $session "javaSettings (. languageVersion .)" -asString]
  }

  if [file exists $completePath] {
    set result ""
    catch {[cd $dir]} result
   
    if { $javaVersion == "JAVA_6" } {
        # no changes in java6 syntax => use java 5 version to compile
        output "javac -source 1.5 -classpath $classpath \"$completePath\"\n"
        catch {exec << [pid] javac -source 1.5 -classpath $classpath "$completePath"} result
    } elseif { $javaVersion == "JAVA_5" } {
        output "javac -source 1.5 -classpath $classpath \"$completePath\"\n"
        catch {exec << [pid] javac -source 1.5 -classpath $classpath "$completePath"} result
    } elseif { $javaVersion == "JAVA_1_3" } {
        # use -source 1.3
        output "javac -source 1.3 -classpath $classpath \"$completePath\"\n"
        catch {exec << [pid] javac -source 1.3 -classpath $classpath "$completePath"} result
    } else {
        output "javac -source 1.4 -classpath $classpath \"$completePath\"\n"
        catch {exec << [pid] javac -source 1.4 -classpath $classpath "$completePath"} result
    }
    
    if {"" != $result} {
        regsub {.java:(\d*):} $result {.java(\1):} result
        output "$result\n"
    }

  } else {
    output "File $completePath does not exist.\n"
  }
}


####### utilities
proc GetBuildArtifact {entity} {
    set session [u2::GetEntity $entity Session]

    set agentFindBuildArtifact [u2::FindByGuid $session @TTDJavaModelCodeSync@Menu@FindBuildArtifact]
    if {$agentFindBuildArtifact == 0} {
        std::Output "can't find FindBuildArtifact agent\n"
        return 0
    }

    set ap [list 0]
    if { [catch {u2::InvokeAgent $session $agentFindBuildArtifact $entity ap} ] } {
        std::Output "can't invoke FindBuildArtifact agent\n"
        return 0
    }
    return [lindex $ap 0]
}

proc GenerateBuildArtifact {ba} {
    set session [u2::GetEntity $ba Session]

    set agentGenerate [u2::FindByGuid $session @TTDJavaModelCodeSync@Menu@Generate]
    if {$agentGenerate == 0} {
        std::Output "can't find Generate agent\n"
        return 0
    }

    set ap [list]
    if { [catch {u2::InvokeAgent $session $agentGenerate $ba ap} ] } {
        std::Output "can't invoke Generate agent\n"
        return 0
    }
    
    return $ba
}

proc CompileBuildArtifact {ba} {
    set session [u2::GetEntity $ba Session]

    set agentCompile [u2::FindByGuid $session @TTDJavaModelCodeSync@Menu@Compile]
    if {$agentCompile == 0} {
        std::Output "can't find Compile agent\n"
        return 0
    }

    set ap [list]
    if { [catch {u2::InvokeAgent $session $agentCompile $ba ap} ] } {
        std::Output "can't invoke Compile agent\n"
        return 0
    }
    
    return $ba
}

proc GetClassPath {ba} {
    set session [u2::GetEntity $ba Session]

    set agentGetClassPath [u2::FindByGuid $session @TTDJavaModelCodeSync@Menu@GetClassPath]
    if {$agentGetClassPath == 0} {
        std::Output "can't find GetClassPath agent\n"
        return ""
    }

    set ap [list "<empty>"]
    if { [catch {u2::InvokeAgent $session $agentGetClassPath $ba ap} ] } {
        std::Output "can't invoke GetClassPath agent\n"
        return ""
    }

    return [lindex $ap 0]
}

proc GetTargetDir {ba} {
    set session [u2::GetEntity $ba Session]

    set agentGetTargetDir [u2::FindByGuid $session @TTDJavaModelCodeSync@Menu@GetTargetDir]
    if {$agentGetTargetDir == 0} {
        std::Output "can't find GetTargetDir agent\n"
        return ""
    }

    set ap [list "<empty>"]
    if { [catch {u2::InvokeAgent $session $agentGetTargetDir $ba ap} ] } {
        std::Output "can't invoke GetTargetDir agent\n"
        return ""
    }
    return [lindex $ap 0]
}

###################  Execute Java File #####################

proc ExecuteJavaClass {class {javaOptions ""} {programOptions ""}} {
    global tcl_platform

    set buildArtifact [GetBuildArtifact $class]
    CompileBuildArtifact $buildArtifact
    set classPath [GetClassPath $buildArtifact]
    set targetDir [GetTargetDir $buildArtifact]
    set fullyQualifedName [GetJavaQualifiedName $class]

    if {$tcl_platform(platform) == "windows"} {
        set classPath "$targetDir;$classPath"
    } else {
        set classPath "$targetDir:$classPath"
    }

    set cmd [list java -esa]
    lappend cmd -cp $classPath
    if {$javaOptions != ""} {
        lappend cmd $javaOptions
    }
    lappend cmd $fullyQualifedName
    if {$programOptions != ""} {
        lappend cmd $programOptions
    }

    catch { eval exec << [pid] $cmd } result

    output "$result\n"
    output "Done\n"
}

###################  Execute Java Applet #####################

proc CreateHtmlFile { fileName applet } {
    set f [open $fileName w]
    puts $f "<html>"
    puts $f "<head>"
    puts $f "<title>"
    puts $f "Test page for $applet"
    puts $f "</title>"
    puts $f "</head>"
    puts $f "<body>"
    puts $f "<applet code=\"$applet.class\">"
    puts $f "</applet>"
    puts $f "</body>"
    puts $f "</html>"
    close $f
}

proc ExecuteJavaApplet {dir class} {
    global tcl_platform

    set className [u2::GetValue $class Name] 

    if { ![u2::IsKindOf [u2::GetEntity $class Owner] Session] } {
        output "Class '$className' is not owned by an unnamed package.\n"
        output "Applet execution is only supported for classes in unnamed packages.\n"
        output "Aborting applet execution.\n"
        return
    }

    set buildArtifact [GetBuildArtifact $class]
    CompileBuildArtifact $buildArtifact
    set targetDir [GetTargetDir $buildArtifact]

    set htmlFile ${className}.html
    set htmlPath [file native [file join $targetDir $htmlFile]]
    if [file exists $htmlPath] {
        output "Using existing html file: $htmlPath\n"
    } else {
        output "Creating test html file: $htmlPath\n"
        CreateHtmlFile $htmlPath $className
    }
    
    if {$tcl_platform(platform) == "windows"} {
        # Windows
        output "Starting web browser ... "
        cd $targetDir
        if [catch {exec << [pid] explorer $htmlFile &} result ] {
            output "Failed\n"
        } else {
            output "Done\n"
        }
    } else {
        # Unix
        output "Information: The .html file needs to be opened manually on Unix platforms.\n"
    } 
}

###################  Export JAR File #####################

proc ExportPackageToJAR { pkg jarFile } {
    set buildArtifact [GetBuildArtifact $pkg]
    CompileBuildArtifact $buildArtifact
    set targetDir [GetTargetDir $buildArtifact]

    cd $targetDir

    set fileListFile tmplist
    if [catch {
            set listFile [open $fileListFile w]
        }] {
        output "\nGeneration of JAR file failed. Unable to create temporary file in folder '$jarExecDir'."
        return
    }
    set classFiles [GetFiles ""]

    foreach classFile $classFiles {
        puts $listFile $classFile
    }
    close $listFile

    # create jar file
    eval "catch {exec << [pid] jar -cvf \"$jarFile\" @tmplist} result"
    
    file delete $fileListFile
    
    return $result
}


#####

proc GenerateJavaDoc {pkg dir} {
    set buildArtifact [GetBuildArtifact $pkg]
    GenerateBuildArtifact $buildArtifact
    set targetDir [GetTargetDir $buildArtifact]
    set classpath [GetClassPath $buildArtifact]

    cd $targetDir  
    
    set packageName [u2::GetValue $pkg Name]
    output "Generating Javadoc for package '$packageName' ...\n"

    set fileListFile tmplist
    if [catch {
            set listFile [open $fileListFile w]
        }] {
        output "\nGeneration of JavaDoc file failed. Unable to create temporary file."
        return
    }
    set javaFiles [GetFiles $packageName {*.java}]
    
    if { [llength $javaFiles] < 1 } {
        output "Error: no source files are found.\n"
        return
    }

    foreach javaFile $javaFiles {
        puts $listFile $javaFile
    }
    close $listFile
    catch {exec << [pid] javadoc -private -classpath $classpath -d $dir @$fileListFile} result
    output "$result\n"
    output "Done\n"
  
    set indexFile [file join $dir "index.html"]
    output "Displaying javadoc for package '$packageName'.\n"
    if [file exists $indexFile] {
        HtmlReport $indexFile
    } else {
        Output "\nUnable to open file $indexFile"
    }
}

# Returns the an element matching the fully qualified java name.
# Returns 0 if no match is found.
proc GetElementFromQualifiedName { qualifiedName { context 0 }} {    
    set names [split $qualifiedName .]
    set first [lindex $names 0]
    if { $context == 0 } {
        set session [GetActiveSession]
        if { "" == $session } {
        	return 0
        }
        set roots [u2::GetEntities $session Root]       
        foreach root $roots {
            if {0 == $root} {
                        continue
            }       
            
            if {[u2::IsKindOf $root Package] || [IsJavaElement $root]} {
                if [IsUnnamedPackage $root] {
                    if {$names == "default"} {                      
                        set ret $root
                        return $ret
                    }                   
                    set rootResult [GetElementFromQualifiedName $qualifiedName $root]
                    if {$rootResult != 0} {
                        return $rootResult
                    }
                } else {                    
                    set rootName [u2::GetValue $root Name]
                    if {$rootName == $first} {
                        if {[llength $names] == 1} {
                            return $root
                        }
                        set rootResult [GetElementFromQualifiedName [join [lrange $names 1 end] .] $root]
                        if {$rootResult != 0} {
                            return $rootResult
                        }
                    }
                }
            }
        }
    } else {        
        set x [GetElement $first $context]      
        if {0 != $x} {
            if {[llength $names] == 1} {
                return $x
            } else {
                return [GetElementFromQualifiedName [join [lrange $names 1 end] .] $x]
            }
        }           
    }
    return 0
}

# Returns the first element matching the name in the supplied context, 0 if no match is found
proc GetElement { name context } {
    set members [u2::GetEntities $context OwnedMember]
    foreach member $members {
                if {0 == $member} {
                    continue
                }
        if {[u2::IsKindOf $member Package] && [IsUnnamedPackage $member]} {
            set inUnnamed [GetElement $name $member]
            if {$inUnnamed != 0} {
                return $inUnnamed
            }
        } else {
            set memberName [u2::GetValue $member Name]
            if {$name == $memberName} {
                return $member
            }
        }
    }
    return 0
}

# Locates the first element matching the fully qualified name
proc LocateJavaElement { qualifiedName } {
    Output "\nLocating '$qualifiedName' ... "
    set isJavaFile [regexp ^(.*).java$ $qualifiedName]  
    if $isJavaFile {
        Output "It is a java file.\n"
        set session [GetActiveSession]
        if { "" != $session } {
            foreach artifact [GetJavaFileArtifacts $session] {
                if {0 == $artifact} {
                    continue
                }
                if {[u2::GetValue $artifact Name] == $qualifiedName} {
                    Output "Done.\n"
                    u2::Locate $artifact
                    return 1
                }
            }
    	}
    } else {
        set element [GetElementFromQualifiedName $qualifiedName]        
        if {0 != $element } {
            Output "Done.\n"
            u2::Locate $element
            return 1
        }
    }
    Output "No matching element found.\n"
    return 0
}

proc UpdateModelFromExternalFileWrapper {file context elements} {    
    output "Updating model from: $file ... "
    return [UpdateModelFromExternalFile "Java" "$file" $context [split $elements]]
}

# Returns 1 if path exists in pathList, 0 otherwise
proc PathInList { path pathList } {
    foreach existingPath $pathList {
        if [string equal [file normalize $path] [file normalize $existingPath]] {
            return 1
        }
    }
    return 0
}

proc ImportJarFile { jarFile {context 0}} {

    if ![file exists $jarFile] {
        return 0
    }


    # Create a dummy package and import the contents of the JAR into it
    set session [GetActiveSession]
    if {"" == $session} {
    	Output "No session found.\n"
    	return 0
    }
    
    set dummyPackage [u2::Create $session Package]
    u2::SetValue $dummyPackage Name [file tail $jarFile]
    UpdateModelFromArchiveFile "Java" $jarFile $dummyPackage
    
    set resource [GetResource $session]

    # Create artifact
    if {0 != $context} {
        set artifact [u2::Create $context Artifact]
    } else {
        set artifact [u2::Create $resource Artifact]
    }

    u2::SetValue $artifact Name [file tail $jarFile]

    # Apply jarFile stereotype on artifact
    set jarFileStereotype [FindByGuid [GetSession $artifact] "@TTDFileModel@jarFile"]
    set jarFileStereotypeInstance [ApplyStereotype $artifact $jarFileStereotype]
    set relativeFile $jarFile
    if {[file pathtype $jarFile] == "absolute"} {
        set projectDir [GetProjectDir [std::GetProject $artifact]]
        if { "" != $projectDir } {
            set relativeFile [GetRelativePath $jarFile $projectDir]     
        }
    }
    u2::SetTaggedValue $jarFileStereotypeInstance "jarFile (. path .)" "\"[string map {\\ \\\\} $relativeFile]\""

    set manifestStereotype [FindByGuid [GetSession $artifact] "@JavaView@jarDependency"]
    
    set createdElements [u2::GetEntities $dummyPackage OwnedMember]
    foreach createdElement $createdElements {

                if {0 == $createdElement} {
                    continue
                }

        # Move element from dummy package to context or root level
        if {0 != $context} {
            Move $context $createdElement
        } else {
            Move $session $createdElement
        }

        if {0 == $context} {
            # Add elements to file
            u2::SetEntity $resource Root $createdElement
        }

        # Set up dependencies between artifact and elements
        set dep [u2::Create $artifact Dependency]
        set si [ApplyStereotype $dep $manifestStereotype]
        # Note: u2::SetValue doesn't work here
        #SetA $dep Supplier [u2::GetValue $createdElement Name]
        u2::SetEntity $dep Supplier $createdElement

        if [u2::IsKindOf $createdElement Package] {
            ApplyPackageStereotypeRecursively $createdElement "@JavaView@jarPackage"
        }
    }
    
    # Delete dummy package 
    Delete $dummyPackage

    return $createdElements
}

proc ApplyPackageStereotypeRecursively { pkg stereotypeGUID } {
    if { 0 == [GetTaggedValue $pkg "'$stereotypeGUID' (. .)" -guid] } {
        ApplyStereotype $pkg [FindByGuid [GetSession $pkg] $stereotypeGUID]
    }
    foreach subPkg [u2::GetEntities $pkg SubPackage] {
                if {0 == $subPkg} {
                    continue
                }
        ApplyPackageStereotypeRecursively $subPkg $stereotypeGUID
    }
}

# Returns one valid resource of a session. Creates a new resource if needed.
proc GetResource { session } {
    set resources [u2::GetEntities $session RootResource]
    if {[llength $resources] > 0} {
                # We assume that the returned resource is not unbound
        return [lindex $resources 0]
    } else {
        return [u2::Create $session Resource]
    }
}

# Adds the supplied path the the list of source locations for the project.
# Creates a new javaSettings stereotype instance if needed.
proc AddProjectSourcelocation { project path } {
    set session [std::GetModels -kind U2 -project $project]
    set existingPaths ""

    set result 0

    # Create stereotype instance if needed
    if { 0 == [u2::GetTaggedValue $session "javaSettings (. .)"] } {
        set sourceLocationStereotype [u2::FindByGuid $session "@JavaView@javaSettings"]
        if {0 != $sourceLocationStereotype} {
            u2::SetEntity $session StereotypeInstance [u2::CreateInstance $sourceLocationStereotype]        
            # Create an empty list expression
            u2::SetTaggedValue $session "javaSettings (. path .)" "{}"
        }
    } else {
        set existingPaths [GetSourceLocations $session]
    }
    
    # Make path relative
    set relativePath $path
    if {[file pathtype $path] == "absolute"} {
        set projectDir [GetProjectDir $project]
        if { ("" != $projectDir) && ($projectDir != $path) } {
            set relativePath [GetRelativePath $path $projectDir]
            # If path equals project directory, add full path
            if { "" == $relativePath } {
                set relativePath "."
            }
        }
    }

    # Add path to list if needed
    if ![PathInList $path $existingPaths] {

        # Retrieve list expression
        set pathExpr [u2::GetTaggedValue $session "javaSettings (. path .)"]
        if { 0 == $pathExpr } {
            # Set the pathlist to this value only
            u2::SetTaggedValue $session "javaSettings (. path .)" \{\"[string map {\\  /} $relativePath]\"\}
        } else {
            # Add the path to the pathlist
            set value [u2::Create $pathExpr CharstringValue]
            u2::SetValue $value ValueString $relativePath
        }
        set result 1
    }

    return $result
}

# Returns a list of paths pointing to the location of the source code for the element
proc GetSourceLocations { element } {
    set sourceLocations [u2::GetTaggedValue $element "javaSettings (. path .)"]
    set result ""
    if { (0 != $sourceLocations) && [u2::IsKindOf $sourceLocations ListExpr] } {
        set projectDir [GetProjectDir [std::GetProject $element]]

        set pathList [u2::GetEntities $sourceLocations expression]
        foreach pathString $pathList {
                        if {0 == $pathString} {
                            continue
                        }
            if [u2::IsKindOf $pathString CharstringValue] {
                set path [u2::GetValue $pathString Value]
                if { "" != $path } {
                    if {[file pathtype $path] != "absolute"} {
                        if { "" != $projectDir } {
                            set path [file normalize [file join $projectDir $path]]
                        }
                    }
                    lappend result $path
                }
            }
        }
    }
    return [lsort -unique $result]
}


# Moves a java package by moving any generated files of the target directory.
# File paths of java file artifacts pointing to the old directory are updated.
# NOTE: Path tagged value of subpackages are not updated, but path of artifacts
# are. Currently path is only set for rootpackages, so this is ok.
proc MoveJavaPackage { pkg targetDir copyFiles {setJavaDirectory 1} } {
    if ![file exists $targetDir] {
        file mkdir $targetDir
    }
    set oldFiles [GetJavaFilesFromPackage $pkg 1]
    foreach oldFile $oldFiles {
        set newFile [file normalize [file join $targetDir [file tail $oldFile]]]
        if { [file exists $oldFile] && ([file normalize $oldFile] != $newFile)} {
            if $copyFiles {
                file copy -force $oldFile $newFile
            } else {
                file rename -force $oldFile $newFile
            }
        }
    }
    
    set sourceDir [GetJavaDirectory $pkg]

    foreach artifact [GetJavaArtifacts $pkg] {
        set oldFile [GetJavaFile $artifact 0]
        SetJavaFile $artifact [file join $targetDir $oldFile]
    }

    foreach subPackage [u2::GetEntities $pkg SubPackage] {
                if {0 == $subPackage} {
                    continue
                }
        set subTarget [file join $targetDir [u2::GetValue $subPackage Name]]
        MoveJavaPackage $subPackage $subTarget $copyFiles 0
    }

    if $setJavaDirectory {
        SetJavaDirectory $pkg $targetDir
    }

    if !$copyFiles {
        catch {file delete $sourceDir}
    }
}

# This is a helper function to MoveJavaPackage to update the paths of all file artifacts
# owned by the moved package
proc UpdateArtifacts { pkg oldPackagePath newPackagePath } {
    foreach artifact $artifacts {
        set oldFile [GetJavaFile $artifact 1]
        set newDir [GetJavaDirectory $pkg]
        SetJavaFile $artifact $newPath
    }
}

##########################################################################
#
#   File related procedures:
#   
#   GetUniqueTail   path1 path2
#   GetCommonPath   path1 path2
#   GetRelativePath path1 path2
#   GetVolume   path
#
##########################################################################

# Returns the tail of path2 not common to path1 and path2
proc GetUniqueTail { path1 path2 } {
    set p1 [file split $path1]
    set p2 [file split $path2]
    set result ""
    set p1Length [llength $p1]
    set p2Length [llength $p2]
    for {set i 0} {($i < $p1Length) || ($i > $p2Length)} {incr i} {
        if {[lindex $p1 $i] != [lindex $p2 $i]} {
            break
        }       
    }
    set tail [lrange $p2 $i end]
    foreach element $tail {
        set result [file join $result $element]
    }
    return $result
}


# Returns the common head of the two file paths
proc GetCommonPath { path1 path2 } {
    set p1 [file split $path1]
    set p2 [file split $path2]
    set p1Length [llength $p1]
    set result ""
    for {set i 0} { $i < $p1Length } {incr i} {
        if [string equal [lindex $p1 $i] [lindex $p2 $i]] {
            set result [file join $result [lindex $p1 $i]]
        } else {
            break
        }       
    }
    return $result
}


# Returns the relative path of path1 compared to path2.
proc GetRelativePath { path1 path2 } {
    set result ""
    set p1Type [file pathtype $path1]
    set p2Type [file pathtype $path2]
    if {($p1Type == "relative") || ($p2Type == "relative")} {
        return $path1
    }

    if {($p1Type == "absolute") || ($p1Type == "absolute")} {
        if { [GetVolume $path1] != [GetVolume $path2] } {
            return $path1
        }
    }

    set tail [GetUniqueTail $path2 $path1]

    set commonPath [GetCommonPath $path1 $path2]
    set diff [expr [llength [file split $path2]] - [llength [file split $commonPath]]]
    for {set i 0} {$i < $diff} {incr i} {
        set result [file join .. $result]
    }
    return [file join $result $tail]
}

# Returns the volume of an absolute path. Returns "" if a non-absolute path is supplied
proc GetVolume { path } {
    set result ""
    if { [file pathtype $path] == "absolute" } {
        set result [lindex [file split $path] 0]
    }
    return $result
}

# Returns 1 if dir1 is a subdirectory of dir2, 0 otherwise
proc IsSubDirOf { dir1 dir2 } {
    if {("" == $dir1) || ("" == $dir2)} {
        return 0
    }
    if {[GetUniqueTail $dir1 $dir2] == ""} {
        return 1
    }
    return 0
}

# Returns the directory of a project
proc GetProjectDir { project } {
    if { (0 == $project) || ("" == $project) } {
        return ""
    }
    return [file dirname [string map {\\  /} [std::GetProjectPath $project]]]
}

# Sets the java file of an artifact. If possible, an absoulte path is 
# converted into a project-relative path.
proc SetJavaFile { artifact path } {
    if { 0 == $artifact } {
        return
    }
    set result $path
    if {[file pathtype $path] == "absolute"} {
        set projectDir [GetProjectDir [std::GetProject $artifact]]
        if { "" != $projectDir } {
            set relativePath [GetRelativePath $path $projectDir]        
            set result $relativePath
        }
    }
    u2::SetTaggedValue $artifact "javaFile (. path .)" "\"[string map {\\ \\\\} $result]\""
}

# Sets the java directory of a package. If possible, an absoulte path is 
# converted into a project-relative path.
proc SetJavaDirectory { pkg path } {
    if { 0 == $pkg } {
        return
    }
    set result $path
    if {[file pathtype $path] == "absolute"} {
        set projectDir [GetProjectDir [std::GetProject $pkg]]
        if { "" != $projectDir } {
            set relativePath [GetRelativePath $path $projectDir]        
            set result $relativePath
        }
    }
    u2::SetTaggedValue $pkg "javaPackage (. path .)" "\"[string map {\\ \\\\} $result]\""
}

proc GetDefaultClasspath { project } {
    if { 0 == $project } {
        return ""
    }
    set pathList ""
    set session [std::GetModels -kind "U2" -project $project]
	
    set javaRoots [GetJavaRoots $session]
    foreach root $javaRoots {
        if [IsUnnamedPackage $root] {
            continue
        } elseif [IsJarArtifact $root] {
            set dir [GetJavaFile $root 1]
            if { "" != $dir } {
                lappend pathList $dir
            }
        } else {
            set dir [GetJavaDirectory $root]
            if { "" != $dir } {
                lappend pathList [file dirname $dir]
            }
        }
    }
    set pathList [concat $pathList [GetSourceLocations $session]]
	
	return $pathList
}

# Returns the complete classpath of the supplied project

proc old_GetClassPath { project } {
    global tcl_platform
    if { 0 == $project } {
        return ""
    }
    set pathList ""
    set session [std::GetModels -kind "U2" -project $project]

    set userClasspath [GetStringList "javaSettings" "classpath" $session]

    if { [llength $userClasspath] != 0 } {
    # there is a user defined classpath

        set projectDir [GetProjectDir $project]
        foreach userPath $userClasspath {
            if { "" != $userPath } {
                set tmpPath $userPath
                if {[file pathtype $userPath] != "absolute"} {
                    if { "" != $projectDir } {
                        set tmpPath [file normalize [file join $projectDir $userPath]]
                    }
                }
                lappend pathList $tmpPath
            }
        }
    }
    
    # add locations of top level packages
    set pathList [concat $pathList [GetDefaultClasspath $project]]

    set pathList [lsort -unique $pathList]
    set result ""
    foreach path $pathList {
        if { "" == $result } {
            set result $path
        } else {
            # Use : as a separator on Solaris and ; on all other platforms
            if {$tcl_platform(os) == "SunOs"} {
                set result ${result}\:$path
            } else {
                set result ${result}\;$path
            } 
        }
    }
    return $result
}

# Returns all java roots, i.e. packages and jar files in a sesion
proc GetJavaRoots { session {includeJars 1} } {
    set roots [u2::GetEntities $session Root]
    set result ""
    foreach root $roots {
                if {0 == $root} {
                    continue
                }
        if { [u2::IsKindOf $root Package] } {
            if [IsJavaPackage $root] {
                lappend result $root
            }
        } elseif { $includeJars && [IsJarArtifact $root] } {
            lappend result $root
        } 
    }
    return $result
}

# Returns all javaFile artifacts in a session.
proc GetJavaFileArtifacts { session } {
    set javaFileStereotype [u2::FindByGuid $session "@TTDFileModel@javaFile"]
    if { 0 == $javaFileStereotype } {
        return ""
    }
    set result ""
    set instanceExpressions [u2::GetReferingEntities $javaFileStereotype InstanceOf]
    foreach ie $instanceExpressions {
        if {0 == $ie} {
            continue
        }
        set owner [u2::GetEntity $ie InstanceExprOwner]
        if {0 != $owner} {
            lappend result $owner
        }
    }
    return $result
}

# Returns a list of string values from the given tagged value of the given stereotype
# Returns "" if stereotype/tagged value doesn't exist, or the tagged value isn't a list 
# expression.
proc GetStringList { stereotypeName taggedValue element } {
    set result ""
    set taggedValue [u2::GetTaggedValue $element "$stereotypeName (. $taggedValue .)"]
    if { (0 != $taggedValue) && [u2::IsKindOf $taggedValue ListExpr] } {
        foreach expr [u2::GetEntities $taggedValue expression] {
                        if {0 == $expr} {
                            continue
                        }
            if [u2::IsKindOf $expr CharstringValue] {
                lappend result [u2::GetValue $expr Value]
            }
        }
    }
    return $result
}

proc ConvertToJavaPackage { pkg } {
    if {![IsKindOf $pkg Package]} {
        return
    }
    if {[IsJarPackage $pkg] } {
        return
    }
    
    set jpStereotype [u2::FindByGuid [u2::GetEntity $pkg Session] "@JavaView@javaPackage"]
    if { $jpStereotype == 0 } {
        return
    }
    if {![IsJavaPackage $pkg] } {
        u2::SetEntity $pkg StereotypeInstance [u2::CreateInstance $jpStereotype]
        JavaSyntax $pkg 1
    }
    
    set members [u2::GetEntities $pkg OwnedMember]
    foreach member $members {
        ConvertToJavaPackage $member
    }
}

proc ShouldConvertToJavaFile { pkg } {
	if { ![u2::IsKindOf $pkg Package] } {
		return 0
	}
	if { [IsJavaPackage $pkg] || [IsJarPackage $pkg]} {
		return 0
	}
	
	set owner [u2::GetEntity $pkg Owner]
	if { ($owner == 0) } {
		return 1
	} 
	if [u2::IsKindOf $owner Session] {
		# top level package
		return 1
	}
	
	if [u2::IsKindOf $owner Package] {
		# convert non top level package only if owner is java package
		return [IsJavaPackage $owner]
	}
	return 0
}

# Loads all the profiles needed by the Java support
proc LoadJavaProfiles {} {

    # Calculation of the base path for the JavaApplication Addin
    set installationEtcPath [file join [GetInstallationDirectoryWrapper] etc]

    set session [GetActiveSession]
    if{ "" == $session } {
    	output "Error: no session found.\n"
    	return
    }
    
    # Load JavaParser profile if needed
    set javaParserProfileLoaded [u2::FindByGuid $session @Java@extends]
    if { 0 == $javaParserProfileLoaded } {
       set JavaParserPath $installationEtcPath/JavaParser.u2
       if [catch {LoadProfile $JavaParserPath} result] {
            # NOTE: Exception is thrown by LoadProfile since JavaProfile
            # contains unbound references. This is intentional but
            # we do not want to show it to the user
       }
    }

    # Load JavaView metamodel if needed
    set javaViewLoaded [u2::FindByGuid $session @JavaView@javaPackage]
    if { 0 == $javaViewLoaded} {
       set JavaViewPath $installationEtcPath/JavaView.u2
       output "Loading Java profile: $JavaViewPath\n"
       if [catch {LoadProfile $JavaViewPath} result] {
      # NOTE: Exception is thrown by LoadProfile since JavaProfile
      # contains unbound references. This is intentional but
      # we do not want to show it to the user
       }
    }

    set metaModel [u2::FindByGuid $session @JavaView@JavaView]
    if { 0 != $metaModel } {
        RegisterMetaModel $metaModel
    } else {
        output "Error: failed to register Java View\n"
    }

    set sessionDeps [u2::GetEntities $session ClientDependency]
    set addJavaPredefinedPackage 1
    foreach dep $sessionDeps {
        if {0 == $dep} {
	        continue
		}
        set supplier [u2::GetEntity $dep Supplier]
        if {0 == $supplier} {
			continue
		}
		set guid [u2::GetValue $supplier Guid]
	    if {"@JavaPredefined" == $guid} {
		    set addJavaPredefinedPackage 0
		}
    }
    
    if $addJavaPredefinedPackage {
		set newDep [u2::Create $session Dependency]
		set accessStereotype [u2::FindByGuid $session "@Predefined@access"]
		u2::SetEntity $newDep StereotypeInstance [u2::CreateInstance $accessStereotype]
		u2::SetValue $newDep Supplier "uid:@JavaPredefined"
    }
   
    ########
    # loading java profile
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]
    set agent [u2::FindByGuid $model "@JavaAgent@LoadJavaRuntime"]
    
    if { 0 != $agent } {
        set param ""

        if { [catch {
            u2::InvokeAgent $model $agent $model param
        } e ] } {
            std::Output "Error when invoking Load Java Runtime agent : $e\n"
        }
    }
    ########

    #######
    # load TTDModelCodeSync profile
    if {[u2::FindByGuid $session @TTDModelCodeSync] == 0} {
       if [catch {LoadProfile [file join $installationEtcPath TTDModelCodeSync.u2]} result] {
            # NOTE: Exception is thrown by LoadProfile since JavaProfile
            # contains unbound references. This is intentional but
            # we do not want to show it to the user
       }
    }
    	
    #######
    # load TTDModelCodeSync profile
    if {[u2::FindByGuid $session @TTDJavaModelCodeSync] == 0} {
       if [catch {LoadProfile [file join $installationEtcPath JavaModelCodeSync.u2]} result] {
            # NOTE: Exception is thrown by LoadProfile since JavaProfile
            # contains unbound references. This is intentional but
            # we do not want to show it to the user
       }
    }
    
    #######
    # load TOR profile
    if {[u2::FindByGuid $session @TTDtor] == 0} {
       if [catch {LoadProfile [file join $installationEtcPath TAUObjectRunTime.u2]} result] {
            # NOTE: Exception is thrown by LoadProfile since JavaProfile
            # contains unbound references. This is intentional but
            # we do not want to show it to the user
       }
    }

    if { [set agentMigration [u2::FindByGuid $session @TTDJavaModelCodeSync@MigrateToBuildArtifacts] ] != 0} {
        set ap {}
        u2::InvokeAgent $session $agentMigration $session ap
    } else {
        std::Output "Can't find MigrateToBuildArtifacts\n"
    }

    u2::Bind $model

    set agentEnable [u2::FindByGuid $session @TTDJavaModelCodeSync@EnableJavaMCSync]
    if { $agentEnable != 0 } {
        set ap {1}
        if { [catch {u2::InvokeAgent $session $agentEnable $session ap} ] } {
            std::Output "Can't invoke EnableJavaMCSync agent\n"            
        }
    } else {
        std::Output "Can't find EnableJavaMCSync agent\n"
    } 
}

proc UnloadJavaProfile { guid } {
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]

	set resource [u2::FindByGuid $model $guid]
	set packages [u2::GetEntities $resource Root]
	
	foreach pack $packages {
		u2::UnloadLibrary $pack	
	}
	
	u2::Delete $resource

}

# Unloads all the profiles needed by the Java support
proc UnloadJavaProfiles {} {

    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind "U2" -project $curProject]

    # set 'Standard View' metamodel
    set models [std::GetModels -kind U2]
    foreach m $models {
        set metaModel [u2::FindByGuid $m "@TTDMetamodel"]
    }
    if {$metaModel != 0} {
        u2::SelectMetaModel $metaModel
    }

    set agent [u2::FindByGuid $model "@JavaAgent@UnloadJavaDependencies"]
    
    # invoke agent to make some unloading
    if { 0 != $agent } {
        set param ""

        if { [catch {
          u2::InvokeAgent $model $agent $model param
         } e ] } {
                std::Output "Error when invoking Unload Java agent : $e\n"
         }
    }	
}

# Returns true if the loadLibrary tagged value of the javaSettings stereotype
# is true, false otherwise. NOTE: This implementation uses the fact that the
# default value of the tagged value is true.
proc LoadLibrary { session } {
    set ident [u2::GetTaggedValue $session "javaSettings (. loadLibrary .)"]
    if { 0 != $ident } {
        set boolValue [u2::GetValue $ident Name]
        if { "false" == $boolValue } {
            return 0
        }
    }
    return 1
}

# Returns the installation directory. Adds a delimiter if the installation
# is in the root of a volume. GetInstallationDirectoryWrapper doesn't do that.
proc GetInstallationDirectoryWrapper {} {
    set result [std::GetInstallationDirectory]
    if { (2 == [string length $result]) && (":" == [string index $result end]) } {
        set result ${result}/
    }
    return $result
}
