# Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:17 PM CET 2014
# Licensed Materials - Property of IBM 
# (c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.
# 
# U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
# -*- mode: Tcl; tcl-indent-level: 4; tab-width: 4; -*-
package require commands
package require dialogs

# Return active project
proc GetActiveSession {} {
    set activeProject [std::GetActiveProject]
    if { $activeProject == 0 } {
        return 0
    }
    set projects [std::GetModels -kind U2 -project $activeProject]
    foreach p $projects {
        return $p
    }
    return 0    
}

# Load library
proc LoadLib { profile msg } {   
    set etcDir [GetInstallationDirectory]etc
    set profilePath "$etcDir/$profile"
    output "Loading the $msg library: $profilePath\n"
    set err [catch { u2::LoadLibrary $profilePath } log ]
}

# Unload library by its GUID
proc UnloadLib { lib } {
    set curProject [std::GetActiveProject]
    set model [std::GetModels -kind U2 -project $curProject]
    set package [u2::FindByGuid $model $lib] 
    if { $package != 0 } {
        set err [catch { u2::UnloadLibrary $package } log ]
        if { $err } {
            output "$log"
        }        
    }
}

# check if <<WSDLFile>> stereotype is applied to the artifact
proc IsWSDLFileArtifact { context } {
    if { [ u2::IsKindOf $context Artifact] != 0}  {
        if { [ u2::GetTaggedValue $context "wsdlFile(. .)"] != 0 } {            
            # Check that manifested package has <<wsdlPackage>> stereotype        
            foreach dep [u2::GetEntities $context ClientDependency] {
                if { [u2::GetTaggedValue $dep "manifest(. .)"] != 0 } {                    
                    set supplier [u2::GetEntity $dep Supplier]
                    if { $supplier != 0 } {
                        if { [u2::GetTaggedValue $supplier "wsdlPackage(. .)"] == 0 } {
                            return 0
                        }
                    }
                }
            }
            return 1
        }
    }
    return 0
}

# return <<WSDLFile>> artifact that manifest the package
proc GetWSDLFileArtifact { p } {
    
    if { [IsWSDLFileArtifact $p ] != 0} {
        return $p
    }
    
    if { [u2::IsKindOf $p "Package" ] != 0 } {  
        foreach dep [u2::GetReferringEntities $p Supplier] {            
            set owner [u2::GetEntity $dep Client]
            if { [u2::GetTaggedValue $dep "manifest(. .)"] != 0 } {                
                if { [u2::GetTaggedValue $owner "wsdlFile(..)"] != 0 }  {                    
                    return $owner
                }
            }
        }   
    }
    return 0
}

# invoke GenerateWSDL2File agent
proc InvokeExportAgent { context fileName } {
    set session [ GetActiveSession ]
    if { $session != 0 } {
        set agent [u2::FindByGuid $session "@WSDLAgents@GenerateWSDL2File"]
        if { $agent != 0 } {
            std::Output "Generating file $fileName..."
            set params { }
            set params [lappend params $fileName]           
            if { [catch { u2::InvokeAgent $session $agent $context params } e ] } {
                std::Output "FAILED!\n"
                std::Output "Error when invoking GenerateWSDL2File agent : $e\n"
            }
            std::Output "DONE!\n"
        }
    }
}

# invoke ReImportWSDL agent
proc InvokeReImportAgent { context } {
    set session [ GetActiveSession ]
    if { $session != 0 } {
        set agent [u2::FindByGuid $session "@WSDLAgents@ReImportWSDL"]
        if { $agent != 0 } {
            std::Output "Re-importing WSDL..."
            set params [list $context]
            if { [catch { u2::InvokeAgent $session $agent $session params } e ] } {
                std::Output "FAILED!\n"
                std::Output "Error when invoking ReImportWSDL agent : $e\n"
            }
            std::Output "DONE!\n"
        }
    }
}

# Import WSDL is enabled only on artifacts
proc OnEnableImportWSDL { cmd } {    
    set sel [std::GetSelection]
    if { [catch { set r [u2::IsKindOf $sel Entity] }] } {
        return 0
    }

    if { [u2::IsKindOf $sel Artifact] } {
        if { [IsWSDLFileArtifact $sel ] != 0 } {
            return 1
        }
    }
    return 0
}

# Generate XSD is enabled on packages and artifacts
proc OnEnableGenerateWSDL { cmd } {    
    set sel [std::GetSelection]
    if { [catch { set r [u2::IsKindOf $sel Entity] }] } {
        return 0
    }

    # Enable command only on the Package with <<wsdlPackage>> stereotype
    if { [u2::IsKindOf $sel Package] } {
        if { [ u2::GetTaggedValue $sel "wsdlPackage(. .)"] != 0 } {    
            return 1
        }        
    }

    if { [u2::IsKindOf $sel Artifact] } {
        if { [IsWSDLFileArtifact $sel ] != 0 } {
            return 1
        }
    }
    
    return 0
}

proc OnEnableWSDL { cmd } {
    if { [OnEnableImportWSDL cmd] } {
        return 1
    }
    if { [OnEnableGenerateWSDL cmd] } {
        return 1
    }
    return 0
}

# Re-import WSDL
proc OnImportWSDL { command } {
    set sel [std::GetSelection]
    set artifact [GetWSDLFileArtifact $sel]
    
    # there is existing <<WSDLFile>> artifact
    if { $artifact != 0 } {
        set v [u2::GetTaggedValue $artifact "wsdlFile(..)" ]        
        if { ($v != 0) && ([u2::IsKindOf $v InstanceExpr] != 0)} {
            InvokeReImportAgent $artifact
        }
        return 0
    }    
}

# Export package to WSDL
proc OnExportWSDL { command } {
    set sel [std::GetSelection]
    set artifact [GetWSDLFileArtifact $sel]
    
    # there is existing <<WSDLFile>> artifact
    if { $artifact != 0 } {        
        set v [u2::GetTaggedValue $artifact "wsdlFile(. path .)" ]        
        if { ($v != 0) && ([u2::IsKindOf $v ValueExpr] != 0)} {
            set f [u2::GetValue $v ValueString]            
            InvokeExportAgent $artifact $f
        }
        return 0
    }
    
    # no <<WSDLFile>> artifact for the context
    # it will be create by the agent
    if { [u2::IsKindOf $sel Package ] } {
        set defaultName "[u2::GetValue $sel Name].wsdl"
        set res [std::FileSaveDialog -filename $defaultName -fileext "wsdl"]
        if { $res != 0 } {
            InvokeExportAgent $sel $res
        }
    }
}

proc CreateMenu { } {
    AddCommand -variable cmdImportWsdl -name "Import WSDL..." -statusmessage "Importing WSDL specification." -tooltip "Import WSDL..." -imagefile "" -OnActivateCommand OnImportWSDL -OnEnableCommand OnEnableImportWSDL
    AddCommand -variable cmdExportWsdl -name "Generate WSDL..." -statusmessage "Generating WSDL specification from UML model." -tooltip "Generate WSDL..." -imagefile "" -OnActivateCommand OnExportWSDL -OnEnableCommand OnEnableGenerateWSDL
    
    AddContextMenu -variable cmnWSDL -commands {separator cmdImportWsdl cmdExportWsdl } -onenablemenu { OnEnableWSDL }
}

proc ActivateMetaModel { } {   
    set models [std::GetModels -kind U2]
    set metamodel 0
    foreach m $models {
        set metamodel [u2::FindByGuid $m "@TTDWSDLView"]
    }
    if { $metamodel != 0} {
        u2::SelectMetaModel $metamodel
    }
}

proc Init {} {
    
    LoadLib "TTDXMLFramework.u2" "XML Framework"
    LoadLib "xsd.u2" "XSD"
    LoadLib "wsdl.u2" "WSDL"
    LoadLib "soap.u2" "SOAP"
    
    CreateMenu
    # ActivateMetaModel
}


proc BeforeUnload { } {
#    UnloadLib "@soap"
#    UnloadLib "@wsdl"
#    UnloadLib "@xsd"
#    UnloadLib "@TTDXMLFramework"
}

Init
