/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:24 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/

/**********************************************************************/
/* communication module for the debug agent with the debug instrument */


/*
The sdt/alfa/tcpip package seems to be designed only for
a multithread environment, so it can not be re-used here,
except the tcpipsock.h part, which provides a UNIX/Windows portable API
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <assert.h> /* to be removed */

#ifdef _Windows
#define THREADWIN32 1
#else
#define THREADSOLARIS 1
#endif

#if _MSC_VER >= 1400
#pragma warning (disable : 4996)  // _CRT_SECURE_NO_DEPRECATE
#endif

#ifndef DEBUG_INSTRUMENT_SIDE

/* compilation within a Tau installation */
#include "../tcpip/tcpipsock.h"

#define SCM_ERROR_REPORT(ERROR_MSG, ERROR_CODE) \
	( ERROR_CODE!=0 ? \
		fprintf(stderr, "sctdacom.c: " ERROR_MSG " (%s)\n", strerror(ERROR_CODE) ) \
	: \
		fprintf(stderr, "sctdacom.c: " ERROR_MSG "\n") \
	);


#else

/* compilation with a development environment */
/* we have a -I to the sdt/alfa hierachy */
#include "tcpipsock.h"

/* Send an error message back to the debugger in a thread-safe way. */
/* DoErrorReport is defined in DbgInstrument.cpp. */
extern /*"C"*/ DoErrorReport(const char *ERROR_MSG, int ERROR_CODE);

/* on Windows ERROR_CODE is a function call, so it can not processed by a macro */
#define SCM_ERROR_REPORT(ERROR_MSG, ERROR_CODE) DoErrorReport(ERROR_MSG, ERROR_CODE);

#endif

/* TODO: fix above macros:
	- exit not good in a thread
	- perror is too late after fprintf
*/

#define XSERVPORT xGetServPort()

int                                 xSCMInitialized;

#include "sctdacom.h"

/**********************************************************************/

void
CB_clear(Charbuf *cb)
{
        assert(cb->size!=0);
        cb->buf[0] = 0;
        cb->reading = 0;
        cb->eof = /*FALSE*/0;
        cb->writing = 0;
}

void
CB_init(Charbuf *cb)
{
        cb->size = 80;
        cb->buf = (char*)malloc(80);
        CB_clear(cb);
}

void
CB_reset(Charbuf *cb)
{
        free(cb->buf);
        cb->buf = NULL;
        cb->size = 0;
}

/* writing functions */

void
CB_putc(Charbuf *cb,char c)
{
        if(cb->writing>=cb->size)
        {
                cb->buf = (char*)realloc(cb->buf,2*cb->size);
                cb->size *= 2;
        }
        cb->buf[cb->writing] = c;
        ++cb->writing;
}

void
CB_unputc(Charbuf *cb,char c)
{
        if(cb->writing>0 && c==cb->buf[cb->writing-1])
	{
        	--cb->writing;
	}
}

void
CB_puts(Charbuf *cb,const char *s)
{
        /* a optimiser */
        while(*s)
        {
                CB_putc(cb,*s++);
        }
}


void
CB_nputc(Charbuf *cb,const char *s,int l)
{
        int i;

        /* a optimiser */
        for(i=0;i<l;i++)
        {
                /*assert(*s!=0);*/
                CB_putc(cb,*s++);
        }
}

/* reading functions */

/* #define CB_eof(cb) ((cb)->eof) */

int
CB_getc(Charbuf *cb)
{
        int res;
        if(CB_eof(cb))
        {
                return 0;
        }
        res =  cb->buf[cb->reading];
        if(res==0)
        {
                cb->eof = /*TRUE*/1;
        }else{
                ++cb->reading;
        }
        return res;
}


/* mapping from tBuffer to Charbuf */

typedef Charbuf tBuffer;

#define BufInitBuf(xSigBuf, bms_SmallBuffer) CB_init(&xSigBuf)
#define BufInitWriteMode(xSigBuf) xSigBuf.writing = 0
#define BufPutSeg(xSigBuf, xRecvBuf, xTotRecvRes) CB_nputc(&xSigBuf,xRecvBuf,xTotRecvRes)
#define BufCloseWriteMode(xSigBuf) /*nothing*/

#define   BufInitReadMode(xSigBuf) xSigBuf.reading = 0

#define BufCloseReadMode(xSigBuf) /*nothing*/

#define BufCloseBuf(xSigBuf) CB_reset(&xSigBuf)


/**********************************************************************/
/* common code to DEBUG_INSTRUMENT_SIDE and !DEBUG_INSTRUMENT_SIDE */

static int xGetServPort()
{
	static int getenv_done = 0;
	static int port;
	const char *s = NULL;
	if (!getenv_done) {
		s = getenv(ENV_TTD_DEBUGGER_PORT);
		if (s) {
			sscanf(s,"%d",&port);
		} else {
			port = XDEFAULTSERVPORT;
		}
		getenv_done = 1;
	}
	return port;
}

/* returns -1 in case of error, otherwise the number of bytes read */
/* char* (*res) should be allocated before calling do_receive function */
static int
do_receive(SCM_SOCKET xRecvSocket, char **res, int *xIgnoreErrors)
{
  int xBufSize = SCM_RECVBUF_MAXSIZE;
  // Buffer of fixed size is allocated in U2MV and used for all calls to this function
  int xRecvRes = 0;
  int xTotRecvRes = 0;
  char *xBufPos = *res;

  memset(*res, 0, xBufSize);
  xRecvRes = SCM_RECV(xRecvSocket, xBufPos, xBufSize, 0);
  xTotRecvRes += xRecvRes;
  
  if(xRecvRes==0) {
    /* a shutdown was performed by the client */
    return -1;
  }

#if 0
  while ((xRecvRes!= SCM_SOCKET_ERROR) && (xRecvRes != 0))
  {

#ifdef THREADLIGHT_TRACE
    printf("Received: %d %s\n", xRecvRes, xBufPos);
    printf("Receive buffer: %s\n", xRecvBuf);
#endif

    xBufPos += xRecvRes;
    
    if ((xBufPos - xRecvBuf) == xBufSize)
    {
      /* The receive buffer is full and there is data left to receive - reallocate the buffer */
      xBufSize = 2 * xBufSize;
      xRecvBuf = (char*)realloc(xRecvBuf, xBufSize);
      xBufPos = xRecvBuf + xRecvRes;
      memset(xBufPos, 0, xBufSize - xTotRecvRes);

      if (xRecvBuf == 0)
      {
        if(!xIgnoreErrors || !*xIgnoreErrors) SCM_ERROR_REPORT("realloc failed for receive buffer", 0)
        free(xRecvBuf);
        return -1;
      }
    }

    xRecvRes = SCM_RECV(xRecvSocket, xBufPos, xBufSize, 0);
    xTotRecvRes += xRecvRes;
  }
#endif

  if (xRecvRes == SCM_SOCKET_ERROR)
  {
    /* Error */
    if(!xIgnoreErrors || !*xIgnoreErrors) SCM_ERROR_REPORT("SCM_RECV failed", SCM_GET_ERROR)
    return -1;
  }

  /* When xRecvRes == 0, the whole message has been received */

  return xTotRecvRes;
}

/**********************************************************************/
/* code derived from cosim/ccsotool.c */

#ifndef DEBUG_INSTRUMENT_SIDE

/* debug agent (application side) */

#define SCM_EXIT_THREAD exit(1);
/* TODO: replace SCM_EXIT_THREAD by appropriate code according to the context */

struct ComCtx_struct {
  SCM_SOCKET xHostSocket;
  SCM_SOCKET xNewConnectionSocket;
};


/*
  Sets up a TCP/IP socket server which waits for connection attempt
*/
static void xDAComInit2(struct ComCtx_struct *ctx)
{
  int xSockOptionOn = 1;
  int xServerPort;

  int BindRes;

  SCM_SOCKET_ADDR xHostAddress;

  /*SCM_THREAD_VARS*/
  /*SCM_THREAD xNewThread;*/

  xServerPort = XSERVPORT;

#ifdef THREADLIGHT_TRACE
  /*  printf("WaitForSignal thread started. Using port %d\n", xServerPort); */
#endif


  memset((char*)&xHostAddress, 0, sizeof(xHostAddress));
  xHostAddress.sin_family = AF_INET;
  xHostAddress.sin_addr.s_addr = SCM_HTONL(INADDR_ANY);

  /* Convert to network byte order */
#ifdef THREADVXWORKS
  xHostAddress.sin_len = sizeof(struct sockaddr_in);
  xHostAddress.sin_port = (unsigned short)xServerPort;
#else
  xHostAddress.sin_port = SCM_HTONS((unsigned short)xServerPort);
#endif

  /* Create socket */
  ctx->xHostSocket = SCM_CREATE_SOCKET(AF_INET, SOCK_STREAM, 0);
  if (ctx->xHostSocket == SCM_INVALID_SOCKET)
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_CREATE_SOCKET failed", SCM_GET_ERROR)
    SCM_EXIT_THREAD
  }

  /* Make socket port reusable immediately
     (Convenient when restoring broken connections quickly) */
  if (SCM_SETSOCKOPT(ctx->xHostSocket , SOL_SOCKET, SO_REUSEADDR, (char*)&xSockOptionOn, sizeof(xSockOptionOn)) == SCM_SOCKET_ERROR)
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_SETSOCKOPT failed", SCM_GET_ERROR)
    SCM_CLOSE(ctx->xHostSocket);
    SCM_EXIT_THREAD
  }


  /* bind */
  BindRes = SCM_BIND(ctx->xHostSocket, SCM_PTR_SOCK_ADDR &xHostAddress, sizeof(xHostAddress));
  if (BindRes == -1)  
  /*
    if (SCM_BIND(ctx->xHostSocket, SCM_PTR_SOCK_ADDR &xHostAddress, sizeof(xHostAddress)) == SCM_SOCKET_ERROR)*/
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_BIND failed", SCM_GET_ERROR)
    SCM_CLOSE(ctx->xHostSocket);
    SCM_EXIT_THREAD
  }

  /* Initialization finished - unlock mutex */
  /*SCM_UNLOCK_THREAD_MUTEX(xReceiverMutex)*/

  if (xServerPort==0) {
    /* a port was chosen by "bind", let us make this choice public (in a file) */
    int AutoPort;
    const char *PortFilename;
    char *PortFilename2;
    FILE *f;
    int st;
#ifdef _Windows
    int len;
#else
    socklen_t len;
#endif
 
    len = sizeof(xHostAddress);
    st = getsockname(ctx->xHostSocket, SCM_PTR_SOCK_ADDR &xHostAddress, &len);
    if (st) {
      SCM_ERROR_REPORT("getsockname failed", SCM_GET_ERROR)
      SCM_CLOSE(ctx->xHostSocket);
      SCM_EXIT_THREAD
    }

#ifdef THREADVXWORKS
  AutoPort = xHostAddress.sin_port;
#else
  AutoPort = SCM_NTOHS(xHostAddress.sin_port); 
#endif

    PortFilename = getenv(ENV_TTD_DEBUGGER_PORT_FILE);
    if (!PortFilename) {
      SCM_ERROR_REPORT("No Port file specified", 0)
      SCM_CLOSE(ctx->xHostSocket);
      SCM_EXIT_THREAD
    }
    PortFilename2 = (char *)malloc(strlen(PortFilename) + 1);
    strcpy(PortFilename2, PortFilename);
    PortFilename2[strlen(PortFilename2)-1] = '2';
    f = fopen(PortFilename2,"w");
    if (!f) {
      SCM_ERROR_REPORT("Opening port file failed", 0)
      SCM_CLOSE(ctx->xHostSocket);
      SCM_EXIT_THREAD
    }
    fprintf(f,"%ld",(long)AutoPort);
    fclose(f);
    rename(PortFilename2,PortFilename);
    free(PortFilename2);
  }

  /* listen */
  if (SCM_LISTEN(ctx->xHostSocket) == SCM_SOCKET_ERROR)
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_LISTEN failed", SCM_GET_ERROR)
    SCM_CLOSE(ctx->xHostSocket);
    SCM_EXIT_THREAD
  }

  /*while (1)*/
  {
    /* Accept incoming connection attempts */
#ifdef _Windows
    int xAddrSize;
#else
    socklen_t xAddrSize;
#endif

    xAddrSize = sizeof(xHostAddress);

    ctx->xNewConnectionSocket = SCM_ACCEPT(ctx->xHostSocket, SCM_PTR_SOCK_ADDR &xHostAddress, &xAddrSize);

    if (ctx->xNewConnectionSocket == SCM_SOCKET_ERROR)
    {
      /* Error */
      SCM_ERROR_REPORT("SCM_ACCEPT failed", SCM_GET_ERROR)
    }
    else
    {
      /* Create a thread to receive the data */
      /*SCM_START_THREAD(xNewThread, xReceiveSignal, ctx->xNewConnectionSocket, XRECVTHRSTACK, XRECVTHRPRIO)*/
      /* ctx->xNewConnectionSocket used by later call to xDAComPoll() */
    }
  }
    
}

/* Server initialization */
extern struct ComCtx_struct * xDAComInit(void)
{
	struct ComCtx_struct *ctx;

	ctx = (struct ComCtx_struct *)calloc(1,sizeof(*ctx));

/* SCM_STARTUP includes a "return" without value */
#define return return NULL
	{ SCM_STARTUP }
#undef return

	xDAComInit2(ctx);

	return ctx;
}

void
xDAComPoll(struct ComCtx_struct *ctx,void (*cb)(/*nodelete*/ char *cmd,/*nodelete*/ char *reqid),long timeout)
{
	fd_set read_fds;
	int n_ready;
	struct timeval timeout_struct;

	FD_ZERO(&read_fds);
	FD_SET(ctx->xNewConnectionSocket, &read_fds);

	timeout_struct.tv_sec = 0;
	timeout_struct.tv_usec = timeout;

	n_ready = select(ctx->xNewConnectionSocket+1, &read_fds, 0, 0, &timeout_struct);

	if (n_ready < 0) {
		perror("select");
		exit(1);
	}

	if(!FD_ISSET(ctx->xNewConnectionSocket,&read_fds)) {
		return;
	}

/* inlined old function xReceiveSignal() */
{
  void *xSocket = (void *)ctx->xNewConnectionSocket;
  SCM_SOCKET xRecvSocket;

  char *xRecvBuf = (char*) malloc (SCM_RECVBUF_MAXSIZE);
  
  tBuffer xSigBuf;
  int xTotRecvRes = 0;

#ifdef THREADOSE
  SCM_GET_THREAD_VAR(xRecvSocket);
#elif THREADVXWORKS
  xRecvSocket = xSocket;
#else
  xRecvSocket = (SCM_SOCKET)xSocket;
#endif

#ifdef THREADLIGHT_TRACE
  printf("xReceiveSignal thread starts ", xRecvSocket );
#endif

  xTotRecvRes = do_receive(xRecvSocket,&xRecvBuf,NULL);
  if(xTotRecvRes<0) {
    /* it is too late to call SCM_GET_ERROR */
    SCM_ERROR_REPORT("shutdown detected", 0)
    SCM_EXIT_THREAD
  }

  /*xSigBuf = 0;*/
  BufInitBuf(xSigBuf, bms_SmallBuffer);
/*  BufInitBuf(xSigBuf, SML_BUF);*/
  /*memset(xSignalId, 0, 128);*/

  BufInitWriteMode(xSigBuf);
  BufPutSeg(xSigBuf, xRecvBuf, xTotRecvRes);
  BufCloseWriteMode(xSigBuf);

  free(xRecvBuf);

  BufInitReadMode(xSigBuf);

  xDAMsgHandleData(xSigBuf.buf,xTotRecvRes,cb);

  BufCloseReadMode(xSigBuf);

  BufCloseBuf(xSigBuf);
  /*free(xRecvBuf);*/

}

}

int/*xbool*/
xDAComUp(struct ComCtx_struct *ctx)
{
	if(!ctx) return 0/*xFalse*/;
	if(!ctx->xNewConnectionSocket) return 0/*xFalse*/;
	return 1/*xTrue*/;
}

int/*xbool*/
xDAComSend(struct ComCtx_struct *ctx,const char *msg)
{
	int xLengthSent;
	int xSignalLength;

	if(!ctx) return 0/*xFalse*/;
	if(!ctx->xNewConnectionSocket) return 0/*xFalse*/;


	/* +1 to send the ending zero */
	xSignalLength = strlen(msg)+1;


  /* Send the signal on the socket
     Check that everything was sent */

  xLengthSent = SCM_SEND(ctx->xNewConnectionSocket, msg, xSignalLength, 0);

  if ( xLengthSent == SCM_SOCKET_ERROR)
  {
    /* Error */
    /* TODO: should be continue debugging ? SCM_CLOSE(ctx->xNewConnectionSocket);*/
    SCM_ERROR_REPORT("SCM_SEND failed", SCM_GET_ERROR)
    return 1/*xTrue*/;
  }
  else if (xLengthSent < xSignalLength)
  {
    /* Only part of the message was sent */
    /* TODO: should be continue debugging ? SCM_CLOSE(ctx->xNewConnectionSocket);*/
    /* TODO: retry if necessary */
    SCM_ERROR_REPORT("SCM_SEND failed - Not whole message sent", SCM_GET_ERROR)
    return 1/*xTrue*/;
  }

  /* SCM_SEND has returned xSignalLength. The whole message was successfully sent. */

#ifdef THREADLIGHT_TRACE
  printf("Signal sent: %s\n", signal);
#endif

	return 1/*xTrue*/;
}


/****************************************************************************************/
#else
/****************************************************************************************/

/* debug instrument (host side) */

#define SCM_EXIT_THREAD return;


struct DbgComCtx {
  SCM_SOCKET xClientSocket;
  
  SCM_SOCKET_ADDR    xHostAddress;
  SCM_HOST_ENTRY     xHostEntry;
  int PortNumber;
  int xIgnoreErrors; /* true when SCM_ERROR_REPORT should not be called */
};

int
DbgComPort(struct DbgComCtx *ctx)
{
	return ctx->PortNumber;
}

void
DbgComIgnoreErrors(struct DbgComCtx *ctx)
{
	ctx->xIgnoreErrors = 1;
}

struct DbgComCtx *
DbgInitComm(const char *xHostName, int xPortNbr)
{
  struct DbgComCtx *ctx;

  int xSockOptionOn = 1;
  int xSockOptionOff = 0;

#ifndef _Windows
struct hostent host_info;
/*struct hostent_data host;*/
char host[4096];
int err;
#endif

/* SCM_STARTUP includes a "return" without value */
#define return return NULL
  { SCM_STARTUP }
#undef return

  if (xPortNbr==-1) xPortNbr = XSERVPORT;

  /* Check that the host is correct */

  ctx = (struct DbgComCtx *)calloc(1,sizeof(*ctx));
  ctx->PortNumber = xPortNbr;

#ifdef _Windows
  ctx->xHostEntry = SCM_GETHOSTBYNAME(xHostName);
#else
  ctx->xHostEntry = gethostbyname_r(xHostName,&host_info,host,4096,&err);
#endif

#ifdef THREADVXWORKS
  if (ctx->xHostEntry == ERROR)
#else
  if ((char*)ctx->xHostEntry == NULL)
#endif
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_GETHOSTBYNAME failed", SCM_GET_ERROR)
    return NULL/*SCM_GET_ERROR*/;
  }

  memset((char*)&ctx->xHostAddress, 0, sizeof(ctx->xHostAddress));
  ctx->xHostAddress.sin_family = AF_INET;

#ifdef THREADVXWORKS
  ctx->xHostAddress.sin_addr.s_addr = ctx->xHostEntry;
#else
  memcpy(&ctx->xHostAddress.sin_addr, ctx->xHostEntry->h_addr, ctx->xHostEntry->h_length);
#endif

  /* The port number is assumed to be correct */
  /* Convert to network byte order */
  ctx->xHostAddress.sin_port = SCM_HTONS((unsigned short)xPortNbr);


  /* Create socket */
  ctx->xClientSocket = SCM_CREATE_SOCKET(AF_INET, SOCK_STREAM, 0);
  if (ctx->xClientSocket == SCM_INVALID_SOCKET)
  {
    /* Error */
    SCM_ERROR_REPORT("SCM_CREATE_SOCKET failed", SCM_GET_ERROR)
    return NULL/*SCM_GET_ERROR*/;
  }

#ifdef THREADLIGHT_TRACE
  printf("Socket created\n");
#endif

  /* Make socket port reusable immediately
    (Convenient when restoring broken connections quickly) */
  if (SCM_SETSOCKOPT(ctx->xClientSocket , SOL_SOCKET, SO_REUSEADDR, (char*)&xSockOptionOn, sizeof(xSockOptionOn)) == SCM_SOCKET_ERROR)
  {
    /* Error */
    SCM_CLOSE(ctx->xClientSocket);
    SCM_ERROR_REPORT("SCM_SETSOCKOPT failed", SCM_GET_ERROR)
    return NULL/*SCM_GET_ERROR*/;
  }

  /* Connect to the receiver */
  if (SCM_CONNECT(ctx->xClientSocket, SCM_PTR_SOCK_ADDR &ctx->xHostAddress, sizeof(ctx->xHostAddress)) == SCM_SOCKET_ERROR)
  {
    /* Error */
    SCM_CLOSE(ctx->xClientSocket);
    SCM_ERROR_REPORT("SCM_CONNECT failed", SCM_GET_ERROR)
    return NULL/*SCM_GET_ERROR*/;
  }

#ifdef THREADLIGHT_TRACE
  printf("Connection accepted\n");
#endif

  return ctx;
}

int do_send(struct DbgComCtx *ctx,const char *cmd)
{
  int xLengthSent;
  int xSignalLength;

  xSignalLength = strlen(cmd)+1;


  /* Send the signal on the socket
     Check that everything was sent */

  xLengthSent = SCM_SEND(ctx->xClientSocket, cmd, xSignalLength, 0);
  
  if ( xLengthSent == SCM_SOCKET_ERROR)
  {
    /* Error */
    SCM_CLOSE(ctx->xClientSocket);
    SCM_ERROR_REPORT("SCM_SEND failed", SCM_GET_ERROR)
    return SCM_GET_ERROR;
  }
  else if (xLengthSent < xSignalLength)
  {
    /* Only part of the message was sent */
    SCM_CLOSE(ctx->xClientSocket);
    SCM_ERROR_REPORT("SCM_SEND failed - Not whole message sent", SCM_GET_ERROR)
    return -1;
  }

  /* SCM_SEND has returned xSignalLength. The whole message was successfully sent. */

#ifdef THREADLIGHT_TRACE
  printf("Signal sent: %s\n", signal);
#endif

  return 0;
}

int DbgBreakComm(struct DbgComCtx *ctx)
{

  /* shutdown of read and write */
  if (SCM_SHUTDOWN(ctx->xClientSocket, SCM_SHUT_SENDRECV) == SCM_SOCKET_ERROR)
  {
    SCM_CLOSE(ctx->xClientSocket);
    SCM_ERROR_REPORT("SCM_SHUTDOWN failed", SCM_GET_ERROR)
    return SCM_GET_ERROR;
  }


  /* Close the connection */
  if (SCM_CLOSE(ctx->xClientSocket) == SCM_SOCKET_ERROR)
  {
    SCM_ERROR_REPORT("SCM_CLOSE failed", SCM_GET_ERROR)
    return SCM_GET_ERROR;
  }

  return 0;
}


int DbgSendReq(struct DbgComCtx *ctx,const char *cmd)
{
  int xRetCode;

  /* Error handling is taken care of in xSendBuf: no check on return value */
  xRetCode = do_send(ctx,cmd);

  return xRetCode;
}

int
DbgGetString(struct DbgComCtx *ctx,char **buf)
{
	int lbuf;

	lbuf = do_receive(ctx->xClientSocket,buf,&ctx->xIgnoreErrors);
	return lbuf;
}

#ifdef TEST

#ifdef OLD_MAIN

int
main(int argc,char *argv[])
{
  int st;

  char cmd[100];


	st = InitComm("localhost\0");
	assert(st==0);

	while(1) {
		if(!fgets(cmd,99,stdin)) {
			break;
		}

		st = DbgSendReq(cmd,NULL);
		assert(st==0);

	}

	DbgBreakComm();

	return 0;
}

#else

int
main(int argc,char *argv[])
{
  char xDestIP[128];
  int xDestPort;
  int st;

  char cmd[100];
	fd_set read_fds;
	int n_ready;
	int nbr_descr;
	char *buf = (char*) malloc (SCM_RECVBUF_MAXSIZE);;
	int lbuf;



  strcpy(xDestIP, "localhost\0");
  xDestPort = XSERVPORT;

	st = DbgInitComm(xDestIP,xDestPort);
	assert(st==0);

	while(1) {
		FD_ZERO(&read_fds);
		FD_SET(fileno(stdin), &read_fds);
		FD_SET(ctx->xClientSocket, &read_fds);
#define imax(i1,i2) ((i1)>(i2) ? (i1) : (i2))
		nbr_descr = 1+imax(fileno(stdin),ctx->xClientSocket);
#undef imax

		n_ready = select(nbr_descr, &read_fds, 0, 0, NULL/*no timeout*/);

		if (n_ready < 0) {
			perror("select");
			exit(1);
		}

		if(FD_ISSET(fileno(stdin),&read_fds)) {
		/* stdin is usually lined-buffered, so it should be possible to read a line
		   (on the FILE level) without blocking */
			if(!fgets(cmd,99,stdin)) {
				perror("fgets");
				break;
			}
			st = DbgSendReq(cmd,NULL);
			assert(st==0);
		}else{
			assert(FD_ISSET(ctx->xClientSocket,&read_fds));
			lbuf = do_receive(ctx->xClientSocket,&buf,NULL);
			assert(lbuf!=0);
			/* just display it */
			fwrite(buf,1,lbuf,stdout);
			fflush(stdout); /* for prompt case */
		}

	}

	DbgBreakComm();
	free(buf);

	return 0;
}

#endif /* OLD_MAIN */

#endif /* TEST */

#endif

/**********************************************************************/


