<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.2 to 2.3          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.3 -->

<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.3</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<!-- replace UseCaseCollaboration with Collaboration -->
<xsl:template match = "UseCaseCollaboration">
  <Collaboration>
	<xsl:call-template name="copy-all-attributes"/>
        <xsl:apply-templates/>
  </Collaboration>
</xsl:template>

<!-- replace Actor with Attribute having the Actor stereotype applied -->
<!-- TODO: finalize -->
<xsl:template match = "Actor">
  <Attribute>
	<xsl:call-template name="copy-all-attributes"/>
    <cStereotypeInstance>
	  <InstanceExpr>
	    <xsl:apply-templates/>
	  </InstanceExpr>
    </cStereotypeInstance>
  </Attribute>
</xsl:template>

<!-- Apply AutomaticLineStyle to the lines in TTD -->
<xsl:template match = "GeneralizationLine | FlowLine | AssociationLine | ExtensionLine">
  <xsl:copy>
    <cStereotypeInstance>
      <InstanceExpr>
        <rInstanceOf>
		  <xsl:attribute name = "R">uid:@TTDLineStyles@automaticLineStyle</xsl:attribute>
		</rInstanceOf>
      </InstanceExpr>
    </cStereotypeInstance>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Convert debugger markers (change GUID refs) -->
<xsl:template match = "InstanceExpr/rInstanceOf">
  <xsl:copy>
    <xsl:choose>
	  <xsl:when test = 'string(@R) = "uid:pIGwhL*67xILfxCZsEZGiWNV"'>
	    <xsl:attribute name = "R">uid:@TTDDebugger@debuggerMarker</xsl:attribute>
	  </xsl:when>
	  <xsl:when test = 'string(@R) = "ref:DebuggerMarker"'>
	    <xsl:attribute name = "R">ref:debuggerMarker</xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:attribute name = "R"><xsl:value-of select = "@R"/></xsl:attribute>
	  </xsl:otherwise>
	</xsl:choose>
  </xsl:copy>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>
