<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.3 to 2.4          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.3 -->

<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove ReturnAction::ConnectionPoint -->
<xsl:template match = "ReturnAction/rConnectionPoint">
</xsl:template>

<!-- remove Virtuality from Attributes - only Parameters can have them now -->
<xsl:template match = "Attribute">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Virtuality'">
        <!-- do not copy this attribute -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- change all UseCases to Operations, because UseCase was removed -->
<xsl:template match = "UseCase">
  <Operation>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </Operation>
</xsl:template>

<!-- ... and rename all features with UseCase in its name - it was changed to Operation (why?) -->
<xsl:template match = "LifeLine/rUsecase | InteractionOperand/rUsecase | 
					   InteractionOccurrence/rUsecase">
  <rOperation>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rOperation>
</xsl:template>

<!-- rename ActionOccurrence::occurenceOf in ::occurrenceOf (correct typo) -->
<xsl:template match = "CompoundActionOccurrence/rOccurenceOf | Destroy/rOccurenceOf |
					   InstanceDelete/rOccurenceOf | MessageReceive/rOccurenceOf |
					   InstanceCreateReceive/rOccurenceOf | OperationCallReceive/rOccurenceOf |
					   OperationReplyReceive/rOccurenceOf | MessageSend/rOccurenceOf |
					   InstanceCreateSend/rOccurenceOf | OperationCallSend/rOccurenceOf |
					   OperationReplySend/rOccurenceOf | NextStateActionOccurrence/rOccurenceOf |
					   TimerReset/rOccurenceOf | TimerSet/rOccurenceOf | TimerTimeout/rOccurenceOf">
  <rOccurrenceOf>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rOccurrenceOf>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>
