<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.4 to 2.5          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.5 -->

<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.5</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- rename InteractionSymbol to InlineFrameSymbol -->
<xsl:template match = "InteractionSymbol">
  <InlineFrameSymbol>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </InlineFrameSymbol>
</xsl:template>

<!-- Removed assert expression - change them to call -->
<xsl:template match = "AssertExpr">
  <CallExpr>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </CallExpr>
</xsl:template>

<xsl:template match = "AssertExpr/cCondition">
  <cArgument>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cArgument>
</xsl:template>

<!-- Remove SignatureSymbol::showAllFeatures -->
<xsl:template match = "DataTypeSymbol | ClassSymbol | InterfaceSymbol | StereotypeSymbol | 
		ExceptionSymbol | OperationSymbol | StateMachineSymbol | SignalSymbol | TimerSymbol">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='ShowAllFeatures'">
		<!-- do not copy this attribute -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Removed RemarkSymbol - use CommentSymbol instead -->
<xsl:template match = "RemarkSymbol">
  <CommentSymbol>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </CommentSymbol>
</xsl:template>

<!-- Move FormalGate composition from Interaction to Operation -->
<xsl:template match = "Operation | StateMachine">
  <xsl:copy>
	<cFormalGate>
	  <xsl:for-each select="cInlineMethod/Interaction/cFormalGate">
		<xsl:call-template name="copy-all-attributes"/>
		<xsl:apply-templates/>
	  </xsl:for-each>
	</cFormalGate>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match = "cInlineMethod/Interaction/cFormalGate">
</xsl:template>

<!-- TextDiagram no longer inherits from TextContainer - remove TextContainer features -->
<xsl:template match = "TextDiagram">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Text'">
		<!-- do not copy this attribute -->
        </xsl:when>
        <xsl:when test="local-name()='IsParseError'">
		<!-- do not copy this attribute -->
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- DefAction now can contain any definition -->
<xsl:template match = "DefAction/cVariable">
  <cDefinition>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cDefinition>
</xsl:template>

<!-- InformalConstraint now inherits from InformalEntity -->
<xsl:template match = "InformalConstraint">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='Text'">
		  <xsl:attribute name = "InformalText">
		    <xsl:value-of select = "."/>
		  </xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>
