<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.5 to 2.6          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.6 -->
<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.6</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove PackageUse - transform it into Dependency with 'access' stereotype-->
<xsl:template match = "cUse">
  <cClientDependency>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cClientDependency>
</xsl:template>

<xsl:template match = "PackageUse">
  <Dependency>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
	<cStereotypeInstance>
	  <InstanceExpr>
	    <rInstanceOf>
		  <xsl:attribute name = "R">ref:access</xsl:attribute>
		</rInstanceOf>
	  </InstanceExpr>
	</cStereotypeInstance>
  </Dependency>
</xsl:template>

<xsl:template match = "PackageUse/rUsedPackage">
  <rSupplier>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </rSupplier>
</xsl:template>

<!-- Remove listed elements -->
<xsl:template match = "PackageUse/rListedElement">
</xsl:template>

<!-- remove PackageUseSymbol -->
<xsl:template match = "PackageUseSymbol">
</xsl:template>

<!-- Renamed BuildObject into Artifact -->
<xsl:template match = "BuildObject | BuildFile">
  <Artifact>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </Artifact>
</xsl:template>

<!-- Rename cBuildArtifact to cSignature, since now BuildArtifact is not that special -->
<xsl:template match = "cBuildArtifact">
  <cSignature>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cSignature>
</xsl:template>

<!-- Remove obsolete features from BuildArtifact / BuildObject -->
<xsl:template match = "BuildObject/cBuildPath | BuildFile/cBuildPath">
</xsl:template>

<xsl:template match = "BuildObject/cBuildTask | BuildFile/cBuildTask">
</xsl:template>

<xsl:template match = "BuildObject/cBuildArtifact | BuildFile/cBuildArtifact">
  <cSignature>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cSignature>
</xsl:template>

<!-- mrRoot transformed into dependency with 'manifest' stereotype -->
<xsl:template match = "BuildObject/rRoot | BuildFile/rRoot">
  <cClientDependency>
    <Dependency>
	  <rSupplier>
		<xsl:call-template name="copy-all-attributes"/>
		<xsl:apply-templates/>
	  </rSupplier>
	  <cStereotypeInstance>
	    <InstanceExpr>
	      <rInstanceOf>
		    <xsl:attribute name = "R">ref:manifest</xsl:attribute>
		  </rInstanceOf>
	    </InstanceExpr>
	  </cStereotypeInstance>
	</Dependency>
  </cClientDependency>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>
