<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.8 (4) to 2.9 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:u2="http://www.telelogic.com/tau"
   exclude-result-prefixes='u2'>

<!-- Change file format version number to 2.8 -->
<xsl:template match="u2 | u2:u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.9</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- renamed AssociationLine::roleLabels to AssociationLine::roleLabel -->
<xsl:template match = "AssociationLine/cRoleLabels | u2:AssociationLine/u2:cRoleLabels">
  <cRoleLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cRoleLabel>
</xsl:template>

<!-- Inheritance to Namespace was moved down from Signature to Operation, Datatype and StructuredClassifier -->
<xsl:template match = "Syntype/cOwnedMember | Signal/cOwnedMember | Timer/cOwnedMember |
						u2:Syntype/u2:cOwnedMember | u2:Signal/u2:cOwnedMember | u2:Timer/u2:cOwnedMember">
</xsl:template>

<!-- CreateExpr::constructor is now derived -->
<xsl:template match = "CreateExpr/rConstructor | u2:CreateExpr/u2:rConstructor">
</xsl:template>

<!-- Changes in arguments (introduction of ActualArgumentContainer interface) -->
<xsl:template match = "Attribute/cConstructorArgument | u2:Attribute/u2:cConstructorArgument">
  <cArgument>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cArgument>
</xsl:template>

<!-- Session now inherits Namespace. The Session::root attribute was replaced by the "ownedMember" -->
<xsl:template match="Session/cRoot | u2:Session/u2:cRoot">
  <cOwnedMember>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cOwnedMember>
</xsl:template>

<!-- Changes in format of CppImport stereotype -->
<xsl:template match="cStereotypeInstance/InstanceExpr/cExpression/BinaryExpr[../../rInstanceOf[@R='uid:@TTDCppImport@cppImportSpecification']][cLeftOperand/Ident[@Name='dialect']] | 
    u2:cStereotypeInstance/u2:InstanceExpr/u2:cExpression/u2:BinaryExpr[../../u2:rInstanceOf[@R='uid:@TTDCppImport@cppImportSpecification']][u2:cLeftOperand/u2:Ident[@Name='dialect']]">
  <xsl:choose>
    <!-- remove BinaryExpr if dialect is not set... -->
    <xsl:when test = "(count(cRightOperand/InstanceExpr/cExpression) &lt; 1) and 
	                  (count(u2:cRightOperand/u2:InstanceExpr/u2:cExpression) &lt; 1)">
	</xsl:when>
	<xsl:otherwise>
	  <BinaryExpr>
	    <xsl:apply-templates/>
        <xsl:call-template name="copy-all-attributes"/>
	  </BinaryExpr>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="cStereotypeInstance/InstanceExpr/cExpression/BinaryExpr/cRightOperand/InstanceExpr[../../../../rInstanceOf[@R='uid:@TTDCppImport@cppImportSpecification']][../../cLeftOperand/Ident[@Name='dialect']] | 
    u2:cStereotypeInstance/u2:InstanceExpr/u2:cExpression/u2:BinaryExpr/u2:cRightOperand/u2:InstanceExpr[../../../../u2:rInstanceOf[@R='uid:@TTDCppImport@cppImportSpecification']][../../u2:cLeftOperand/u2:Ident[@Name='dialect']]">
  <xsl:for-each select="cExpression/BinaryExpr/cLeftOperand/Ident | u2:cExpression/u2:BinaryExpr/u2:cLeftOperand/u2:Ident">
    <xsl:copy>
      <xsl:call-template name="copy-all-attributes"/>
	</xsl:copy>
  </xsl:for-each>
</xsl:template>

<!-- Remove isExternal attribute from 'C Application' stereotype -->
<xsl:template match = 'cStereotypeInstance/InstanceExpr/cExpression/BinaryExpr[../../rInstanceOf[@R="uid:bgGJLLXuUkvLyV9X1LVTr6RV"]][cLeftOperand/Ident[@Name="isExternal"]] |
		u2:cStereotypeInstance/u2:InstanceExpr/u2:cExpression/u2:BinaryExpr[../../u2:rInstanceOf[@R="uid:bgGJLLXuUkvLyV9X1LVTr6RV"]][u2:cLeftOperand/u2:Ident[@Name="isExternal"]] |
		cStereotypeInstance/InstanceExpr/cExpression/BinaryExpr[../../rInstanceOf[@R="ref:#rbgGJLLXuUkvLyV9X1LVTr6RV#&apos;C Application&apos;#"]][cLeftOperand/Ident[@Name="isExternal"]] |
		u2:cStereotypeInstance/u2:InstanceExpr/u2:cExpression/u2:BinaryExpr[../../u2:rInstanceOf[@R="ref:#rbgGJLLXuUkvLyV9X1LVTr6RV#&apos;C Application&apos;#"]][u2:cLeftOperand/u2:Ident[@Name="isExternal"]]'>
</xsl:template>

<!-- Changes in syntax - TargetCode expression is no longer Primary -->
<xsl:template match = 'IndexExpr/cExpr/InformalExpr | u2:IndexExpr/u2:cExpr/u2:InformalExpr |
				FieldExpr/cExpression/InformalExpr | u2:FieldExpr/u2:cExpression/u2:InformalExpr |
				CallExpr/cTo/InformalExpr | u2:CallExpr/u2:cTo/u2:InformalExpr |
				TemplateInstantiation/cActualTemplateParameter/InformalExpr |
				u2:TemplateInstantiation/u2:cActualTemplateParameter/u2:InformalExpr'>
  <ParenthesisExpr>
    <cExpression>
	  <xsl:copy>
	    <xsl:apply-templates/>
        <xsl:call-template name="copy-all-attributes"/>
	  </xsl:copy>
	</cExpression>
  </ParenthesisExpr>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

</xsl:transform>
