<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 3.2 (4) to 3.3 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:u2="http://www.telelogic.com/tau"
   exclude-result-prefixes='u2'>

<!-- Change file format version number to 3.3 -->
<xsl:template match="u2:u2|u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">3.3</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- Move out Labels from ConnectorNode to ActivityImplementation -->
<xsl:template match="u2:ActivityImplementation | ActivityImplementation">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <u2:cOwnedMember>
      <xsl:for-each select="u2:cNode/u2:ConnectorNode/u2:cLabel/u2:Label | cNode/ConnectorNode/cLabel/Label">
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:for-each>
    </u2:cOwnedMember>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<xsl:template name="convert-activity-container">
  <xsl:attribute name="Guid"><xsl:value-of select="@Guid"/></xsl:attribute>
  <xsl:for-each select="*">
    <xsl:choose>
      <xsl:when test="name()='cActivity' or name()='u2:cActivity'">
        <u2:cInlineActivity>
          <xsl:for-each select="*">
            <xsl:copy>
              <xsl:call-template name="copy-all-attributes"/>
              <xsl:apply-templates/>
            </xsl:copy>
          </xsl:for-each>
        </u2:cInlineActivity>
      </xsl:when>
      <xsl:otherwise>
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
</xsl:template>

<xsl:template match="u2:ActionNode | ActionNode">
  <xsl:copy>
    <xsl:attribute name="Name"><xsl:value-of select="u2:cActivity/u2:Operation/@Name | cActivity/Operation/@Name"/></xsl:attribute>
    <xsl:call-template name="convert-activity-container"/>
  </xsl:copy>
</xsl:template>

<xsl:template match="u2:SendSignalNode | SendSignalNode">
  <xsl:copy>
    <xsl:call-template name="convert-activity-container"/>
  </xsl:copy>
</xsl:template>

<xsl:template match="u2:AcceptEventNode | AcceptEventNode">
  <xsl:copy>
    <xsl:call-template name="convert-activity-container"/>
  </xsl:copy>
</xsl:template>


<!-- ConnectorNode sastrLabel conversion -->
<xsl:template match="u2:ConnectorNode | ConnectorNode">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:attribute name="Label"><xsl:value-of select="u2:cLabel/u2:Label/@Name | cLabel/Label/@Name"/></xsl:attribute>
  </xsl:copy>
</xsl:template>

<!-- Remove scope qualifier from 'Makefile Generator Settings' -->
<xsl:template match="
        u2:InstanceExpr/u2:rInstanceOf/
            u2:Ident[@Name='Makefile Generator Settings' and u2:rDefinition/@R='uid:@TTDMakefileGen@MakefileGenSettings']/
                u2:cScopeQualifier[u2:Ident/@Name='MakefileGen' and u2:Ident/u2:rDefinition/@R='uid:@TTDMakefileGen']
      |    InstanceExpr/   rInstanceOf/
               Ident[@Name='Makefile Generator Settings' and    rDefinition/@R='uid:@TTDMakefileGen@MakefileGenSettings']/
                   cScopeQualifier[   Ident/@Name='MakefileGen' and    Ident/   rDefinition/@R='uid:@TTDMakefileGen']">
</xsl:template>


</xsl:transform>
  
