/*************************************************************
Generated for IBM Rational Tau 4.3.0.6.24425 at Wed Nov 12 03:54:30 PM CET 2014
Licensed Materials - Property of IBM 
(c) Copyright IBM Corporation 2002,2014.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
// Common definitions for Tau C++ APIs
#ifndef __TAUAPICOMMON_H
#define __TAUAPICOMMON_H

#ifdef WIN32
#ifndef _SBX_EXT_DATA
#define _SBX_EXT_DATA __declspec(dllimport)
#endif
#else
#ifndef _SBX_EXT_DATA
#define _SBX_EXT_DATA
#endif 
#endif

#ifdef _MSC_VER 
#pragma warning(push)
#pragma warning(disable: 4290)	// __declspec(nothrow)
#pragma warning(disable: 4251)	// class needs to have dll interface
#endif

// tstring - API string type
#include <string>
#ifndef TSTRING
#ifdef _UNICODE
typedef std::wstring tstring;
#else
typedef std::string tstring;
#endif
#define TSTRING
#endif

#ifndef _T
#ifdef WIN32
#define _T(x) L##x
#else
#define _T(x) x
#endif
#endif

namespace u2 // This namespace name is kept for backwards compatibility (used since long for the Tau UML2 C++ API)
{
    // Representation of a dynamic error occurring when using the Tau C++ APIs.
    // N.B. Some Unix implementations do not always support exception types with RTTI thrown over 
    //      DLL-boundaries so use a plain string for them.
#ifdef _MSC_VER
    class _SBX_EXT_DATA APIError 
    {
    public:
        APIError(const tstring& strErrorMessage);
        virtual ~APIError();
        virtual const tstring& GetMessage();

    protected:
        const tstring m_strErrorMessage;
    };
#else
    typedef const tstring::value_type* APIError;
#endif
    _SBX_EXT_DATA const tstring::value_type* GetAPIErrorMessage(APIError& e);

    _SBX_EXT_DATA void ThrowAPIError(const tstring &strMessage) throw(u2::APIError);

    // Globally unique ID (used for API interfaces)
    struct _SBX_EXT_DATA GUID 
    {
        unsigned long	Data1;
        unsigned short  Data2;
        unsigned short  Data3;
        unsigned char	Data4[8];

        // Convert the GUID to a string representation
        void ToString(std::string& s) const;

        // Assign a GUID from a string representation
        static bool FromString(const std::string&s, GUID& g);
    };

//////////////////////////////////////////////////////////////////////////

    // Common interface base, managing object lifetime using reference counting
    class _SBX_EXT_DATA IRefCounted
    {
    public:

        // Object lifetime management functions (based on reference counting).
        // Call IncRef() when starting to use a new interface, and DecRef() when an
        // interface shall no longer be used.
        // N.B. Functions returning new interfaces do IncRef() automatically on behalf of the client.
        // Not all objects use reference counting for their lifetime management. Such objects may use
        // empty implementations of these functions, and they need then not be called by clients.
        virtual unsigned long IncRef() = 0;
        virtual unsigned long DecRef() = 0;
    };

    // Common ancestor interface implemented by all C++ API interfaces
    // for which there is a need of dynamic "casting" between interfaces.
    class _SBX_EXT_DATA IUnknown : public IRefCounted
    {
    public:
        // Queries the implementing object for the interface with the specified GUID.
        // If the object does not realize that interface ppObject will be set to NULL,
        // otherwise it will be set to the requested interface.
        // In case the requested interface is available and returned to the caller, IncRef will be called on it.
        // It is the responsibility of the client to call DecRef when it no longer wants to use the interface.
        virtual void QueryInterface(const GUID& g, void** ppObject) = 0;

        // Returns an untyped pointer to the object that implements the interface.
        // These are for internal use only!!!
        virtual void* GetImplementingObject() = 0;
        virtual const void* GetImplementingObject() const = 0;

        static const GUID guid; // Unique identification of the interface.
    };

    // Convenience utility for casting between u2 interfaces. If casting is successful
    // a pointer to the requested interface is returned, otherwise NULL.
    template <class I> 
    inline I* cast(IUnknown* pUnknown) 
    {
        I* p = 0;
        if (pUnknown)
            pUnknown->QueryInterface(I::guid, (void**) &p);
        return p;
    }

    // const version of the above
    template <class I> 
    inline const I* cast(const IUnknown* pUnknown) 
    {
        I* p = 0;
        if (pUnknown)
            const_cast<IUnknown*>(pUnknown)->QueryInterface(I::guid, (void**) &p);
        return p;
    }

    // Standard implementation of IRefCounted
    template <class T>
    class IRefCountedImpl : public T 
    {
    public:
        IRefCountedImpl() : refCount(0)
        {}

        virtual ~IRefCountedImpl() {}

        virtual unsigned long IncRef() 
        {
            refCount++;
            return refCount;
        }

        virtual unsigned long DecRef()  
        {	    
            refCount--;
            if (refCount == 0)
                delete this;
            return refCount;
        }

    protected:
        unsigned long refCount;
    };

    // Empty implementation of IRefCounted to be used when no reference counting is necessary
    template <class T>
    class IRefCountedImpl_NoRefCount : public T 
    {
    public:
        virtual unsigned long IncRef()  
        { 
            return 0;
        }

        virtual unsigned long DecRef() 
        {	    
            return 0;
        }
    };    
    
    // Smart pointer for accessing an IUnknown-derived interface without explicit calls to IncRef()/DecRef()
    template <class Type>
    class Ptr
    {
    public:
        // Construct from IUnknown, by doing u2::cast to Type. Increments the reference count.
        explicit Ptr(const IUnknown* pUnk = 0) 
        {
            m_pObj = const_cast<Type*>(u2::cast<Type>(pUnk));
        }

        // Construct from Type pointer. If bAttach is true, the ownership of the pointer is taken without incrementing the reference count.
        Ptr(const Type* pT, bool bAttach)
        {
            m_pObj = const_cast<Type*>(pT);
            if (!bAttach)
                m_pObj->IncRef();
        }

        Ptr(const Ptr<Type>& other) 
        {
            m_pObj = other.m_pObj;
            if (m_pObj) 
                m_pObj->IncRef();
        }

        ~Ptr() 
        {
            if (m_pObj) 
                m_pObj->DecRef();
        }

        Ptr<Type>& operator=(const Ptr<Type>& other) 
        {
            if (this == &other)
                return *this; // Self-assignment

            if (m_pObj) 
                m_pObj->DecRef();
            m_pObj = other.m_pObj;
            if (m_pObj) 
                m_pObj->IncRef();

            return *this;
        }

        bool operator==(const Ptr<Type>& rhs) const
        {
            // Check identity by casting to IUnknown.
            bool bSame = (cast<IUnknown>(*this) == cast<IUnknown>(&rhs));
            if (m_pObj)
	      m_pObj->DecRef();
            rhs.DecRef();

            return bSame;
        }

        Type* get() const 
        {
            return m_pObj; 
        }

        Type* operator->() const 
        { 
            return m_pObj; 
        }

        Type& operator*() const
        {
            return *m_pObj;
        }

        bool operator!() const
        {
            return m_pObj == 0;
        }

        operator bool() const
        {
            return m_pObj != 0;
        }

    private:
        Type* m_pObj;
    };

}

// Comparison operator for u2::GUID structs
_SBX_EXT_DATA bool operator==(const u2::GUID& g1, const u2::GUID& g2);

#endif // __TAUAPICOMMON_H
